function obj = clean(obj,xfunc,minmaxfreqs,varargin)
% Subtract coherent data/noise between channels
% Usage: cleanobj = clean(obj,xfunc,minmaxfreqs,...);
% Returns an ts_data with the effect of the "driving" channels
%	subtracted from the "response" channels
%
% Input:
%	obj  ts_data object
%	xfunc     ts_xferfunc object containing transfer functions between "driving"
%                and "response" channels (transfer functs will be processed in order)
%				 Can be calculated using function CLEAN_CALC
%   minmaxfreqs [fmin fmax] frequency band to apply transfer function in [default=all]
%	(field,value) pairs as desired:
%		'method': ('timedomain',['frequencydomain'])
% Output:
%   cleanout     ts_data object
%
% See also ts_data/CLEAN_CALC

% 10/2002 Wayne Crawford

doInTimeDomain=0;
if nargout ==0, error('Must provide an output argument'); end

ts_verbose(1,'Subtracting correlated data from ts_data using transfer functions');


freqs = xfunc.freqs;
numxfuncs = xfunc.nChannels;

if exist('minmaxfreqs','var'),
	gooddata = freqs>minmaxfreqs(1) & freqs <= minmaxfreqs(2);
else
	gooddata = ones(size(freqs));
end

for i=1:numxfuncs,
	drivechan = xfunc.channel(i).drivechan;  respchan =  xfunc.channel(i).respchan;
	ts_verbose(1,'Subtracting channel %d effect from channel %d',drivechan,respchan);
	drivefilt = obj.channel(drivechan).metadata.response;
	respfilt =  obj.channel(respchan).metadata.response;
	% Need the following to relate uncorrected time data to corrected xfer function
	driveresp = response(drivefilt,freqs);
	respresp =  response(respfilt, freqs);
	
	% Calculate the convolution operator to subtract from the "drive" from the "response" channel
	xfs = xfunc.channel(i).data .* (respresp./driveresp).*gooddata.*xfunc.channel(i).gooddata;
	xfreqs = xfunc.freqs;
	
	drive = obj.channel(drivechan).data;
	resp0 = obj.channel(respchan).data;
	% Added 02/21/2006: Subtract mean and trend before convolving
	x=1:length(drive);
	p=polyfit(x,drive,1); drive=drive-polyval(p,x);
	p=polyfit(x,resp0,1); resp0=resp0-polyval(p,x);

	if doInTimeDomain,
        if xfreqs(2)-xfreqs(1) ~= 1/obj.channel(drivechan).samprate,
            error('transfer function delta freq (%g Hz) doesn''t correspond to drive channel sample rate (%g sps)', ...
                xfreqs(2)-xfreqs(1), obj.channel(drivechan).samprate);
        elseif xfreqs(2)-xfreqs(1) ~= 1/obj.channel(respchan).samprate,
            error('transfer function delta freq (%g Hz) doesn''t correspond to response channel sample rate (%g sps)', ...
                xfreqs(2)-xfreqs(1), obj.channel(respchan).samprate);
        end
        
        xf2side = [0 xfs fliplr(conj(xfs(1:end-1)))];
	
		% Added 02/23/2006: Apply a taper before ifft
		if 0,	% Doesn't work ...
			w = prol4pi(length(xf2side));
			xf2side = 2*pi*w.*xf2side;
		end
	
		ifilter = (fftshift(ifft(xf2side)));	% Time-domain equivalent of the transfer function
		filtlen = length(ifilter);	
	
		% Subtract driving channel convoluted by "transfer function" from
		% response channel
		datalen = length(drive);
		offset = filtlen/2; iadds = offset:datalen+offset-1;
		correction = conv(drive,ifilter);		% Result is length(drive0)+length(ifilter)-1
		correction = correction(iadds)-mean(correction(iadds));	% Take out mean values
		resp1 = resp0 - correction;
    else	% Do in freq domain, can handle different samp rates
		datalen = length(drive);
		% round up to higher power of 2 (by zero padding)
		dataPower = ceil(log2(datalen));fftlen = 2^dataPower;
		driveF = fft([drive zeros(1,fftlen-datalen)]);
		resp0F = fft([resp0 zeros(1,fftlen-datalen)]);

		driveF = driveF(2:fftlen/2+1);	% Since drive is real, can just grab the pos freqs;
		resp0F = resp0F(2:fftlen/2+1);	% Since data is real-valued, can just grab the pos freqs;
        samprate = obj.channel(drivechan).samprate;
        if samprate~= obj.channel(respchan).samprate,
            error('Can''t apply transfer function between two channels with different sampling rate');
        end
		freqstep = samprate/fftlen; driveFreqs = freqstep*(1:fftlen/2);
		xf = interp1([0 xfreqs],[xfs(1) xfs],driveFreqs);

		resp1F = resp0F - driveF.*xf;
		resp1F2side = [0 resp1F fliplr(conj(resp1F(1:end-1)))];
			
		resp1 = ifft(resp1F2side);	% Time-domain equivalent of the transfer function
		% rounded up, cut
		resp1 = resp1(1:datalen);

	end
	obj.channel(respchan).data = real(resp1);
	obj.channel(respchan).metadata.component(2)='C';
    obj.channel(respchan).metadata.information=[obj.channel(respchan).metadata.information '-[' obj.channel(drivechan).metadata.information ']'];
end
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function w = prol4pi(n)

fOnE = ones(1,n);
x = (2*(1:n)-fOnE)/n;
u = (2*fOnE-x) .*(x);
	
v =		(((((2.6197747176990866e-11*u + 2.9812025862125737e-10*fOnE) .*u + ...
				3.0793023552299688e-9*fOnE) .*u + 2.8727486379692354e-8*fOnE) .*u + ...
				2.4073904863499725e-7*fOnE) .*u + 1.8011359410323110e-6*fOnE) .*u;
w =		((((((v+1.1948784162527709e-5*fOnE) .*u+6.9746276641509466e-5*fOnE) .*u+...
				3.5507361197109845e-4*fOnE) .*u + 1.5607376779150113e-3*fOnE) .*u + ...
				5.8542015072142441e-3*fOnE) .*u + 1.8482388295519675e-2*fOnE) .*u;
v =		((((((w+4.8315671140720506e-2*fOnE) .*u+1.0252816895203814e-1*fOnE) .*u+...
				1.7233583271499150e-1*fOnE) .*u + 2.2242525852102708e-1*fOnE) .*u + ...
				2.1163435697968192e-1*fOnE) .*u + 1.4041394473085307e-1*fOnE) .*u;
w =		((((v+5.9923940532892353e-2*fOnE) .*u+1.4476509897632850e-2*fOnE) .*u+ ...
				1.5672417352380246e-3*fOnE) .*u + 4.2904633140034110e-5*fOnE);
w = w*sqrt(2./.508125548147497);

