function xdata = clean_calc(obj,chanlist,windowlen,outlratio,varargin)
% Calculate the transfer functions needed to clean coherent noise
%
% Usage: clean_xf = clean_calc(obj,chanlist,windowlen,outlierRatio)
%
% Inputs:
%	obj	 is an ts_data object
%	chanlist is a list of channels to remove from the remaining channels,
%				in the order you want them removed
%	windowlen is the length in seconds of each FFT window (will be
%				automatically increased to a number of samples equal to
%				the next highest power of 2, if necessary) [2048*ch1
%				samprate]
%	outlierRatio is used to reject time windows with spurious energy
%				any spectra with energy outlierratio times the minimum
%				energy will be rejected.  If you don't specify this or set
%               to 0, no outlier checking will be done
%   varargin    field,value pairs to be passed on directly to 
%               ts_data.calcSpectra()
%
% Output
%	clean_xf is the cleaning transfer function.  It can be input to ts_spectra
%			as the value for the field 'subtractxf', or as the input to
%			ts_data.clean(...)
%
% See also ts_data.clean
debug=0;

if ~exist('windowlen','var'); windowlen=2048/obj.channel(1).samprate; end
if ~exist('outlratio','var'); outlratio=0; end

taper='prol4pi';
goodFreqMax=0.08;
linecolororder='rgbkcmy';

ts_verbose(1,'Calculating data cleaner transfer functions');

nc = obj.nChannels; iR = 1:nc;

for i = 1:length(chanlist),
    iD = chanlist(i);
    iR = iR(iR~=iD);
    for j=1:length(iR),
        xfPairs{i}{j}=[iD iR(j)];
    end
end

if isnumeric(gcf)
    ifig=gcf+1;
else
    ifig=get(gcf,'Number'); % Matlab 2014b and later
end
for i=ifig:ifig+nc-1,
    figure(i); clf
end
kfig=ifig+nc;
% CORRECT FOR COHERENT "NOISE"
for i = 1:length(chanlist)+1,
    if debug, disp('A'); end
    if i>1,
        driveChan=chanlist(i-1);
    end
    colorspec = linecolororder(rem(i,length(linecolororder))+1);
    if i == 1,
        if exist('varargin','var'),
            [sdata,~,offsetlist] = obj.calcSpectra('windowlen',windowlen, ...
                'taper',taper,'removeoutliers',outlratio,varargin{:});
        else
            [sdata,~,offsetlist] = obj.calcSpectra('windowlen',windowlen, ...
                'taper',taper,'removeoutliers',outlratio);
        end
        origsdata = sdata;
    else
        if exist('varargin','var'),
            sdata = obj.calcSpectra('windowlen',windowlen, 'taper',taper, ...
                'offsetlist',offsetlist,'subtractxf',xdata,varargin{:});
        else
            sdata = obj.calcSpectra('windowlen',windowlen, 'taper',taper, ...
                'offsetlist',offsetlist,'subtractxf',xdata);
        end
    end
    if debug, disp('B'); end
    % Plot spectra at each stage
    if i==1, lgnd{i}='Orig';
    else
        lgnd{i}=sprintf('%s-%s',lgnd{i-1},strtrim(obj.channel(driveChan).metadata.fullname));
    end
    for j=1:nc,
        figure(j+ifig-1);
        if i==1 | ~any(j==chanlist(1:i-1)) % Only plot a channel if first time (orig) or was corrected this time
            h{j}(i) = loglog(sdata.freqs,sdata.spectdata(j,:),colorspec);
        end
        if i==1,
            hold on;
            title(sprintf('%s',sdata.sinfo(j).name));
            xlabel('Frequency (Hz)');
            ylabel(sprintf('PSD (%s)',sdata.sinfo(j).units));
        end
    end
    figure(kfig); plotcoher(sdata);
    title(obj.channel(1).metadata.station); drawnow %,'interpreter','none'); drawnow;
    kfig=kfig+1;
    if debug, disp('C'); end
    
    if i>length(xfPairs), break, end
    
    % CALCULATE TRANSFER FUNCTIONS
    for j=1:length(xfPairs{i}),	% Subtract effect on other source channels
        idrive = xfPairs{i}{j}(1);
        iresp =  xfPairs{i}{j}(2);
        
        ts_verbose(1,'Phase %d: Calc xfunc between %d(%s) and %d(%s)', ...
            i,idrive,sdata.sinfo(idrive).name, ... % get(sdata,'chname',idrive), ...
            iresp, sdata.sinfo(iresp).name);  % get(sdata,'chname',iresp));
        
        temp = sdata.calcXF(idrive,iresp); % ts_xferfun(sdata,idrive,iresp);
        % FORCE TRANSFER FUNCTION APPLICATION AT COMPLIANCE FREQS
        temp.gooddata = (temp.freqs < goodFreqMax);
        if ((i==1)&&(j==1)),
            xdata = temp;
        else xdata = xdata+temp;
%        else xdata = merge(xdata,temp);
        end
    end
    if debug, disp('D'); end
end
figure(kfig); plot(xdata); drawnow;

for i=ifig:ifig+nc-1,
    figure(i); axis tight,
    set(gca,'YGrid','on'); set(gca,'YMinorGrid','off');
    set(gca,'XGrid','on'); set(gca,'XMinorGrid','off');
    legend(lgnd);
end
