function retval = clean_rotate(obj)
    % Rotate seismometer channels to minimize tilt noise on the vertical
    %
    % WARNING: EXPERIMENTAL!!!!!
    % Does not remove the cos(dynRot) rotation effect on the vertical, merely
    % rotates the channels to remove the sin(staticRot)*sin(dynRot) component.
    %
    % Assumes that channels are named Z12, ZNE or ZYX and a right-handed
    % coordinate system (GSN standard)
    % Estimates the rotation error by calculating correlation between the
    % channels filtered from 0.001 to 0.005 Hz (strongest tilt effect and
    % generally weaker compliance frequencies).  Calculates the variance of the
    % new filteredZ, then searches for rotations in the neighborhood that
    % minimize this variance.  Finally, returns the rotated, unfiltered
    % channels (both Z and horizontal channels will be changed, though
    % generally the effect is barely noticeable on the horizontals)
    %
    % Usage: tdata = clean_calc(obj)
    %
    % Inputs:
    %	obj	 is an ts_data object
    %
    % Output
    %	tdata is the same as before, except the seismometer channels are
    %	rotated to minimize noise on the "Z" channel
    %
    % See also ts_data.clean
    
    retval=[];
    
    %[iZ,iN,iE]=
    iZ=0;iN=0;iE=0;
    for i = 1:obj.nChannels,
        cmp=obj.channel(i).md.component(3);
        if cmp=='Z',
            iZ=i;
        elseif ~isempty(strfind('1NY',cmp))
            iN=i;
        elseif  ~isempty(strfind('2EX',cmp))
            iE=i;
        end
    end
    if iZ==0,
        fprintf('no Z channel found!');
    end
    if iN==0,
        fprintf('no 1,N, or Y channel found!');
    end
    if iE==0,
        fprintf('no 2,E, or X channel found!');
    end
    if any([iZ iN iE]==0),
        error('Quitting');
    end
    
    lowfreq=0.001; highfreq=0.005;
    fdata=obj.filter('BwPass',2,0.001,0.005);
    fdata=fdata.cut(ceil(fdata.channel(1).samprate/lowfreq),0);    % Get rid of edge affected data
    fdata=fdata.detrend();
    
    Z=fdata.channel(iZ).data;
    N=fdata.channel(iN).data;
    E=fdata.channel(iE).data;    
    [ZNangle,ZEangle]=estimateAngles(Z,N,E);
    
    figure(2);
    plot([ZNangle,ZEangle]*180/pi,'o');
    set(gca,'XLim',[0.5 2.5],'XTick',[1 2],'XTickLabels',{'ZN','ZE'});
    xlabel('Channel')
    ylabel('estimated static Z offset angle(degrees)')
    
    % Use corrcoeff to determine how significant the correlation is 
    [R,~]=corrcoef([(Z.^2)' (N.^2)' (E.^2)']);
    
    % Find the best rotation angles
    bestAngles = rotate_1D(R,ZNangle,ZEangle,fdata,Z,N,E,iZ,iN,iE);  % 1D method, find first rotation, then second
    %bestAngles = rotate_2D(R,ZNangle,ZEangle,fdata,Z,N,E,iZ,iN,iE);  % 2D method, solve for first and second rotations at once
    fprintf('          |   ZN rot  |    ZE rot  | variance\n');
    fprintf('without   |    %6.2f |    %6.2f  | %6g\n',       0,       0,rotatedZ_variance2(N,E,Z,[0,0]));
    fprintf('first est |    %6.2f |    %6.2f  | %6g\n',-ZNangle*180/pi,-ZEangle*180/pi,rotatedZ_variance2(N,E,Z,[-ZNangle,-ZEangle]));
    fprintf('final est |    %6.2f |    %6.2f  | %6g\n',  bestAngles(1)*180/pi,  bestAngles(2)*180/pi,rotatedZ_variance2(N,E,Z,bestAngles));
end

function [ZNangle,ZEangle]=estimateAngles(Z,N,E)
    ZoverN= Z ./ N;
    ZoverE= Z ./ E;
    %NoverE= N ./ E;
    
    % sqrt(median(ZoverN.^2)) gives a more reliable transfer ratio,
    % but need median(ZoverN) for the sign
    ZNratio=sqrt(median(ZoverN.^2))*sign(median(ZoverN));
    ZEratio=sqrt(median(ZoverE.^2))*sign(median(ZoverE));
    %NEratio=sqrt(median(NoverE.^2))*sign(median(NoverE));
    
    ZNangle=asin(ZNratio);
    ZEangle=asin(ZEratio);
end

function [Xnew,Ynew]=rotateData(X,Y,angle)    % always put in X/Y order
    % Rotates X & Y by angle radians
    rotMatrix=[cos(angle) -sin(angle); sin(angle) cos(angle)];
    new=rotMatrix*[X;Y];
    Xnew=new(1,:);
    Ynew=new(2,:);
end

function [Nnew,Enew,Znew]=rotateData2(N,E,Z,angle)    % always put in X/Y order
    % Rotates X & Z by angle(1) radians, then Y and Z by angle(2) radians
    rotMatrix=[cos(angle(1)) -sin(angle(1)); sin(angle(1)) cos(angle(1))];
    new=rotMatrix*[N;Z];
    Nnew=new(1,:);
    Znew=new(2,:);
    
    rotMatrix=[cos(angle(2)) -sin(angle(2)); sin(angle(2)) cos(angle(2))];
    new=rotMatrix*[E;Znew];
    Enew=new(1,:);
    Znew=new(2,:);
end

function variance=rotatedZ_variance2(N,E,Z,angle2)   
    [~,~,Znew]=rotateData2(N,E,Z,angle2);
    variance=var(Znew);
end

function variance=rotatedZ_variance(X,Z,angle)   
    [~,Znew]=rotateData(X,Z,angle);
    variance=var(Znew);
end

function bestAngles = rotate_1D(R,ZNangle,ZEangle,fdata,Z,N,E,iZ,iN,iE)
    if R(1,2) > R(1,3)  % Z better correlated with N than with E
        startZNangle=-ZNangle;
        [Nnew,Znew]=rotateData(N,Z,startZNangle);    % always put in X/Y order
        newf=fdata;
        newf.channel(iN).data=Nnew;
        newf.channel(iZ).data=Znew;
        figure(3);
        plot(newf);
        % use angle=fminbd(fun,0,2*-ZNangle) to find the best angle
        bestZNangle=fminsearch(@(x) rotatedZ_variance(N,Z,x),startZNangle,optimset('display','iter'));
        [Nnew,Znew]=rotateData(N,Z,bestZNangle);    % always put in X/Y order
        newerf=fdata;
        newerf.channel(iN).data=Nnew;
        newerf.channel(iZ).data=Znew;
        figure(4);
        plot(newerf);
        
        Z=newerf.channel(iZ).data;
        N=newerf.channel(iN).data;
        E=newerf.channel(iE).data;    
        [ZNangle,ZEangle]=estimateAngles(Z,N,E);

        figure(5);
        plot([ZNangle,ZEangle]*180/pi,'o');
        set(gca,'XLim',[0.5 2.5],'XTick',[1 2],'XTickLabels',{'ZN','ZE'});
        xlabel('Channel')
        ylabel('estimated static Z offset angle(degrees)')

        % Use corrcoeff to determine how significant the correlation is 
        [R,~]=corrcoef([(Z.^2)' (N.^2)' (E.^2)']);
        startZEangle=-ZEangle;
        [Enew,Znew]=rotateData(E,Z,startZEangle);    % always put in X/Y order
        newf=newerf;
        newf.channel(iE).data=Enew;
        newf.channel(iZ).data=Znew;
        figure(6);
        plot(newf);
        
        bestZEangle=fminsearch(@(x) rotatedZ_variance(E,Z,x),startZEangle,optimset('display','iter'));
        [Enew,Znew]=rotateData(E,Z,bestZEangle);    % always put in X/Y order
        newestf=fdata;
        newestf.channel(iE).data=Enew;
        newestf.channel(iZ).data=Znew;
        figure(7);
        plot(newestf);
    else
        startZEangle=-ZEangle;
        [Enew,Znew]=rotateData(E,Z,startZEangle);    % always put in X/Y order
        newf=fdata;
        newf.channel(iE).data=Enew;
        newf.channel(iZ).data=Znew;
        figure(3);
        plot(newf);
        
        bestZEangle=fminsearch(@(x) rotatedZ_variance(E,Z,x),startZEangle,optimset('display','iter'));
        [Enew,Znew]=rotateData(E,Z,bestZEangle);    % always put in X/Y order
        newerf=fdata;
        newerf.channel(iE).data=Enew;
        newerf.channel(iZ).data=Znew;
        figure(4);
        plot(newerf);
        
        Z=newerf.channel(iZ).data;
        N=newerf.channel(iN).data;
        E=newerf.channel(iE).data;    
        [ZNangle,ZEangle]=estimateAngles(Z,N,E);

        figure(5);
        plot([ZNangle,ZEangle]*180/pi,'o');
        set(gca,'XLim',[0.5 2.5],'XTick',[1 2],'XTickLabels',{'ZN','ZE'});
        xlabel('Channel')
        ylabel('estimated static Z offset angle(degrees)')

        % Use corrcoeff to determine how significant the correlation is 
        [R,~]=corrcoef([(Z.^2)' (N.^2)' (E.^2)']);

        startZNangle=-ZNangle;
        [Nnew,Znew]=rotateData(N,Z,startZNangle);    % always put in X/Y order
        newf=newerf;
        newf.channel(iN).data=Nnew;
        newf.channel(iZ).data=Znew;
        figure(6);
        plot(newf);
        
        bestZNangle=fminsearch(@(x) rotatedZ_variance(N,Z,x),startZNangle,optimset('display','iter'));
        [Nnew,Znew]=rotateData(N,Z,bestZNangle);    % always put in X/Y order
        newestf=fdata;
        newestf.channel(iN).data=Nnew;
        newestf.channel(iZ).data=Znew;
        figure(7);
        plot(newestf);
    
    end
    bestAngles=[bestZNangle,bestZEangle];
end

function bestAngles= rotate_2D(R,ZNangle,ZEangle,fdata,Z,N,E,iZ,iN,iE)
    % Solve for two rotation angles at once.  Should solve for
    % rotation aboutand tilt around Z axis, but for now just tilts on the
    % farthest off X- or Y- axis, then the other one
    startZNangle=-ZNangle;
    startZEangle=-ZEangle;
    bestAngles=fminsearch(@(x) rotatedZ_variance2(N,E,Z,x),[startZNangle,startZEangle],optimset('display','iter'));
    [Nnew,Enew,Znew]=rotateData2(N,E,Z,bestAngles);
    newf=fdata;
    newf.channel(iN).data=Nnew;
    newf.channel(iE).data=Enew;
    newf.channel(iZ).data=Znew;
    
    figure(7);
    plot(newf);
    
    figure(6);
    plot([startZNangle,startZEangle]*180/pi,'o');
    hold on
    plot(bestAngles*180/pi,'x');
    set(gca,'XLim',[0.5 2.5],'XTick',[1 2],'XTickLabels',{'ZN','ZE'});
    xlabel('Channel')
    ylabel('estimated static Z offset angle(degrees)')
    
end


