function obj = synchronize(obj,newsamprate)
% Make all channels have same start time, length and sample rate
%    objout = synchronize(obj [,newsamprate])
% Inputs [default]:
%  obj              An ts_data object
%  newsamprate        Desired sampling rate.  Options are:
%                       'low'   The lowest rate of the channels [DEFAULT]
%                       'high'  The highest rate of the channels
%                       samprate The given rate (sps)
% Outputs:
%	objout		Synchronized data (ts_data object)
%
% Examples:
%      outdata = synchronize(obj);          % Synchronize at lowest sampling rate
%      outdata = synchronize(obj,'high');   % Synchronize at highest sampling rate
%      outobj=synchronize(obj,10);          % Synchronize at 10 sps
%											
%

% W Crawford 01/2011

if nargout ==0, error('Must provide an output argument'); end
if ~exist('newsamprate','var'); newsamprate='low'; end
if obj.nChannels ==1,
    ts_warning('no need to synchronize, obj has only 1 channel!');
    return;
elseif obj.nChannels==0,
    ts_warning('obj is empty!')
    return;
end

% Calculate new sampling rate
if ischar(newsamprate)
    switch newsamprate,
        case 'high',
            newsamprate=obj.channel(1).samprate;
            for i=2:obj.nChannels,
                if obj.channel(1).samprate > newsamprate
                    newsamprate=obj.channel(1).samprate;
                end
            end
         case 'low',
            newsamprate=obj.channel(1).samprate;
            for i=2:obj.nChannels,
                if obj.channel(1).samprate < newsamprate
                    newsamprate=obj.channel(1).samprate;
                end
            end
        otherwise
            error('Unknown samprate argument: "%s"',newsamprate);
    end
end

% Find data start (latest starttime)
refstart=obj.channel(1).starttime;
for i=2:obj.nChannels
    if obj.channel(i).starttime > refstart
        refstart=obj.channel(i).starttime;
    end
end
% Find data end (earliest endtime)
refend=obj.channel(1).endtime;
for i=2:obj.nChannels
    if obj.channel(1).endtime < refend
        refend=obj.channel(1).endtime;
    end
end

% Check to see if there will be any data!
if refend-refstart < 0,
    error('Not all data overlap');
elseif refend-refstart < 2/newsamprate,
    error('ridiculously small overlap');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% WCC STOPPED HERE!!!!!!
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cut data
ts_verbose(1,'Cutting data from %s to %s',char(refstart),char(refend));
obj=obj.cut(refstart,refend);

% Resample data
for i=1:obj.nChannels,
    sr=obj.channel(i).samprate;
    if sr ~= newsamprate,
        integerdivide= ~rem(sr,newsamprate);
        if integerdivide,       % If possible, average over integer # of samples
            obj.channel(i)=resample(obj.channel(i),sprintf('/%d',sr / newsamprate));
        else
            obj.channel(i)=resample(obj.channel(i),newsamprate);
        end
    end
    obj.channel(i)=cut(obj.channel(i),refstart,refend);   % Could give 1 sample shorter or longer
end

nS_ref=obj.channel(1).nSamples;
for i=2:obj.nChannels,
    nS=obj.channel(i).nSamples;
    if nS ~= nS_ref,
        ts_verbose(1,'Adjusting length of channels to match each other');
        diff=nS-nS_ref;
        if abs(diff) > 1, 
            ts_warning('Synchronized channels were off by more than one sample'); 
        end
        if nS > nS_ref,    % Cut down new channel
            obj.channel(i).data=obj.channel(i).data(1:nS_ref);
        else
            nS_ref=nS;
            for j=1:i-1,
                obj.channel(j).data=obj.channel(j).data(1:nS_ref);
            end
        end
    end
end
            
