classdef ts_data
    % PRIMARY TISKIT CLASS: Handle multiple time series
    %      Contains multiple ts_data_channel objects and a
    %               ts_data_figureproperties object
    %
    % ts_data properties:
    %    channel     - vector of data channels                   (ts_data_channel vector)
    %                 can also be accessed using "ch"
    %    figureprops - how to plot figures                       (ts_data_figureproperties)
    %    nChannels   - READ ONLY: how many elements in "channel" (scalar)
    %
    % ts_data methods:
    %   Creator:
    %      ts_data       - Creator
    %   Modify data:
    %      demean        - apply ts_data_channel.detrend() to each channel
    %      detrend       - apply ts_data_channel.detrend() to each channel
    %      filter        - apply ts_data_channel.filter() to each channel
    %      resample      - apply ts_data_channel.resample() to each channel
    %      trueAmplitude - apply ts_data_channel.trueAmplitude() to each channel
    %   Information
    %      isempty        - Returns 1 is there is no data
    %      issynchronized - Returns 1 if all channels start at the same time,
    %                       have the same sample rate and the same length
    %   Transform object(s)
    %      cat            - Concatenate two or more ts_data objects (end to end)
    %                       All channels must be consecutive and have
    %                       the same sampling rate same.  Keeps metadata from
    %                       the first channel
    %      cut            - cut each channel, using ts_data_channel.cut()
    %      extract        - return a ts_data channel with only the requested
    %                       channels
    %      synchronize    - make all channels start and end at same time
    %      stack          - stack channels
    %   Write data to file:
    %      writeMSEED     - write to miniSEED file
    %      writeSAC       - write to SAC file(s)
    %      writeSEGY      - write to SEGY file
    %      writeSEISAN    - write to SEISAN file
    %   Redefine Matlab methods
    %      plus           - Adds two ts_data objects together.  Concatenates
    %                       channels if they are consecutive and have the same
    %                       metadata, puts in a separate channel otherwise
    %   Other
    %      setstarttimes  - set the starttime of all channels to the given value
    %      calcSpectra    - calculate power spectral density and coherence
    %      clean_calc     - calculate transfer function to remove correlated noise
    %      clean          - remove correlated noise from one or more channels
    %
    % EXAMPLES:
    % sps=40; t=0:1/sps:100; d=[sin(t/5);cos(t/6)];
    % d=ts_data(d,sps);
    % d.channel(1).md.component='sin';
    % d.channel(2).md.component='cos';
    % d
    % plot(d);
    
    %% PUBLIC PROPERTIES
    properties
        channel=ts_data_channel();	% Array of ts_data_channel objects
        figureprops=ts_data_figureproperties();   % Information about how to plot time figures
    end
    %% DEPENDENT PROPERTIES
    properties (Dependent=true)
        nChannels;    % Number of channels of data
        ch;           % Alias for "channel" property
    end
    
    %% PUBLIC METHODS
    methods
        %% CREATOR
        function obj = ts_data(data,samprate,dfile)
            %ts_data Create an ts_data object (time series data and parameters)
            %
            % Usage:
            %	tdata = ts_data(data,samprate);
            % 	tdata = ts_data(data,samprate,datafile);
            % Inputs:
            %     data      The data matrix: each row is a separate channel
            %               -or-    a ts_data_channel object
            %     samprate  The sampling rate (samps/sec).
            %     datafile  An ts_file object containing instrument information.
            %               The number of 'readchannels' in this object must equal
            %               the number of rows in the data matrix
            %
            % Example
            %      tdata = ts_data([1:200;sin([1:200]/20),10);
            %
            % ts_data objects are most commonly created using
            %           ts_file.read()
            
            % W Crawford 10/99. Upgraded to OO 01/2011
            
            ts_init;	% Set up global variables
            ts_debug(1,'Starting...');
            if nargin==0; return; end    % Handle zero-argument case
            
            % First, initialize a generic ts_data class.
            if exist('data','var'),
                ts_debug(1,'data matrix exists');
                if isa(data,'ts_data_channel');
                    if length(data) > 1,
                        error('can''t put in a matrix of ts_data_channels (yet!)');
                    end
                    obj.channel=data;
                    return
                elseif ~isnumeric(data)
                    error('data matrix is neither ts_data_channel nor numeric');
                end
                [M,N] = size(data);
                if (M>N)    % channels are in columns, transpose matrix
                    ts_warning('Your data matrix has more rows than columns, you max need to transpose');
                    %                     ts_verbose(0,'There are more rows than columns, transposing data matrix');
                    %                     data=data';
                    %                     [M,N]=size(data);
                end
                if ~exist('samprate','var');
                    samprate=1;
                    ts_warning('No sampling rate specified, assuming 1 sps');
                elseif ~isnumeric(samprate) || length(samprate) > 1
                    error('2nd argument must be the sampling rate (scalar numeric)')
                end
                if ~exist('dfile','var'),	% If no datafile is specified
                    dfile=ts_file('','');
                    dfile.nChannels=M;
                    dfile.readchans=1:M;
                elseif ~isa(dfile,'ts_file'),
                    error('3rd argument is not a ts_file object');
                end
                % stuff data into channels
                chfile=dfile;
                for i=1:M,
                    chfile.readchans=dfile.readchans(i);
                    obj.channel(i)=ts_data_channel(data(i,:),chfile);
                    obj.channel(i).samprate=samprate;
                end
            end
            %obj
            ts_debug(1,'Ending...');
        end % Creator
        
        %% SET METHODS (ELIMINATE BAD INPUTS)
        function obj=set.channel(obj,value)
            if ~isa(value,'ts_data_channel'), error('value must be a ts_data_channel object'); end
            obj.channel=value;
        end
        function obj=set.figureprops(obj,value)
            if ~isa(value,'ts_data_figureproperties'), error('value must be a ts_data_figureproperties object'); end
            obj.figureprops=value;
        end
        
        %% ACCESS METHODS FOR DEPENDENT VARIABLES
        function out=get.nChannels(obj)
            out=length(obj.channel);
        end
        function obj=set.nChannels(obj,value)
            if ~isnumeric(value), error('non-numeric value'); end
            if value < 0, error('value less than 0'); end
            value=round(value);
            numchans=obj.nChannels;
            if value>numchans,
                for i=numchans+1:value,
                    obj.channel(i)=ts_data_channel();
                end
            elseif value < numchans,
                for i=numchans:-1:value,
                    obj.channel(i)=[];
                end
            end
        end
        function out=get.ch(obj)
            out=obj.channel;
        end
        function obj=set.ch(obj,value)
            obj.channel=value;
        end
        
        %% DISPLAY AND CHAR METHODS
        function txt=char(obj,padding)
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            
            txt=     sprintf('%s%12s: %g\n',pad,'nChannels',obj.nChannels);
            txt=[txt sprintf('%s%12s: %s object\n',pad,'figureprops',class(obj.figureprops))];
            txt=[txt sprintf('%s%12s: %d-element %s object\n',pad,'channel',obj.nChannels,class(obj.channel))];
            txt=[txt sprintf('%s%12s %20s %25s %10s %10s\n',pad,'#','fullname','starttime','samprate','nSamples')];
             for i=1:obj.nChannels,
                 txt=[txt sprintf('%s%12d %20s %25s %10g %10d\n',pad,i, ...
                     obj.channel(i).metadata.fullname, ...
                     obj.channel(i).starttime.char(), ...
                     obj.channel(i).samprate, ...
                     obj.channel(i).nSamples  )];
             end
%                 txt=[txt sprintf('%s\n',char(obj.channel(i),12+padding))];
%             end
%             txt=[txt sprintf('%s%12s: %d-element vector of %s objects\n',pad,'channel',obj.nChannels,class(obj.channel))];
%             for i=1:obj.nChannels,
%                 txt=[txt sprintf('%s  %11s%d:\n',pad,'CHANNEL',i)];
%                 txt=[txt sprintf('%s\n',char(obj.channel(i),12+padding))];
%             end
        end
        function disp(obj)
            disp(char(obj));
        end
        
        %%  DATA PROCESSING METHODS
        function obj=demean(obj)
            % demean all channels
            % see ts_data_channel.detrend() for details
            ts_debug(1,'Starting...');
            for i=1:obj.nChannels;
                obj.channel(i)=demean(obj.channel(i));
            end
            ts_debug(1,'Ending...');
        end
        function obj=detrend(obj,N)
            % detrend all channels
            % see ts_data_channel.detrend() for details
            ts_debug(1,'Starting...');
            if ~exist('N','var'); N=1; end
            for i=1:obj.nChannels;
                obj.channel(i)=detrend(obj.channel(i),N);
            end
            ts_debug(1,'Ending...');
        end
        function obj=resample(obj,newsamprate)
            % resample data
            % See ts_data_channel.resample() for details
            if nargout==0, error('You ran this function without any output argument!  The result will go nowhere!'); end
            for i=1:obj.nChannels,
                %                start,endat     % DEBUG
                obj.channel(i)=resample(obj.channel(i),newsamprate);
            end
        end
        function obj = trueAmplitude(obj,minfreq)
            % calculate true channel amplitudes
            % See ts_data_channel.trueAmplitude() for details
            if nargout==0, error('You ran this function without any output argument!  The result will go nowhere!'); end
            if isempty(obj), return; end
            for i=1:obj.nChannels,
                obj.channel(i)=obj.channel(i).trueAmplitude(minfreq);
            end
        end
        function c = xcorr(obj,option)
            % calculate cross-correlation between all channels, returns result
            % in a 2*M-1 x N^2 matrix, where N is the number of channels and M
            % is the channel length: each column is the correlation for a
            % different combination of channels: the correlation between
            % channels i and j is found in column (j-1)*N+i and its
            % conjugate (j, i) is found in column (i-1)*N+j
            %
            % The zero lag is at row M
            %
            % All channels must be the same length!!!
            % See ts_data_channel.xcorr_static() for details
            if nargout==0, 
                error('You ran this function without any output argument! '); 
            end
            if isempty(obj), return; end
            chanlen=length(obj.channel(1).data);
            a=zeros(chanlen,obj.nChannels);
            for i=1:obj.nChannels,
                if i==1,
                    if obj.channel(i).nSamples ~= chanlen
                        error('Channels are not the same length')
                    end
                end
                %i
                a(:,i)=obj.channel(i).data(:);
            end
            if nargin==1,
                c=ts_data_channel.xcorr_static(a);
            else
                c=ts_data_channel.xcorr_static(a,option);
            end
        end
        function obj = filter(obj,varargin)
            % One-way causal Butterworth filter
            % Just passes on variables to ts_data_channel.filter() for each
            % channel
            % For details, See Also:
            %    ts_data_channel.filter
            if nargout==0, error('You ran this function without any output argument!  The result will go nowhere!'); end
            for i=1:obj.nChannels
                obj.channel(i)=obj.channel(i).filter(varargin{:});
            end
        end
        function obj=stack(obj)
            % Stack data to one channel
            %  Currently just synchronizes and sums data
            %  Should probably at least oversample so that data are better
            %  aligned
            obj=obj.synchronize();
            for i = 2:obj.nChannels,
                obj.channel(1).data=obj.channel(1).data+obj.channel(i).data;
            end
            obj=obj.extract(1);
        end
        %%  OTHER METHODS
        function obj=setstarttimes(obj,value)   % Set starttime of all channels to the same value
            if nargout==0, error('You ran this function without any output argument!  The result will go nowhere!'); end
            value=ts_datetime(value);   % Converts string if necessary
            for i=1:obj.nChannels,
                obj.channel(i).starttime=value;
            end
        end
        function out=isempty(obj)
            out = (length(obj.channel)==1 && isempty(obj.channel(1)));
        end
        function obj=cut(obj,start,endat)
            % Cut out a section of ts_data, using ts_data_channel.cut()
            % See ts_data_channel.cut() for details
            % start,endat % DEBUG
            if nargout==0, error('You ran this function without any output argument!  The result will go nowhere!'); end
            j=0;
            for i=1:obj.nChannels,
                %                start,endat     % DEBUG
                temp = cut(obj.channel(i),start,endat);
                if ~isnan(temp.data),
                    j=j+1;
                    obj.channel(j)=temp;
                end
            end
            if j==0,
                error('no data was within the cut bounds!');
            end
            if j<i,
               obj=obj.extract(1:j);
            end
        end
        function out=issynchronized(obj)
            % Returns 1 if channels all have same length, samprate, and
            % starttimes are grouped within 1 data sample
            out=1;
            if obj.nChannels<=1, return; end
            nS_ref=obj.channel(1).nSamples;
            sr_ref=obj.channel(1).samprate;
            startTime_ref=obj.channel(1).starttime;
            maxoffset=0;
            minoffset=0;
            for i=2:obj.nChannels,
                if obj.channel(i).nSamples ~= nS_ref,
                    out=0;
                    return;
                elseif abs(obj.channel(i).samprate - sr_ref)/sr_ref>0.01,
                    out=0; ts_verbose(0,'sample rate %%%.1f%% off',100*abs(obj.channel(i).samprate - sr_ref)/sr_ref)
                    return;
                else
                    offset=obj.channel(i).starttime-startTime_ref;
                    if offset> maxoffset, maxoffset=offset;
                    elseif offset < minoffset, minoffset=offset;
                    end
                end
            end
            if maxoffset-minoffset > 1/sr_ref,
                out=0;
                ts_verbose(1,'data is not synced (max diff between starttimes (%g s) > sample step (%g s)', ...
                    maxoffset-minoffset,1/sr_ref);
            end
        end
        function objout = extract(obj,channels)
            % Extract selected channels from a ts_data object
            % tdatanew = extract(tdataorig,channels)
            %	where channels is a list of channels to keep
            
            %oldnumchans = obj.nChannels;
            %             badchans = find(channels> obj.nChannels | channels < 1);
            %             if ~isempty(badchans),
            ts_debug(1,'Starting...');
            if nargout==0,
                error('Function called without any output argument!');
            end
            badchans = (channels> obj.nChannels) | (channels < 1);
            if any(badchans),
                error('Bad channels requested: %s',num2str(channels(badchans)));
            end
            nonints=channels~=round(channels);
            if any(nonints),
                error('There are non-integer values in your channel list: %s',num2str(channels(nonints)));
            end
            
            ts_verbose(1,'Extracting ts_data channels...');
            objout = obj;
            objout.channel=ts_data_channel();
            for i = 1:length(channels);
                objout.channel(i) = obj.channel(channels(i));
            end
            ts_verbose(1,'done');
            ts_debug(1,'Ending...');
        end
        function obj = cat(obj,varargin)
            % Concatenate two or more ts_data objects (end to end)
            % obj = cat(obj1,obj2,...) concatenates the data in two or more ts_data objects
            % The objects must have the same # of channels and each channel has the
            % same sample rate and is consecutive in time.  Prints a warning message if the data are not consecutive in time.
            % Keeps filter, name, etc parameters from first data set
            if nargout==0, error('You ran this function without any output argument!  The result will go nowhere!'); end
            
            ts_debug(1,'Starting...'); tic
            if nargout ==0, error('Must provide an output argument'); end
            
            numargs = length(varargin);
            if numargs < 1,
                error('%d arguments, need at least 2',numargs+1);
            end
            
            ts_verbose(1,'Concatenating %d data sets',numargs+1);
            if isempty(obj); error('first object is empty'); end
            numchans=obj.nChannels;
            for iobj=2:numargs,
                catobj = varargin{iarg};
                if catobj.nChannels ~= numchans,
                    error('object #%d has different # of channels than object #1',iobj);
                end
                % Check if can be concatenated
                for i = 1:numchans,
                    if (obj.channel(1).samprate ~= catobj.channel(i).samprate)
                        error('Object %d, channel %d samprate is different from object 1',iobj,i);
                    elseif isnotconsecutive(obj.channel(i),catobj.channel(i))
                        error('Object %d, channel %d is not consecutive with object 1',iobj,i);
                    end
                end
                % Concatenate
                for i=1:numchans,
                    obj.channel(i) = cat(obj.channel(i),catobj.channel(i));
                end
                
            end
            ts_debug(1,'%5.2f seconds elapsed',toc);
        end % CAT
        function obj=merge(obj,obj2)
            % add new channel(s).  Replaced by "+" operator
            ts_warning('ts_data.merge() is replaced by "+": Use c=a+b instead of c=merge(a,b)');
            obj=obj+obj2;
        end
        function objA=plus(objA,objB)
            % Add channels to a ts_data object
            %  Creates a new channel if there are no "identical" channels
            %   -or- the "identical" channels are not consecutive
            %  Concatenates otherwise
            if isa(objB,'ts_data_channel');
                temp=ts_data(); temp.channel=objB;
                objB=temp;
            elseif ~isa(objB,'ts_data'),
                error('only a ts_data or ts_data_channel object can be added to a ts_data object');
            end
            for i=1:objB.nChannels,
                didCat=0;
                for j=1:objA.nChannels	% Check if can be concatenated with existing channel (consecutive and same info)
                    if (objA.channel(j).metadata==objB.channel(i).metadata)
                        ts_verbose(1,'metadatas match');
                        if isConsecutive(objA.channel(j),objB.channel(i),2) % allow 2-sample offset
                            ts_verbose(1,'Concatenating channel %d to channel %d',i,j);
                            objA.channel(j)=cat(objA.channel(j),objB.channel(i));
                            didCat=1;
                            break;
                        else
                            ts_verbose(1,'metadatas match but data not consecutive, creating new channel/stream');
                        end
                    else
                    	ts_verbose(1,'metadatas don''t match, creating new channel');
                    end
                end
                if ~didCat
                    % ADD NEW CHANNEL
                    objA.channel(objA.nChannels+1)=objB.channel(i);
                end
            end
        end
        
        [sobj, raw, offsetlist] = calcSpectra(obj,varargin)
        
        obj =   clean(obj,xfunc,minmaxfreqs,varargin);    % DEFINED IN FILE
        
        xdata = clean_calc(obj,chanlist,windowlen,outlratio,varargin); % DEFINED IN SEPARATE FILE
        
        [axhand,linehand] = plot(obj,start,plotlength,varargin); % DEFINED IN FILE
        
        obj = synchronize(obj,newsamprate); % DEFINED IN SEPARATE FILE
        
        filename = writeSAC(obj,outfname); % DEFINED IN SEPARATE FILE
        
        filename = writeSEGY(tdata,fname,siteName,shotInfo,delay,lengthSecs,traceFormat); % DEFINED IN SEPARATE FILE
        
        filename = writeSEISAN(tdata,fname); % DEFINED IN SEPARATE FILE
    end % PUBLIC METHODS
end % CLASSDEF
%     methods (Access=private) % Access by class methods only
%
%         %         function [address,delay] = calcAddress(tdata,ich,time);
%         %             [address,delay]=calcAddress(obj.channel(ich),time);
%         %         end
%
%     end
