classdef ts_data_channel_metadata
    % Class for metadata about a time series data channel.
    %
    % ts_data_channel_metadata properties:
    %  Site/station information (text):
    %    network     - Network name                          (2-letter string)
    %    station     - Station name                          (string)
    %    component   - Component name                        (3-letter string)
    %    location    - Usually '00'                          (2-letter string)
    %    information - Instrument models, serial #s, etc     (string)
    %    fullname    - Grouyps 'network.station.component'   (READONLY string)
    %  Timing:
    %    timezero    - GMT time at which station was         (ts_datetime)
    %                  synchronized
    %    tzoffset    - Instrument clock offset at            (scalar)
    %                  synchronization (seconds)
    %                  == (inst-GMT)
    %    clockdrift  - Linear clock drift                    (scalar)
    %                  == (endTime(inst)-endTime(GPS) ) / ...
    %                     (endTime-startTime)
    %    timing      - Groups {timezero, tzoffset,...        (cell array)
    %                          clockdrift}
    %  Position:
    %    lon         - Decimal degrees E                     (scalar)
    %    lat         - Decimal degrees N                     (scalar)
    %    elev        - Instrument elevation above sealevel   (scalar)
    %                  (meters)
    %    wdepth      - Water depth at station (0 if land     (scalar)
    %                  station, -elev if seafloor station)
    %    coordinates - Gets or sets [lon, lat, elev, wdepth] (4x1 vector)
    %  Instrument response:
    %    response    - Instrument response                   (ts_response)
    %
    % ts_data_channel_metadata methods:
    %    ts_data_channel_metadata - Constructor
    %  Information:
    %    GPSTime            - Convert instrument time to GPS time
    %    instrumentTime     - Convert GPS time to instrument time
    %  Transformation:
    %    update             - Updates values to those in a reference
    %  Overloaded Matlab operators:
    %    eq                 - Returns 1 if name,location,network & component
    %                         are identical
    %    neq                - Returns 1 if name,location,network or component
    %                         are not identical
    %
    % EXAMPLE:
    % meta=ts_data_channel_metadata('Napoli','IT','STS-2',40.83,14.25,25,0)
    % meta.component='BHZ';
    % newmeta=meta;
    % meta==newmeta
    % newmeta.component='BHN';
    % meta==newmeta
    % newmeta.component='ALongString';
    
    properties
        station='';     % Station name: Used to be ts_station.name
        % Two-letter code for seismology networks
        network='';
        % Two-letter code for instrument location within a station
        % Default is '00', change if you have more than one instrument,
        % otherwise identical, at the same station
        location='00';
        % Three letter code for component ('BHZ', 'HHE', etc)
        % See SEED manual, Appendix A, for channel naming convention
        component=''
        %comments=''		% Comment string (redundant with information??)
        information=''; % Can hold instrument types, serial numbers, etc
        % GMT time at which station was synchronized
        % Used to account for synchronization offsets and/or a non-GMT timezone
        timezero=ts_datetime();
        % Instrument clock offset from GMT at timezero   (ts_datetime)
        % (inst-GMT, seconds) of the clock
        tzoffset=0;
        % Linear clock drift between timezero and end of data
        % == (endTime(inst)-endTime(GPS) ) / (endTime-startTime)
        clockdrift=0;
        lon=0;          % Station location: Decimal degrees E
        lat=0;          % Station location: Decimal degrees N
        elev=0;         % Station location: Elevation above sealevel (m)
        wdepth=0;       % Water depth at station (0 if land station, -elev if station at sea floor)
        % Instrument response (ts_response)
        response=ts_response();
    end % properties
    properties (Dependent=true)
        fullname;   % READ-ONLY: String combining 'network.station.component'
        coordinates % Grouping of [lon lat elev wdepth]
        timing      % Grouping of {timezero, tzoffset, clockdrift}
    end
    
    methods
        %% CONSTRUCTOR
        function obj=ts_data_channel_metadata(station,network,information,lat,lon,elev,wdepth,timezero)
            % Constructor
            % Usage=obj=ts_data_channel_metadata(station,network,information,lat,lon,elev,wdepth,timezero)
            % You can stop where you want, the rest will take their default
            % values
            ts_init()
            if exist('station','var')
                if ~ischar(station)
                    error('"station" must be a character string')
                end
                obj.station=station;
            else return; 
            end
            if exist('network','var')
                if ~ischar(network)
                    error('"network" must be a character string')
                end
                obj.network=network;
            else return; 
            end
            if exist('information','var')
                if ~ischar(information)
                    error('"information" must be a character string')
                end
                obj.information=information;
            else return; 
            end
            if exist('lat','var')
                if ~isnumeric(lat) || length(lat)~=1
                    error('"lat" must be a numeric scalar')
                end
                obj.lat=lat;
            else return; 
            end
            if exist('lon','var')
                if ~isnumeric(lon) || length(lon)~=1
                    error('"lon" must be a numeric scalar')
                end
                obj.lon=lon;
            else return; 
            end
            if exist('elev','var')
                if ~isnumeric(elev) || length(elev)~=1
                    error('"elev" must be a numeric scalar')
                end
                obj.elev=elev;
            else return; 
            end
            if exist('wdepth','var')
                if ~isnumeric(wdepth) || length(wdepth)~=1
                    error('"wdepth" must be a numeric scalar')
                end
                obj.wdepth=wdepth;
            else return; 
            end
            if exist('timezero','var')
                if length(timezero)>1, 
                    error('More than one "timezero" value');
                elseif ~isa(timezero,'ts_datetime') && ~ischar(timezero)
                    error('"timezero" is not ts_datetime or character');
                end
                obj.timezero=timezero;
            end
        end % ts_data_channel_metadata
        
        %% SET functions (check ranges)
        function obj = set.station(obj,value)
            if ischar(value), obj.station=value;
            else error('station must be a character string');
            end
        end
        function obj = set.network(obj,value)
            if ischar(value), obj.network=value;
            else error('network must be a character string');
            end
        end
        function obj = set.location(obj,value)
            if ~ischar(value), error('location must be a 2-character string, or empty'); end
            if (length(value)==2), obj.location=value;
            elseif isempty(value), obj.location='';
            else
                if length(value)==1, obj.location=[' ' value];
                else obj.location=value(1:2);
                end
                ts_warning('forced location to be a 2-character string: from "%s" to "%s"',value,obj.location);
            end
        end
       function obj = set.component(obj,val)
           if ~isa(val,'char'), error('component must be a character string'); end
           val=strtrim(val);
           if length(val)>3,
               ts_warning('component can have at most 3 characters, forcing from %s to %s',val,val(1:3));
               val=val(1:3);
           end
           obj.component=val;
       end
%         function obj = set.comments(obj,val)
%             if ~isa(val,'char'), error('comments must be a character string'); end
%             obj.comments=val;
%         end
        function obj = set.information(obj,value)
            if ischar(value), obj.information=value;
            else error('information must be a character string');
            end
        end
        function obj = set.timezero(obj,value)
            if ischar(value)
                obj.timezero=ts_datetime(value);
            elseif isa(value,'ts_datetime'),
                if length(value)~=1, error('timezero must be single value'); end
                obj.timezero=value;
            else
                error('timezero must be a character string or ts_datetime object');
            end
            if obj.timezero==ts_datetime(); ts_warning('You set timezero to null value!'); end
            if ~isa(obj.timezero,'ts_datetime');
                error('Impossible!!! timezero is not a ts_datetime!!!');
            end
        end
        function obj = set.tzoffset(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.tzoffset=value;
                else error('tzoffset must be a scalar value');
                end
            else error('tzoffset must be a scalar numerical value');
            end
        end
        function obj = set.clockdrift(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.clockdrift=value;
                else error('clockdrift must be a scalar value');
                end
            else error('clockdrift must be a scalar numerical value');
            end
        end
        function obj = set.lat(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    if value >= -90 && value <= 90
                        obj.lat=value;
                    elseif value==-12345 || isnan(value)
                        obj.lat=NaN;
                    else
                        error('lat must be between -90 and 90 (was %d)',value);
                    end
                else error('lat must be a scalar value');
                end
            else error('lat must be a scalar numerical value');
            end
        end
        function obj = set.lon(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    if value >= -360 && value <= 360,
                        obj.lon=value;
                    elseif value==-12345 || isnan(value)
                        obj.lon=NaN;
                    else error('lon must be between -360 and 360');
                    end
                else error('lon must be a scalar value');
                end
            else error('lon must be a scalar numerical value');
            end
        end
        function obj = set.elev(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.elev=value;
                else error('elev must be a scalar value');
                end
            else error('elev must be a scalar numerical value');
            end
        end
        function obj = set.wdepth(obj,value)
            if isnumeric(value)
                if max(size(value))==1,
                    obj.wdepth=value;
                else error('wdepth must be a scalar value');
                end
            else error('wdepth must be a scalar numerical value');
            end
        end
        function obj = set.response(obj,val)
            if ~isa(val,'ts_response'), error('value must be a ts_response object'); end
            obj.response=val;
        end
        
        %% SET methods for dependent variables
        function obj = set.coordinates(obj,val)
            if ~isvector(val),error('Not a vector'); end
            if ~isnumeric(val),error('Non-numeric vector'); end
            if length(val)~=4, error('Not a 4-element vector'); end
            obj.lon=val(1);
            obj.lat=val(2);
            obj.elev=val(3);
            obj.wdepth=val(4);
        end
        function obj = set.timing(obj,val)
            if ~isvector(val),error('Not a vector'); end
            if ~iscell(val),error('Non-cell vector'); end
            if length(val)~=3, error('Not a 3-element vector'); end
            if ~isa(val{1},'ts_datetime'), error('First element is not a ts_datetime object'); end
            if ~isnumeric(val{2}), error ('2nd element is not numeric'); end
            if ~isnumeric(val{3}), error ('3rd element is not numeric'); end
            obj.timezero=  val{1};
            obj.tzoffset=  val{2};
            obj.clockdrift=val{3};
        end
        
        %% GET methods for dependent variables
        function out = get.fullname(obj)
            out=[obj.network '.' obj.station '.' obj.component];
            if strcmp(out,'..'), out=' '; end
        end
        function out = get.coordinates(obj)
            out=[obj.lon obj.lat obj.elev obj.wdepth];
        end
        function out = get.timing(obj)
            out={obj.timezero, obj.tzoffset, obj.clockdrift};
        end
        
        %% EQUALITY and INEQUALITY
        function result=eq(a,b)
            % EQ returns 1 if name,location,network & component are identical
            result=~neq(a,b);
        end
        
        function result=neq(a,b)
            % NEQ returns 1 if any of station,location,network and component are not identical
            if ~strcmp(a.station,b.station); result=1; return; end
            if ~strcmp(a.location,b.location); result=1; return; end
            if ~strcmp(a.network,b.network); result=1; return; end
            if ~strcmp(a.component,b.component); result=1; return; end
            result=0;
        end
        %% TEXT DISPLAY
        function str = char(obj,padding)
            %Generate text describing the ts_data_channel_metadata object
            % padding pads the left side, to give an offset
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            txt=[]; for i=1:length(obj), txt=[txt sprintf('''%6s'' ',char(obj(i).station))]; end
            str=     sprintf('%s%12s: %s\n',pad,'station',txt);
            txt=[]; for i=1:length(obj), txt=[txt sprintf('''%6s'' ',char(obj(i).network))]; end
            str=[str sprintf('%s%12s: %s\n',pad,'network',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('''%6s'' ',char(obj(i).information))]; end
            str=[str sprintf('%s%12s: %s\n',pad,'information',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%9s',char(obj(i).timezero,'%y%j..'))]; end
            str=[str sprintf('%s%12s: %s\n',pad,'timezero',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%8g ',obj(i).tzoffset)]; end
            str=[str sprintf('%s%12s: %s\n',pad,'tzoffset',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%8g ',obj(i).clockdrift)]; end
            str=[str sprintf('%s%12s: %s\n',pad,'clockdrift',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%8g ',obj(i).lon)]; end
            str=[str sprintf('%s%12s: %s\n',pad,'lon',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%8g ',obj(i).lat)]; end
            str=[str sprintf('%s%12s: %s\n',pad,'lat',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%8g ',obj(i).elev)]; end
            str=[str sprintf('%s%12s: %s\n',pad,'elev',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%8g ',obj(i).wdepth)]; end
            str=[str sprintf('%s%12s: %s\n',pad,'wdepth',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('''%6s'' ',char(obj(i).component))]; end
            str=[str sprintf('%s%12s: %s\n',           pad,'component',txt)];
            txt=[]; for i=1:length(obj), txt=[txt sprintf('%2dx%-2d    ',length(obj(i).response.poles),length(obj(i).response.zeros))]; end
            str=[str sprintf('%s%12s: %s\n',pad,'response (PxZ)',txt)];
        end
        function disp(obj)
            % Displays object information when not followed by a semi-colon
            disp(char(obj));
        end
        
        function outTime = GPSTime(obj,instTime)
            % Convert instrument time to GPS time
            % Usage: GPSTime=obj.GPSTime(instTime)
            %   where instTime is the instrument time
            %
            % Corrects for time-zero offset and clock drift (see driftCorrect for details)
            outTime=obj.driftCorrect(instTime,'forward');
        end
        
        function outTime = instrumentTime(obj,GPStime)
            % Convert GPS time to instrument time
            % Usage: instTime=obj.instrumentTime(GPSTime)
            %   where GPStime is the true (GPS) time
            %   used when searching for data in an instrument file
            %
            % Corrects for time-zero offset and clock drift, (see driftCorrect for details)
            outTime=obj.driftCorrect(GPStime,'backward');
        end
        function obj=update(obj,ref)
            % Updates values in a ts_data_channel_metadata object to those in a reference,
            % but ONLY if the original value was default (allows user to enter
            % new values that won't override those read in from the data file)
            ts_debug(1,'Starting');
            obj_def=ts_data_channel_metadata();
            if strcmp(obj.station,    obj_def.station),     obj.station=   ref.station; end
            if strcmp(obj.network,    obj_def.network),     obj.network=   ref.network; end
            if strcmp(obj.location,   obj_def.location),    obj.location=  ref.location; end
            if strcmp(obj.component,  obj_def.component),   obj.component= ref.component; end
%            if strcmp(obj.comments,   obj_def.comments),    obj.comments=  ref.comments; end
            if strcmp(obj.information,obj_def.information), obj.information=ref.information; end
            if         obj.timezero   == obj_def.timezero && ref.timezero ~= obj_def.timezero,   obj.timezero=  ref.timezero; end
            if         obj.lat        == obj_def.lat,        obj.lat=       ref.lat; end
            if         obj.lon        == obj_def.lon,        obj.lon=       ref.lon; end
            if         obj.elev       == obj_def.elev,       obj.elev=      ref.elev; end
            if         obj.wdepth     == obj_def.wdepth,     obj.wdepth=    ref.wdepth; end
            if         obj.tzoffset   == obj_def.tzoffset,   obj.tzoffset=  ref.tzoffset; end
            if         obj.clockdrift == obj_def.clockdrift, obj.clockdrift=ref.clockdrift; end
            if         obj.response   == obj_def.response,   obj.response=  ref.response; end
            ts_debug(1,'Ending');
        end
        
        
    end % Public methods
    
    methods (Access = private)
        function outTime = driftCorrect(obj,mytime,option)
            % Correct an instrument time for (linear) clock drift
            % Usage: correctTime = driftCorrect(obj,mytime);
            %     -or-  instTime = driftCorrect(obj,mytime,'GPStoInst');
            %
            %   Uses the 'timezero', 'tzoffset' and 'clockdrift' parameters
            %       in the ts_channel_metadata object, where:
            %       clockdrift	= (instElapsed - GPSElapsed)/GPSElapsed
            %       timezero	is the (GPS) time at which the instrument was
            %                       synchronized to GPS
            %       tzoffset    is the offset in seconds of the instrument from GPS
            %                   time at timezero (==instTime-timezero @ timezero)
            %
            % Input:
            %   mytime is the time given by the instrument clock (ts_datetime object)
            %       or a time interval (in seconds) [timezero and tzoffset are unused in this case]
            % Output:
            %	ts_datetime object
            %
            % Equations used:
            %   insttime= GPStime  + tzoffset + GPSElapsed*clockdrift
            %   GPStime = insttime - tzoffset - instElapsed*clockdrift/(1+clockdrift)
            %
            %  The first form gives the GPS time corresponding to an instrument time
            %	(useful for correcting instrument times in printouts)
            %  the second form gives the instrument time corresponding to a GPS time
            %	(useful for choosing an instrument data sample corresponding to a GPS time)
            
            ts_debug(2,'Starting...');
            if ~exist('option','var'), option='forward'; end
            cdr=obj.clockdrift; if length(cdr)>1, error('clockdrift should be a scalar'); end
            tzo=obj.tzoffset;   if length(tzo)>1, error('tzoffset should be a scalar'); end
            tz=obj.timezero;    if length(tz)>1, error('timezero should be a scalar'); end
            switch lower(option),
                case {'insttogps','forward','forwards'},    % Convert from instrument to GPS
                    if isa(mytime,'ts_datetime'),
                        if cdr==0,
                            outTime=mytime-tzo; % avoids calculating mytime-timezero, which can be long if timezero not set
                        else
                            if tz==ts_datetime();  % Time-zero was never set, don't correct
                                error('You set a clock drift, but not time zero!  Cannot correct for clock drift, quitting...');
                            end
                            instElapsed=mytime-(tz+tzo);	% Seconds since sync
                            outTime=mytime - tzo - instElapsed*cdr/(1+cdr);
                        end
                    else
                        ts_verbose(2,'"mytime" is not a ts_datetime object, assuming it is a time offset');
                        %outTime=mytime/(1+clockdrift);      % True solution
                        outTime=mytime - mytime*cdr;  % First order solution, might be more numerically stable
                    end
                case {'gpstoinst','backward','backwards'},  % Convert from GPS to instrument
                    if isa(mytime,'ts_datetime'),
                        if cdr==0,
                            outTime=mytime+tzo;     % avoids calculating mytime-timezero, which can be long if timezero not set
                        else
                            if tz==ts_datetime();  % Time-zero was never set, don't correct
                                error('You set a clock drift, but not time zero!');
                            end
                            GPSElapsed=mytime-tz;				% Seconds since sync
                            outTime=mytime + tzo + GPSElapsed*cdr;
                        end
                    else
                        ts_verbose(1,'"mytime" is not a ts_datetime object, assuming it is a time offset');
                        GPSElapsed=mytime;
                        outTime=GPSElapsed+GPSElapsed*cdr;
                    end
                otherwise,
                    error(['Unknown option: ' option]);
            end
            ts_debug(2,'Ending');
        end % private method driftCorrect
    end % Private methods
end % classdef

% Removed setdisp because it is only useful for hgetset
% superclasses, and we don't want stations to be handles!!!!
%         %% REDEFINE SETDISP TO SPECIFY INPUTS
%         function setdisp(obj)
%             pnames=properties(obj);
%             for i=1:length(pnames),
%                 p=pnames{i};
%                 v=get(obj,p);
%                 switch p,
%                     case 'lat', vstr='scalar between -90 and 90';
%                     case 'lon', vstr='scalar between -360 and 360';
%                     case 'timezero', vstr='ts_datetime object';
%                     case {'name','network','information'},
%                         vstr='character string';
%                     case {'tzoffset','clockdrift','elev'},
%                         vstr='scalar value';
%                     case 'wdepth', vstr='non-negative scalar value';
%                     otherwise, vstr='';
%                 end
%                 fprintf('%15s:  %s\n',p,vstr);
%             end
%         end

