classdef ts_event
    % Class for event information 
    % 
    %
    % ts_event properties:
    %   time   - Event time                      (ts_datetime)
    %   lat    - Event latitude                  (scalar)
    %   lon    - Event longitude                 (scalar)
    %   depth  - Event depth below sea level (m) (scalar)
    %   wdepth - Water depth at event (m)        (scalar)
    %
    % ts_event methods:
    %   ts_event - Creator
    %
    % EXAMPLE:
    % ev=ts_event('2012/12/25 10:37:14.5',45.5,2.3,14,0)
    
    properties
       time=ts_datetime;% Event time
       lat=[];          % Latitude (decimal degrees, N=positive)
       lon=[];          % Longitude (decimal degrees)
       depth=[];        % Depth (m)
       wdepth=[];       % Water depth (m)
    end
    
    methods
        %% CREATOR
        function obj=ts_event(varargin)
            % Create a ts_event object
            %
            % Usage:
            %   obj=ts_event()                          % Creates empty event
            %   obj=ts_event(event)                     % Copies an existing ts_event
            %   obj=ts_event(time,lat,lon,depth,wdepth) % time is a datetime string
            %                                             or a ts_datetime object
            ts_init;
            if nargin==0; return; end % Handel zero-argument case
            if nargin==1,
                if isa(varargin{1},'ts_event')
                    obj=varargin{1};
                    return
                else
                    error('Single argument is not a ts_event object')
                end
            end
            if nargin==5,
                if isa(varargin{1},'ts_datetime')
                    obj.time=varargin{1};
                elseif ischar(varargin{1})
                    obj.time=ts_datetime(varargin{1});
                else
                    error('first argument (datetime) is not a ts_datetime or string');
                end
                
                if isnumeric(varargin{2}) && length(varargin{2})==1
                    obj.lat=varargin{2};
                else
                    error('second argument (lat) is not a numeric scalar');
                end
                
                if isnumeric(varargin{3}) && length(varargin{3})==1
                    obj.lon=varargin{3};
                else
                    error('third argument (lon) is not a numeric scalar');
                end
                
                if isnumeric(varargin{4}) && length(varargin{4})==1
                    obj.depth=varargin{4};
                else
                    error('fourth argument (depth) is not a numeric scalar');
                end
                
                if isnumeric(varargin{5}) && length(varargin{5})==1
                    obj.wdepth=varargin{5};
                else
                    error('fifth argument (wdepth) is not a numeric scalar');
                end
            else
                error('Illegal # of input parameters: %d',nargin);
            end
        end

        %% SET METHODS
        function obj=set.time(obj,val)
            val=ts_datetime(val);   % Let ts_datetime handle errors
            obj.time=val;
        end
        function obj=set.lat(obj,val)
            if ~isa(val,'numeric'), error('value must be numeric'); end
            if length(val)>1, error('value must be a scalar'); end
            if val>90. || val <-90., ts_verbose(1,'latitude value was %g, not between -90 and 90',val); end
            obj.lat=val;
        end
        function obj=set.lon(obj,val)
            if ~isa(val,'numeric'), error('value must be numeric'); end
            if length(val)>1, error('value must be a scalar'); end
            obj.lon=val;
        end
        function obj=set.depth(obj,val)
            if ~isa(val,'numeric'), error('value must be numeric'); end
            if length(val)>1, error('value must be a scalar'); end
            obj.depth=val;
        end
        function obj=set.wdepth(obj,val)
            if ~isa(val,'numeric'), error('value must be numeric'); end
            if length(val)>1, error('value must be a scalar'); end
            obj.wdepth=val;
        end
        
        %% DISPLAY AND CHAR METHODS
        function txt=char(obj,padding)
            % Text representation of object contents
            % Usage: 
            %    txt=obj.char()
            %       -or-
            %    txt=obj.char(padding), where padding is a number of spaces
            %                           to put on the left side
            if ~exist('padding','var'), padding=0; end
            pformat=sprintf('%%%ds',padding);
            pad=sprintf(pformat,'');
            txt=[];
            txt=[txt sprintf('%s%10s: %s\n',pad,'time',char(obj.time))];
            txt=[txt sprintf('%s%10s: %g\n',pad,'lat',obj.lat)];
            txt=[txt sprintf('%s%10s: %g\n',pad,'lon',obj.lon)];
            txt=[txt sprintf('%s%10s: %g\n',pad,'depth',obj.depth)];
            txt=[txt sprintf('%s%10s: %g',pad,'wdepth',obj.wdepth)];
         end        
       function disp(obj)
           disp(char(obj));
       end     

    end %methods
end % classdef
        