function [tdata,dfile] = readMSEED(dfile,startAt,readLen)
    %READMSEED Read a MiniSEED file into a ts_data object
    % Usage:
    % 	tdata = readMSEED(dfile,start, length) 
    %   [tdata,dfile]=readMSEED(dfile,-1);
    %	
    % Inputs:
    %   dfile:      The file name or an ts_file object.
    %   start [0]:	if a scalar, start that many seconds into the file
    %				if an ts_datetime, start at given time
    %               if -1, return empty tdata, startTime=time of first sample,
    %                   endtime=time of last sample.
    %   length[0]:	number of seconds to read (if 0, read one day's data)
    %
    % Output:
    %	tdata		an ts_data object.
    %   dfile       the ts_file objet, with the custom parameter cell array
    %                   containing the first and last Times in the data file
    %

    % W Crawford 5/2009
    %%%%%%%% START ALL ts_file.read* WITH THESE LINES
    ts_debug(1,'Starting'); tic;
    if ~exist('startAt','var'), startAt=0; end
    if ~exist('readLen','var'), readLen=0; end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % OPEN FILE
    fname =	dfile.filename;
    ts_verbose(1,'Reading MSEED file %s',fname);
    fid = fopen(fname,'rb','ieee-le');
    currEndian=0;
    if fid==-1, error('failed to open the file %s\n',fname); end

    tdchans={};
    while 1,    % Loop through records
        recstart=ftell(fid);
        fixed = readDataFixedHead(fid);
        for i = 1:fixed.nBkts,
            btype=  fread(fid,1,'int16');           % Blockette type
            nextBkt=fread(fid,1,'int16');           % Byte number of next blockette
            switch btype,
                case 1000, dos= readBlkt_dataOnlySEED(fid);
                otherwise,
                    ts_verbose(0,'Skipping unknown blockette type: %s...\n',btype);
            end
            ts_debug(1,'Next blockette is at byte %d',nextBkt);
            fseek(fid,recstart+nextBkt,'bof');  % Go to next blockette
        end
        fseek(fid,recstart+fixed.datastart,'bof');  % Go to start of data

        data=readData(fid,fixed,dos);
        tdchans=stuffData(data,fixed,tdchans);
        fseek(fid,recstart+fixed.datastart+dos.reclen,'bof'); % Go to next record
        fseek(fid,recstart+dos.reclen,'bof'); % Go to next record
    end
    tdata=merge(tdchans);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function tdchans=stuffData(data,fixed,tdchans)
    
    tdata=ts_data(data,fixed.samprate);
    tdata.channel(1).metadata.network=fixed.network;
    tdata.channel(1).metadata.component=fixed.chname;
    tdata.channel(1).metadata.starttime=fixed.starttime;
    tdata.channel(1).metadata.station=fixed.station;
    if isempty(tdchans)
        tdchans{1}=tdata;
        return
    end
    
    % Find out if this channel exists already
    ich=0;
    for i=1:length(tdchans),  % Find if it corresponds to an existing channel
        oChan=get(tdchans{i},'chOrigNum',1);
  %      if sameChannel(tdchans{i},tdata)    % There should be a function like this
        if (strcmp(fixed.network,get(tdchans{i},'chNetwork',1)) && ...
            strcmp(fixed.chname,get(tdchans{i},'chName',1)) && ...
            strcmp(fixed.station,get(tdchans{i},'chInstName',1)))
            ich=i;
            break;
        end
    end
    if ich,  % If so, concatenate data
        tdchans{ich}=cat(tdchans{ich},tdata);
    else   % If not, make a new channel
        i=length(tdchans)+1;
        tdchans{i}=tdata;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f = readDataFixedHead(fid)

    A=   char(fread(fid,[1 20],'char'));
    c=textscan(A,'%6d%s %5s%2s%3s%2s');
    f.irec=c{1};
    f.dataQual=c{2}{:};
    f.station=c{3}{:};
    f.locID=c{4}{:};
    f.chname=c{5}{:};
    f.network=c{6}{:};
    % Actually, the first number has some other significance than "channel number"
    year=        fread(fid,1,'uint16');    % Start year
    jday=        fread(fid,1,'uint16');    %   Day of Year (Jan 1 is 1)
    hour=        fread(fid,1,'uint8');     %   Hours of Day (0-23)
    minute=      fread(fid,1,'uint8');     %   Minutes of Day (0-59)
    isec=        fread(fid,1,'uint8');     %	Seconds of day (0-59, 60 for leap secs)
    temp=        fread(fid,1,'uint8');     %	Unused for data (required for alignment)
    subsecs=     fread(fid,1,'uint16');    %   .0001 seconds (0-9999)
    secs=isec+subsecs/10000;
    f.starttime=ts_datetime(year,jday,hour,minute,secs);
    f.sampsperrec=fread(fid,1,'uint16');  % Number of samples in record
    srfact=      fread(fid,1,'uint16');    % Sample rate factor
    srmult=      fread(fid,1,'uint16');    % Sample rate multiplier
    if srmult==0, f.samprate=f.srfact;
    elseif srmult>0,
            if srfact>=0, f.samprate=srfact*srmult;
            else f.samprate = -1*srmult/srfact;
            end
    else
        if srfact>=0, f.samprate=-1*srfact/srmult;
        else f.samprate= 1/(srfact*srmult);
        end
    end
    f.actflag=   fread(fid,1,'uint8');     % Activity flags
    f.recioflag= fread(fid,1,'uint8');    % I/O and clock flags
    f.dqflag=    fread(fid,1,'uint8');     % Data quality flags
    f.nBkts=     fread(fid,1,'uint8');     % Number of blockettes that follow (before data)
    f.driftCorr= fread(fid,1,'int32');    % Time correction
    f.datastart= fread(fid,1,'uint16');     % Beginning of data 
    f.firstBktAddr=fread(fid,1,'uint16');            % First blockette (right after this 48-byte header)

	ts_verbose(1,'Data Fixed Header: %s',char(f.starttime));
	ts_verbose(1,'                   network=%s, station=%s, record %d, chname=%s', ...
       f.network, f.station, f.irec, f.chname);
	ts_verbose(1,'                   %d blockettes', f.nBkts);
	ts_verbose(2,'                   %d samps/rec, sps=%g, datastart=%d', ...
        f.sampsperrec,f.samprate,f,f.datastart);
end   
    
    
function  dos = readBlkt_dataOnlySEED(fid)   
    %% Read Data Only SEED Blockette
    dos.encFmt= fread(fid,1,'int8');	% encoding format
    dos.byteswp=fread(fid,1,'int8');               % Byte swapping order (0 = little-endian)
    dos.reclen= 2^(fread(fid,1,'int8'));    % Record length
    temp=fread(fid,1,'int8');               % Reserved
    ts_verbose(1,'Data Only SEED blockette: format=%d (%s), byteswap=%d, reclen=%d', ...
        dos.encFmt,getEncFormatName(dos.encFmt),dos.byteswp,dos.reclen);
end

function data=readData(fid,fixed,dos)
    nSamps=fixed.sampsperrec;
    numBytes=dos.reclen - fixed.datastart;
    data=zeros(1,nSamps);
    formatName=getEncFormatName(dos.encFmt);
    switch dos.encFmt,
        case 2, % 24-bit integers
            data=fread(fid,nSamps,'int24');
        case 3, % 32-bit integers
            data=fread(fid,nSamps,'int32');
        case 10, % Steim1 compression
             data=readSteim1(fid,numBytes,nSamps,dos.byteswp);
        otherwise, error('Data format #%d <%s> not implemented',dos.encFmt,formatName);
    end
end

function data=readSteim1(fid,numBytes,nSamps,byteswap)
% Byteswap: 0 = VAX/8086 (little-endian?)
%           1 = 68000/SPARC (big-endian?)
    % [str,maxsize,endian]=computer;
	nframes=floor(numBytes/64); % Steim compression is made up of 64-byte frames
    data=zeros(1,nSamps);
    return
    w=fread(fid,[16 nframes],'uint32');
	for i=1:nframes,
        c=fread(fid,16,'ubit2');
        c=c(end:-1:1);              % WHY???
        w(2:16)=fread(fid,15,'uint32');
        if ~byteswap % Data are read as little-endian...
            w=swapbytes(uint32(w));
            c=c([13:16 9:12 5:8 1:4]);
        end
        if i==1,
            data(1)=w(2);
            datalast=w(3);
            start=4;
            idata=1;
        else,
            start=2;
        end
        for k=start:16,
            switch c(k)
                case 0  %Should not occur:
                    error('w_k contains non-data information');
                case 1, % Four 8-byte samples
                    data(idata+1:idata+4)=data(idata)+cumsum(double(typecast(uint32(w(k)),'uint8')));
                    idata=idata+4;
                case 2, % Two 16-byte samples
                    data(idata)+cumsum(double(typecast(uint32(d(j)),'uint16')))
                    data(idata+1:idata+2)=data(idata)+cumsum(double(typecast(uint32(w(k)),'uint16')));
                    idata=idata+2;
                case 3  % One 32-byte sample
                    data(idata+1)=data(idata)+w(k);
                    idata=idata+1;
            end
        end
    end
    if idata~=nSamps
        error('Record said %d samples, read %d',nSamps,idata);
    end
    if data(1,end) ~= datalast,
        error('last data (%d) != stated last data (%d)');
    end
end


function str=getEncFormatName(encFmt)
    switch encFmt,
        case 0, str='ASCII Text';
        case 1, str='16-bit integers';
        case 2, str='24-bit integers';
        case 3, str='32-bit integers';
        case 4, str='IEEE floating point';
        case 5, str='IEEE DP floating point';
        case 10, str='Steim1';
        case 11, str='Steim2';
        case 12, str='GEOSCOPE Multiplexed 24-bit';
        case 13, str='GEOSCOPE Multiplexed 16-bit gain ranged, 3 bit exponent';
        case 14, str='GEOSCOPE Multiplexed 16-bit gain ranged, 4 bit exponent';
        case 15, str='US National Network compression';
        case 16, str='CDSN 16 bit gain ranged';
        case 17, str='Graefenberg 16 bit gain ranged';
        case 18, str='IPG - Strasbourg 16 bit gain ranged';
        case 19, str='Steim3 compression';
        otherwise, str='Unknown Format';
    end
end