function [tdata,dfile] = readAH(dfile,startAt,readLen)
% TS_READAH Read AH-format seismic data into an ts_data object
% Usage:
%	idata = readAH(theFile)		% read the entire file
%	idata = readAH(theFile,startAt,readLen) % reads part of the file
%
%	theFile can be a filename or an ts_file object with the filename
%		parameter set to the correct value
%
% "startAt" and "readLen" are ignored
%
% An AH file contains one record, which may include multiple channels, 
%  with instrument responses for each channel

% The header names come from the ah2sac program in Pascal
% v1.9
%/*	structure for data file header	--	witte, 11 June 85	*/
%
% W Crawford 2/1999

%%%%%%%% START ALL ts_file.read* WITH THESE LINES
ts_debug(1,'Starting'); tic;
%if ~exist('startAt','var'), startAt=0; end
%if ~exist('readLen','var'), readLen=0; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isa(dfile,'ts_file')
	filename = dfile.filename;
else					% 'inst' must just be the filename
	filename = dfile;
	dfile = ts_file(filename,'AH');
end

ts_verbose(1,'Reading AH file %s',filename);
[tdata, dfile] = readAHdata(dfile);	% Read in the whole file
ts_debug(1,'Ending after %.1f seconds',toc);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read the whole file in
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tdata, dfile] = readAHdata(dfile)
filename = dfile.filename;
fid = fopen(filename,'rb','ieee-be');
if (fid == -1),
	tdata=[];
	fprintf('====Couldn''t open file %s====',filename);
	return;
end

[ahhed, error] = readAHhead(fid);
ich = 0;
while ~error,

	ndata = ahhed.record_info.ndata;
	dtype = 'float32';  % AH data is stored as 32-byte floats
	switch ahhed.record_info.type,
		case 1, % FLOAT
            data=fread(fid,[1,ndata],dtype);
		case 2, % COMPLEX
            bigtemp=fread(fid,ahhed,[2,ndata],dtype);
			data = abs(bigtemp(1,:)+sqrt(-1)*bigtemp(2,:));
			clear bigtemp;
			disp('Only saving magnitude of complex data!');
		case 3, % VECTOR
            data=fread(fid,ahhed,[2,ndata],dtype);
		case 4, % TENSOR
            data=fread(fid,ahhed,[3,ndata],dtype);
		case 5, % WHAT????
            bigtemp=fread(fid,ahhed,[4,ndata],dtype);
			data(1,:) = abs(bigtemp(1,:)+1i*bigtemp(2,:));
			data(2,:) = abs(bigtemp(3,:)+1i*bigtemp(4,:));
			clear bigtemp;
		case 6, % DOUBLE
            dtype = 'float64';
			data=fread(fid,[1,ndata],dtype);
		otherwise, 
            error('Record info type -- %d -- is unknown',ahhed.record_info.type);
	end
	numchans = length(data(:,1));
    numSamps = length(data(1,:));
	samprate=1/ahhed.record_info.delta;
	dfile.nChannels=numchans;

    % Extract first and last time info
    st = ahhed.record_info.abstime;
    firstTime=ts_datetime(st.yr,st.mo,st.day,st.hr,st.mn,st.sec);
    lastTime=firstTime+(numSamps-1)/samprate;
	dfile.ch_metadata.timezero=firstTime;
    dfile.privateFirstTime=firstTime;
    dfile.privateLastTime=lastTime;

    % Extract station info
	si = ahhed.station_info;    
	dfile.ch_metadata.station=si.code;
	dfile.ch_metadata.lat=si.slat;
    dfile.ch_metadata.lon=si.slon;
    dfile.ch_metadata.elev=si.elev;
	np = si.calib.poles(1);
	if np > 0, poles = si.calib.poles(2:np+1); else poles = []; end
	nz = ahhed.station_info.calib.zeros(1);
	if nz > 0, zeros = si.calib.zeros(2:nz+1); else zeros = []; end
	gain = si.A0 * si.DS;
	oldich=ich;
	for ich = oldich+1:oldich+numchans,
		dfile.ch_metadata(ich).station= si.chan;
		dfile.ch_metadata(ich).response=ts_response('PZ','','',gain,poles,zeros);
    end
    
    % Create the ts_data object
	tdata = ts_data(data,samprate,dfile);
    tdata.channel(1).starttime=firstTime;
	%ev = ahhed.event_info; et = ev.ot;
	%dfile.event=ts_event(ts_datetime(et.yr,et.mo,et.day,et.hr,et.mn,et.sec),ev.lat,ev.lon,ev.dep,0);
	[ahhed, error] = readAHhead(fid);	
end
fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ahhed,error] = readAHhead(fid)

error = 0;
CODESIZE=6;
CHANSIZE=6;
STYPESIZE=8;
COMSIZE=80;
LOGSIZE=202;
NEXTRAS=21;
NOCALPTS=30;
AHHEADSIZE=1024;

% First check to see if we have a complete header
status = fseek(fid,AHHEADSIZE,'cof'); % Scan forward over the header size
if status == -1, error=1; ahhed = []; return, end
fseek(fid,-AHHEADSIZE,'cof'); % Scan back to start of header

ahhed.station_info.code=ah_freadchar(fid,CODESIZE,4);	% station code		
ahhed.station_info.chan=ah_freadchar(fid,CHANSIZE,6);	% lpz,spn, etc.	
ahhed.station_info.stype=ah_freadchar(fid,STYPESIZE,6);	% wwssn,hglp,etc.	
ahhed.station_info.slat=	fread(fid,1,'float32');		% station latitude 	
ahhed.station_info.slon=	fread(fid,1,'float32');		%    "    longitude 	
ahhed.station_info.elev=	fread(fid,1,'float32');		%    "    elevation 	
ahhed.station_info.DS=		fread(fid,1,'float32');			% gain				
ahhed.station_info.A0=		fread(fid,1,'float32');			% normalization	
if ahhed.station_info.A0 == 0, ahhed.station_info.A0 = 1; end
if ahhed.station_info.DS == 0, ahhed.station_info.DS = 1; end
for i = 1:NOCALPTS,										% calibration info 
	ahhed.station_info.calib.poles(i) = fread(fid,1,'float32')+sqrt(-1)*fread(fid,1,'float32');
	ahhed.station_info.calib.zeros(i) = fread(fid,1,'float32')+sqrt(-1)*fread(fid,1,'float32');
end
ahhed.event_info.lat =		fread(fid,1,'float32');		% event latitude	
ahhed.event_info.lon =		fread(fid,1,'float32');		%   "   longitude	
ahhed.event_info.dep =		fread(fid,1,'float32');		%   "   depth		
% AH CALLS YEARS THROUGH MINUTES int16, I think it's another xdr weirdo
ahhed.event_info.ot.yr=		fread(fid,1,'int32');		%   "   origin time 	
ahhed.event_info.ot.mo=		fread(fid,1,'int32');		% 		month	
ahhed.event_info.ot.day=	fread(fid,1,'int32');		% 		day		
ahhed.event_info.ot.hr=		fread(fid,1,'int32');		% 		hour	
ahhed.event_info.ot.mn=		fread(fid,1,'int32');		% 		minute	
ahhed.event_info.ot.sec=	fread(fid,1,'float32');		% 		second	
ahhed.event_info.ecomment=ah_freadchar(fid,COMSIZE,6);	% comment line
ahhed.record_info.type=		fread(fid,1,'int16');		% data type (int,float,...) 	
ahhed.record_info.ndata=	fread(fid,1,'int32');		% number of samples		
ahhed.record_info.delta=	fread(fid,1,'float32');		% sampling interval		
ahhed.record_info.maxamp=	fread(fid,1,'float32');		% maximum amplitude of record 	
% AH CALLS YEARS THROUGH MINUTES int16, I think it's another xdr weirdo
ahhed.record_info.abstime.yr=fread(fid,1,'int32');		% start time of record section 
ahhed.record_info.abstime.mo=fread(fid,1,'int32');		% 		month	
ahhed.record_info.abstime.day=fread(fid,1,'int32');		% 		day		
ahhed.record_info.abstime.hr=fread(fid,1,'int32');		% 		hour	
ahhed.record_info.abstime.mn=fread(fid,1,'int32');		% 		minute	
ahhed.record_info.abstime.sec=fread(fid,1,'float32');	% 		second	
ahhed.record_info.rmin=		fread(fid,1,'float32');		% minimum value of abscissa 	
ahhed.record_info.rcomment=ah_freadchar(fid,COMSIZE,8);	% comment line
ahhed.record_info.log=ah_freadchar(fid,LOGSIZE,6);	% log of data manipulations
ahhed.extra=				fread(fid,NEXTRAS,'float32');	% freebies 


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function str = ah_freadchar(fid,numchars,skipchars)

fseek(fid,skipchars,'cof');	% I don't understand this, has something to do with
							% UNIX xdr and machine independence
temp = fread(fid,numchars,'char')';
firstzero = find(temp==0,1,'first');
if firstzero == 1, str = []; return; end

if isempty(firstzero), firstzero = numchars+1; end
str = char(temp.*(temp<128 & temp>32));
str =str(1:firstzero-1);
