function [tdata,dfile] = readLDEO(dfile,startAt,readLen,option,value)
% TS_READLDEO Read Lamont-Doherty OBS format data into an ts_data object
%    tdata = readLDEO(filename,[startAt,readLen,[option,value]])
% Inputs [default]:
%   dfile       A ts_file object.
%   startAt [0]:if a scalar, start that many seconds into the file
%               if an ts_datetime, start at given time
%   readLen[0]:	number of seconds to read (if 0, read to end of file)
%
%   dfile should also contain
%		custom{1}: number of bytes per record [1024]
%  		custom{2}: size of header (bytes)     [34]
%   	custom{3}: address of time word from start of header [24]
%   	custom{4}: address of sample rate byte [23]
%
% option:	'decimation'	% Decimate input data by x=value
%
% Output:
%   tdata           An ts_data object, any requested times/addresses
%   				before or after the data in the file are returned as nans
%   dfile           ts_file.read() requires this!  firstTime and lastTime
%                   are stuffed with time of first and last data in file
%
% A LDEO file contains the raw OBS data for the LDEO OBSIF.  Each file contains
%  multiple records and all channels for one instrument, plus time stamps and
%  the sampling rate, but no instrument response information

%
% W Crawford 10/2003

%%%%%%%% START ALL ts_file.read* WITH THESE LINES
ts_debug('Starting.  In directory %s\n',pwd); tic;
if ~exist('startAt','var'), startAt=0; end
if ~exist('readLen','var'), readLen=0; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if get(dfile,'tzoffset'),
    error('Time-zero correction (%g seconds) not implemented for %s',FNS);
end
% Read raw data parameter info
ldeo.nChannels = dfile.nChannels;
if length(dfile.custom)==4,
	ldeo.recLen=	dfile.custom{1};
	ldeo.headBytes=	dfile.custom{2};
	ldeo.tb=		dfile.custom{3};
	ldeo.srPos=		dfile.custom{4};
else
	ldeo.recLen=	1024;
	ldeo.tb=		24;	% practically speaking, must use nChan>=4
	ldeo.srPos=		23;
	switch ldeo.nChannels,
	case 1, ldeo.headBytes=25;
	case 2, ldeo.headBytes=22;
	case 3, ldeo.headBytes=25;
	case 4, ldeo.headBytes=28;
	case 5, ldeo.headBytes=34;
	case 6, ldeo.headBytes=34;
	case 7, ldeo.headBytes=37;
	case 8, ldeo.headBytes=40;
	end
end
syncTime=  dfile.timezero;
driftRate= dfile.clockdrift;
readChans= dfile.readchans;
decimate=0; if exist('option','var'), switch option, case {'decim','decimate'}, decimate=value; end; end;

sampsPerRec = calcSPR(ldeo);


fname = dfile.filename;
if isnumeric(startAt) && startAt~=-1,
	text = ['Reading LDEO file ' fname ', starting at ']; 
	if isa(startAt,'ts_datetime')
    	ts_verbose(1,'Reading LDEO file %s, starting at time %s', fname, char(startAt));
	else
    	ts_verbose(1,'Reading LDEO file %s, starting at %.2f seconds offset', fname, startAt);
	end
end

% GET STARTTIME AND SAMPLE RATE FROM FILE HEADER
whichHead =1;	% read time from header 1, because header 0 is sometimes bad
[offsetSecs, samprate] = readHeader(fname,1,ldeo); % offsetSecs is from sync time
timeoffset = whichHead*sampsPerRec/samprate;  % Want time referenced to address 1
startSecs = offsetSecs-timeoffset;
fileStartTime = syncTime+startSecs;
% Get last data time
fid=fopen(fname);
fseek(fid,0,'eof');
fileBytes=ftell(fid);
fclose(fid);
numRecs=floor(fileBytes/ldeo.recLen);
numSamps=numRecs*sampsPerRec;
totalSecs=(numSamps-1)/samprate;
fileEndTime=fileStartTime+totalSecs;

if isnumeric(startAt) && startAt==-1,
    tdata=ts_data();    % just an empty instance
    return;
end

% CALCULATE WHERE TO START AND END READING
if isa(startAt,'ts_datetime')
	offsetSecs = startAt-fileStartTime;
else
	offsetSecs = startAt;
end

firstAddress = 1+round(2*offsetSecs - offsetSecs*samprate);

if isa(readLen,'numeric'),	% length in seconds
		numPoints = readLen*samprate;
else
	error('readLen should be a scalar, is a: %s',class(readLen));
end

% READ DATA
t1 = cputime;
data = readData(dfile,firstAddress,numPoints,ldeo,decimate);
t2 = cputime;
ts_debug(1,'Data read took %.1f seconds\n',t2-t1);

secOffset = (firstAddress-1)/samprate;
if decimate,
	secOffset = secOffset + decimate/(2*samprate);
	samprate = samprate/decimate;
end
dataStartTime =  fileStartTime+secOffset;

% STUFF DATA INTO ts_data OBJECT
ts_debug(1,[ 'readchans = [' sprintf('%d ',readChans) ']']);
dfile.readchans=readChans;
tdata = ts_data(data,samprate,dfile);
tdata.recnum=startAt;   % Is this to die for?

% DON'T FORGET TO SET TIME!!!
tdata.starttime=dataStartTime;
dfile.privateFirstTime=fileStartTime;
dfile.privateLastTime=fileEndTime;

ts_debug(3,'starttime: %.3f seconds',clocktime);

return;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [startSecs, samprate] = readHeader(fname,whichOne,ldeo)

machineformat = 'ieee-le';
fid = fopen(fname,'rb',machineformat);
if (fid == -1), error('Could not read file %s',fname); end
[f,p,m] =fopen(fid);
ts_debug(1,'filename=%s,permission=%s, machineformat=%s',f,p,m);

% GO TO THE RIGHT SPOT
fseek(fid,whichOne*ldeo.recLen,'bof');
[startSecs, samprate] = readHeaderInfo(fid,ldeo);
fclose(fid);
return;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [countSecs, samprate] = readHeaderInfo(fid,ldeo)

tb = ldeo.tb;		% Time word location?
% READ HEADER INFO
a = fread(fid,ldeo.headBytes,'uint8');
countSecs = ((((a(tb)*256. +a(tb+1))*256. +a(tb+2))*256. +a(tb+3))*256. +a(tb+4))/1000.;
samprate = a(ldeo.srPos);
fseek(fid,-ldeo.headBytes,'cof');	% Return file pointer to where it was
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function chdata = readData(dfile,startAddress,numPts,ldeo,decimate)
% READDATA Reads in data from an LDEO OBS file
% Usage: chdata = readData(dfile,startAddress,numPts,ldeo,decimate)
%
% dfile =        ts_file object (needs filename, nChannels)
% startAddress = how far into the file to start reading
% numPts =       how many samples to read (0== to end of the file)
% ldeo =         structure containing .recLen, .headBytes, .nChannels
% decimate =     only read every "decimate" samples

counter = 0; data=[];

tv=version; iVer = str2double(tv(1));	% Get Matlab major version #
if iVer > 5, readType = 'uint8';	% Matlab 6+ doesn't allow adding uint8s anyway, so let it read into a double array
else readType = 'uint8';			% Matlab 5 always returns as double array
end

rl=ldeo.recLen;
hb=ldeo.headBytes;
maxRecsAtOnce=1024;

fname = dfile.filename;
readchans = dfile.readchans;
nChans = ldeo.nChannels;
[sampsPerRec,dataBytes] = calcSPR(ldeo);
	
machineformat = 'ieee-le';
fid = fopen(fname,'rb',machineformat);
if (fid == -1), error('Could not read file %s',fname); end
[f,p,m] =fopen(fid);
ts_debug(sprintf('filename=%s,permission=%s, machineformat=%s',f,p,m));

% GET FILE LENGTH
fseek(fid,0,'eof');
flength = ftell(fid);
if (flength>1024*1024*1024), lengthMsg=sprintf('%.2f GBytes',flength/(1024*1024*1024));
elseif (flength>1024*1024),  lengthMsg=sprintf('%.2f MBytes',flength/(1024*1024)); 
elseif (flength>1024),       lengthMsg=sprintf('%.2f kBytes',flength/(1024)); 
else        				 lengthMsg=sprintf('%d Bytes',flength);
end

if numPts == 0,
	totalSamples = floor(flength/rl)*sampsPerRec;
	numPts = totalSamples-startAddress+1;
	fprintf('reading to end of file = %d samples',numPts);
end

% Convert start address, numpts to start record+addoffset, numrecs
if startAddress<1,
	nanHdr = 1-startAddress; startAddress=1; numPts = numPts-nanHdr;
else
	nanHdr=0;
end
startRec = floor((startAddress-1)/sampsPerRec);	% Start at record 0
addrOffset = startAddress - startRec*sampsPerRec - 1;
endRec = floor((startAddress+numPts-1)/sampsPerRec);
numRecs = endRec-startRec+1;

% READ DATA
DIALOGS = ts_get('dialogs');
if DIALOGS,
	hwait=waitbar(0,sprintf('Reading %s, %.0f samples...',fname,round(numPts)));
	h=get(hwait,'Children');
	set(get(h,'Title'),'Interpreter','none');
end
fseek(fid,startRec*rl,'bof');
recsLeft=numRecs;
%leftoverdata=[]; chdata = [];
leftoverdata=[];

% Preallocate chdata, saves time for big files
if ~decimate, ichsamps=numRecs*sampsPerRec;
else ichsamps=floor(numRecs*sampsPerRec/decimate);
end
chdata = zeros(nChans,ichsamps); iAddr=1;

while(1),
	if recsLeft > maxRecsAtOnce,
		readRecs = maxRecsAtOnce; recsLeft = recsLeft-maxRecsAtOnce;
    else
		readRecs = recsLeft; recsLeft=0;
	end
	[bnew,count] = fread(fid,readRecs*rl,readType);
 	if(count == 0), break; end  % End of file
	counter = counter + count;
	temp = floor(count/rl);
	bnew = bnew(1:temp*rl);
	bnew = reshape(bnew,rl,temp);
	bnew = bnew(hb+1:rl,:);	% strip off header bytes
	bnew = reshape(bnew,temp*dataBytes,1);	% make it one long data stream again
 	datalen = length(bnew); extraBytes=rem(datalen,3);
	if extraBytes,
		warning('data bytes were not a multiple of 3, cutting down');
 		bnew = bnew(1:datalen-extraBytes);
	end
	newdata = [leftoverdata; ((bnew(1:3:end)*65536. + bnew(2:3:end)*256. + bnew(3:3:end)) - 2^21)];
	newChDataLen = floor(length(newdata)/nChans);
	if decimate,
		decimleftover = rem(newChDataLen,decimate);
		newChDataLen = newChDataLen-decimleftover;
	end
	leftover = rem(length(newdata),nChans*newChDataLen);
	if leftover,
		leftoverdata = newdata(end+1-leftover:end);
		newdata = newdata(1:end-leftover);
    else
		leftoverdata = [];
	end
	%whos, nChans,newChDataLen	% DEBUG
	newchdata = reshape(newdata,nChans,newChDataLen);
	if decimate,
		numNewSamps = newChDataLen/decimate;
		temparr = zeros(nChans,numNewSamps);
		for i = 1:nChans,
			temp = reshape(newchdata(i,:),decimate,numNewSamps);
			%whos
			temparr(i,:)=mean(temp);
		end
		newchdata = temparr;
    else
		numNewSamps=newChDataLen;
	end

	%chdata = [chdata newchdata];
	chdata(:,iAddr:iAddr+numNewSamps-1) = newchdata;
	iAddr = iAddr+numNewSamps;
	clear bnew;
	% UPDATE WAITBAR
	%pos = ftell(fid);
	%recsLeft	%DEBUG
	if DIALOGS,
		waitbar((numRecs-recsLeft)/numRecs,hwait);
		%disp(sprintf('%.3f',(numRecs-recsLeft)/numRecs));	% DEBUG
	end
	if recsLeft==0, break; end
end   
if DIALOGS, close(hwait); end
if counter <= 0, error('Found no data in the input file'); end
	
%numRecs = floor(counter/rl);
%d%ataLen = numRecs*(rl-hb)/(3*nChans);
		
%size(data), numChans, dataLen
%chdata = reshape(data,nChans,dataLen);

[M,N]=size(chdata);
if decimate,
	numPts=floor(numPts/decimate);
	addrOffset = round(addrOffset/decimate);
	nanHdr=round(nanHdr/decimate);
end
% Cut off unwanted first addresses
if addrOffset > 0,
	chdata = chdata(:,(addrOffset+1):end);
end
N = N-addrOffset;
% pad end with NaNs if necessary
if N < numPts,
%	N,numPts,	%debug
	disp ('padding end with Nans');
	chdata = [chdata nan*ones(M,numPts-N)];
end
% pad beginning with NaNs if necessary
if nanHdr>0,
	disp ('padding beginning with Nans');
	chdata = [nan*ones(M,nanHdr) chdata];
end
% Cut off extra at end
chdata = chdata(:,1:numPts);

fclose(fid);
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [sampsPerRec,dataBytes] = calcSPR(ldeo)

dataBytes = ldeo.recLen-ldeo.headBytes;
sampsPerRec = (dataBytes)/(3*ldeo.nChannels);
return;
