function [tdata,dfile] = readPSEGY(dfile,startAt, readLen)
% READPSEGY Read PASCALL SEGY seismic data into an ts_data object
% idata = readPSEGY(theFile) reads a PASCALL SEGY file into an ts_data object
% idata = readPSEGY(theFile,startAt,readLen) does the same
%
%	theFile is a ts_file object
%	Doesn't use startAt or readLen, leaves that to ts_file.read(), which
%	calls this
%
% A Pascal SEGY file contains one record, one channel, and no instrument responses

% The header names come from the PASSCAL SEGY data format
%	at http://www-mpl.ucsd.edu/obs/reports/tn061/p-segy.html
%
% W Crawford 2/1999

%%%%%%%% START ALL ts_file.read* WITH THESE LINES
ts_debug(1,'Starting'); tic;
%if ~exist('startAt'), startAt=0; end
%if ~exist('readLen'), readLen=0; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

filename = dfile.filename;

ts_verbose(1,'Reading Pascall SEGY file %s',filename);
[tdata, dfile] = readData(dfile);
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read the whole file in
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tdata, dfile] = readData(dfile);
filename = get(dfile,'filename');
fid = fopen(filename,'rb');
%fid = fopen(filename,'rb','ieee-be');
if (fid == 0), error(sprintf('Could not read file %s',filename)); end

%--------------------------
% Read the header and convert some values into useful units
%--------------------------
hdr = readSEGYHeader(fid);

if hdr.sampleLength == 32767, numsamp= hdr.num_samps;
else numsamp= hdr.sampleLength; end

if hdr.data_form == 0; data_form = 'int16';
elseif hdr.data_form == 1, data_form = 'int32';
else error('Unknown header.data_form'); end

if hdr.deltaSample == 1, sampinterval=hdr.samp_rate/1e6;
else sampinterval=hdr.deltaSample/1e6; end

if hdr.coordUnits ~= 2,
	disp('Inst positions are not in expected lat/lon format!');
end
% Convert from seconds to degrees
if hdr.coordScale > 0, xmult = hdr.coordScale/3600;	% Convert from seconds to degrees
else xmult = 1/(-hdr.coordScale*3600); end

if hdr.elevationScale > 0, zmult = hdr.elevationScale;
else zmult = 1/(-hdr.elevationScale); end

%--------------------------
dfile.instname = 'SEGYinst';	% Inst name
samprate = 1/sampinterval;				% sampling rate
dfile.nChannels = 1;							% number of channels
dfile.clockdrift= 0;							% Clock drift
dfile.starttime=ts_datetime(hdr.year,hdr.day,hdr.hour,hdr.minute, ...					% start minute
								hdr.second + hdr.m_secs/1000);
dfile.channel(1).metadata.lat =    hdr.recLatOrY*xmult;
dfile.channel(1).metadata.lon =    hdr.recLongOrX*xmult;		% Longitude (degrees)
dfile.channel(1).metadata.elev =   hdr.recElevation*zmult;
dfile.channel(1).metadata.wdepth = hdr.recWaterDepth*zmult;		% Water Depth (meters)
dfile.event.time =   ts_datetime(hdr.trigyear, hdr.trigday, hdr.trighour, ...
						hdr.trigminute, hdr.trigsecond + hdr.trigmills/1000);
dfile.event.lat    = hdr.sourceLatOrY*xmult;
dfile.event.lon    = hdr.sourceLongorX*xmult;
dfile.event.depth  = hdr.sourceDepth*zmult;
dfile.event.wdepth = hdr.sourceWaterDepth*zmult;
dfile.channel(1).component = num2str(hdr.channel_number);
dfile.channel(1).metadata.response.inUnits= 'unknown';

% READ IN DATA
[data,count] = fread(fid,numsamp,data_form);
fclose(fid);
eval(['data = ' data_form '(data);']);
if count ~= numsamp,
	error(sprintf('Header specified %.0f samples, read %.0f',numsamp,count));
end
tdata = ts_data(data,samprate,dfile);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read the 240-byte header
function hdr = readSEGYHeader(fid);

hdr.lineSeq        =fread(fid,1,'int32'); % 0 Sequence numbers within line */
hdr.reelSeq        =fread(fid,1,'int32'); % 4 Sequence numbers within reel */
hdr.event_number   =fread(fid,1,'int32'); % 8 Original field record # or trigger # */
hdr.channel_number =fread(fid,1,'int32'); % 12 Trace chan # w/in orig field record */
hdr.energySourcePt =fread(fid,1,'int32'); % 16 X */
hdr.cdpEns         =fread(fid,1,'int32'); % 20 X */
hdr.traceInEnsemble=fread(fid,1,'int32');% 24 X */
hdr.traceID        =fread(fid,1,'int16'); % 28 Trace ID code: seismic data = 1 */
hdr.vertSum        =fread(fid,1,'int16'); % 30 X */
hdr.horSum         =fread(fid,1,'int16'); % 32 X */
hdr.dataUse        =fread(fid,1,'int16'); % 34 X */
hdr.sourceToRecDist=fread(fid,1,'int32'); % 36 X */
hdr.recElevation   =fread(fid,1,'int32'); % 40 X */
hdr.sourceSurfaceElevation=fread(fid,1,'int32'); % 44 X */
hdr.sourceDepth    =fread(fid,1,'int32'); % 48 X */
hdr.datumElevRec   =fread(fid,1,'int32'); % 52 X */
hdr.datumElevSource=fread(fid,1,'int32'); % 56 X */
hdr.sourceWaterDepth=fread(fid,1,'int32'); % 60 X */
hdr.recWaterDepth  =fread(fid,1,'int32'); % 64 X */
hdr.elevationScale =fread(fid,1,'int16'); % 68 Elevation Scaler: scale = 1 */
hdr.coordScale     =fread(fid,1,'int16'); % 70 Coordinate Scaler: scale = 1 */
hdr.sourceLongOrX  =fread(fid,1,'int32'); % 72 X */
hdr.sourceLatOrY   =fread(fid,1,'int32'); % 76 X */
hdr.recLongOrX     =fread(fid,1,'int32'); % 80 X */
hdr.recLatOrY      =fread(fid,1,'int32'); % 84 X */
hdr.coordUnits     =fread(fid,1,'int16'); % 88 Coordinate Units:  = 2 (Lat/Long) */
hdr.weatheringVelocity=fread(fid,1,'int16');% 90 X */
hdr.subWeatheringVelocity=fread(fid,1,'int16'); % 92 X */
hdr.sourceUpholeTime=fread(fid,1,'int16'); % 94 X */
hdr.recUpholeTime  =fread(fid,1,'int16'); % 96 X */
hdr.sourceStaticCor=fread(fid,1,'int16'); % 98 X */
hdr.recStaticCor   =fread(fid,1,'int16'); %100 X */
hdr.totalStatic    =fread(fid,1,'int16'); %102 Total Static in MILLISECS added to
												% Trace Start Time (lower 2 bytes)*/
hdr.lagTimeA       =fread(fid,1,'int16'); %104 X */
hdr.lagTimeB       =fread(fid,1,'int16'); %106 X */
hdr.delay          =fread(fid,1,'int16'); %108 X */
hdr.muteStart      =fread(fid,1,'int16'); %110 X */
hdr.muteEnd        =fread(fid,1,'int16'); %112 X */
hdr.sampleLength   =fread(fid,1,'int16'); %114 # samples in trace (unless==32767) */
hdr.deltaSample    =fread(fid,1,'int16'); %116 Samp interval (usec) (unless == 1) */
hdr.gainType       =fread(fid,1,'int16'); %118 Gain Type: 1 = Fixed Gain */
hdr.gainConst      =fread(fid,1,'int16'); %120 Gain of amplifier */
hdr.initialGain    =fread(fid,1,'int16'); %122 X */
hdr.correlated     =fread(fid,1,'int16'); %124 X */
hdr.sweepStart     =fread(fid,1,'int16'); %126 X */
hdr.sweepEnd       =fread(fid,1,'int16'); %128 X */
hdr.sweepLength    =fread(fid,1,'int16'); %130 X */
hdr.sweepType      =fread(fid,1,'int16'); %132 X */
hdr.sweepTaperAtStart=fread(fid,1,'int16');%134 X */
hdr.sweepTaperAtEnd=fread(fid,1,'int16'); %136 X */
hdr.taperType      =fread(fid,1,'int16'); %138 X */
hdr.aliasFreq      =fread(fid,1,'int16'); %140 X */
hdr.aliasSlope     =fread(fid,1,'int16'); %142 X */
hdr.notchFreq      =fread(fid,1,'int16'); %144 X */
hdr.notchSlope     =fread(fid,1,'int16'); %146 X */
hdr.lowCutFreq     =fread(fid,1,'int16'); %148 X */
hdr.hiCutFreq      =fread(fid,1,'int16'); %150 X */
hdr.lowCutSlope    =fread(fid,1,'int16'); %152 X */
hdr.hiCutSlope     =fread(fid,1,'int16'); %154 X */
hdr.year           =fread(fid,1,'int16'); %156 year of Start of trace */
hdr.day            =fread(fid,1,'int16'); %158 day of year at Start of trace */
hdr.hour           =fread(fid,1,'int16'); %160 hour of day at Start of trace */
hdr.minute         =fread(fid,1,'int16'); %162 minute of hour at Start of trace */
hdr.second         =fread(fid,1,'int16'); %164 second of minute at Start of trace */
hdr.timeBasisCode  =fread(fid,1,'int16'); %166 Time basis code: 2 = GMT */
hdr.traceWeightingFactor=fread(fid,1,'int16'); %168 X */
hdr.phoneRollPos1  =fread(fid,1,'int16'); %170 X */
hdr.phoneFirstTrace=fread(fid,1,'int16'); %172 X */
hdr.phoneLastTrace =fread(fid,1,'int16'); %174 X */
hdr.gapSize        =fread(fid,1,'int16'); %176 X */
hdr.taperOvertravel=fread(fid,1,'int16'); %178 X */
hdr.station_name   =fread(fid,6,'char');  %180 Station Name code (5 chars + \0) */
hdr.sensor_serial  =fread(fid,8,'char');  %186 Sensor Serial code (7 chars + \0) */
hdr.channel_name   =fread(fid,4,'char');  %194 Channel Name code (3 chars + \0) */
hdr.totalStaticHi  =fread(fid,1,'int16'); %198 Total Static (MSEC) added to Trace
											% Start Time (high 2 bytes)*/
hdr.samp_rate      =fread(fid,1,'int32'); %200 Sample interval (USEC) */
hdr.data_form      =fread(fid,1,'int16'); %204 Data Format flag: 0=16bit, 1=32bit int
hdr.m_secs         =fread(fid,1,'int16'); %206 MSEC of Seconds of Start of trace
hdr.trigyear       =fread(fid,1,'int16'); %208 year of Trigger time */
hdr.trigday        =fread(fid,1,'int16'); %210 day of year at Trigger time */
hdr.trighour       =fread(fid,1,'int16'); %212 hour of day at Trigger time */
hdr.trigminute     =fread(fid,1,'int16'); %214 minute of hour at Trigger time */
hdr.trigsecond     =fread(fid,1,'int16'); %216 second of minute at Trigger time */
hdr.trigmills      =fread(fid,1,'int16'); %218 MSEC of seconds of Trigger time */
hdr.scale_fac      =fread(fid,1,'float32'); %220 Scale Factor (IEEE 32 bit float) */
hdr.inst_no        =fread(fid,1,'int16'); %224 Instrument Serial Number */
hdr.not_to_be_used =fread(fid,1,'int16'); %226 X */
hdr.num_samps      =fread(fid,1,'int32'); %228 Number of Samples as a 32 bit integer
											% (when sampleLength == 32767) */
hdr.max            =fread(fid,1,'int32'); %232 Maximum value in Counts */
hdr.min            =fread(fid,1,'int32'); %236 Minimum value in Counts */
