function [tdata,dfile] = readSAC(dfile,startAt,readLen)
% TS_READSAC Read SAC-format seismic data into an ts_data object
    % tdata = readSAC(dfile) reads a SACfile into an ts_data object
    % tdata = readSAC(dfile,startAt,readLen) does the same thing
    %
    %	theFile is a ts_file object
    %	Ignores startAt and readLen, leaves that to calling function ts_file.read
    %	
    % A SAC file contains one record, one channel, and no instrument responses

    % The header names come from the SAC data format, llnl somewhere
    %
    % W Crawford 7/1999

    %%%%%%%% START ALL ts_file.read* WITH THESE LINES
    ts_debug(1,'Starting\n');  tic;
    if ~exist('startAt','var'), startAt=0; end
    if ~exist('readLen','var'), readLen=0; end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    tdata=readSACFile(dfile);

    % put file start and end times in dfile
    ch=tdata.channel(1);
    dfile.privateFirstTime=ch.starttime;
    dfile.privateLastTime =ch.starttime+(ch.nSamples-1)/ch.samprate;
    
return;

function tdata=readSACFile(dfile)
    % Read the whole file
    fname =	dfile.filename;
    ts_verbose(1,'Reading SAC file %s',fname);
    itime=1; 

    fid = fopen(fname,'rb','ieee-be');	% Macs, Suns
    if (fid == 0), error('Could not read file %s',filename); end
    % Read the header
    hdr = apl_readSACHeader(fid);
    if hdr.iftype ~= itime,
    ts_verbose(2,'hdr.iftype=%d, expected %d (time series file)',hdr.iftype,itime);
    ts_verbose(2,'checking if file is little-endian');
        fclose(fid);
        fid = fopen(fname,'rb','ieee-le');	% Intel boxes
        hdr = apl_readSACHeader(fid);
        if hdr.iftype ~= itime, 
            fprintf('hdr.iftype=%d, expected %d (time series file)\n',hdr.iftype,itime);
            error('Tried little- and bid-endian, quitting');
        end
    end

    % Convert some values into useful units
    idisp=6; ivel=7; iacc=8; ivolts=50;
    switch hdr.idep,
        case idisp,	chunits = 'nm';
        case ivel,	chunits = 'nm/s';
        case ivolts, chunits = 'vel(volts)';
        case iacc,	chunits = 'nm/s^2';
        otherwise,	chunits = 'counts?';
    end
    if hdr.leven ~= 1, error('data are not evenly spaced'); end

    %--------------------------
    startTime = ts_datetime(hdr.nzyear,hdr.nzjday, hdr.nzhour,hdr.nzmin, hdr.nzsec + hdr.nzmsec/1000);
    samprate = 1/(hdr.delta);	% sampling rate
    dfile.ch_metadata.station = hdr.kstnm;	% Station name
    dfile.nChannels = 1;		% number of channels
    dfile.ch_metadata(1).clockdrift = 0;		% Clock drift
    dfile.ch_metadata(1).timezero = startTime;
    dfile.ch_metadata(1).lat = hdr.stla;
    dfile.ch_metadata(1).lon = hdr.stlo;
    dfile.ch_metadata(1).elev = hdr.stel;
    dfile.ch_metadata(1).wdepth = hdr.stdp;		% Water Depth (meters)
    dfile.event.time = startTime+hdr.o;
    dfile.event.lat=hdr.evla;
    dfile.event.lon=hdr.evlo;
    dfile.event.depth=-hdr.evel;
    dfile.event.wdepth=hdr.evdp;
    dfile.ch_metadata(1).component = hdr.kcmpnm;
    dfile.ch_metadata(1).response = ts_response('PZ',chunits,'',1,[],[]);

    % READ IN DATA
    [data,count] = fread(fid,hdr.npts,'float32');
    data = data';
    fclose(fid);
    if count ~= hdr.npts,
        error('Header specified %.0f samples, read %.0f',hdr.npts,count);
    end
    tdata = ts_data(data,samprate,dfile);
    tdata.channel(1).starttime=startTime;
return

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Read the 158-byte header
function hdr = apl_readSACHeader(fid)

    hdr.delta		=fread(fid,1,'float32');
    hdr.depmin		=fread(fid,1,'float32');
    hdr.depmax		=fread(fid,1,'float32');
    hdr.scale		=fread(fid,1,'float32');
    hdr.odelta		=fread(fid,1,'float32');
    hdr.b			=fread(fid,1,'float32');
    hdr.e			=fread(fid,1,'float32');
    hdr.o			=fread(fid,1,'float32');
    if hdr.o==-12345, hdr.o=0; end
    hdr.a			=fread(fid,1,'float32');
    hdr.internal	=fread(fid,1,'float32');
    hdr.t			=fread(fid,10,'float32');
    hdr.f			=fread(fid,1,'float32');
    hdr.resp		=fread(fid,10,'float32');
    hdr.stla		=fread(fid,1,'float32');
    hdr.stlo		=fread(fid,1,'float32');
    hdr.stel		=fread(fid,1,'float32');
    hdr.stdp		=fread(fid,1,'float32');
    hdr.evla		=fread(fid,1,'float32');
    hdr.evlo		=fread(fid,1,'float32');
    hdr.evel		=fread(fid,1,'float32');
    hdr.evdp		=fread(fid,1,'float32');
    hdr.unused		=fread(fid,1,'float32');
    hdr.user		=fread(fid,10,'float32');
    hdr.dist		=fread(fid,1,'float32');
    hdr.az			=fread(fid,1,'float32');
    hdr.baz			=fread(fid,1,'float32');
    hdr.gcarc		=fread(fid,1,'float32');
    hdr.internal2	=fread(fid,1,'float32');
    hdr.internal3	=fread(fid,1,'float32');
    hdr.depmen		=fread(fid,1,'float32');
    hdr.cmpaz		=fread(fid,1,'float32');
    hdr.cmpinc		=fread(fid,1,'float32');
    hdr.unused2		=fread(fid,11,'float32');
    hdr.nzyear		=fread(fid,1,'int32');
    hdr.nzjday		=fread(fid,1,'int32');
    hdr.nzhour		=fread(fid,1,'int32');
    hdr.nzmin		=fread(fid,1,'int32');
    hdr.nzsec		=fread(fid,1,'int32');
    hdr.nzmsec		=fread(fid,1,'int32');
    if hdr.nzyear==-12345, hdr.nzyear=0; end
    if hdr.nzjday==-12345, hdr.nzjday=0; end
    if hdr.nzhour==-12345, hdr.nzhour=0; end
    if hdr.nzmin== -12345, hdr.nzmin=0; end
    if hdr.nzsec== -12345, hdr.nzsec=0; end
    if hdr.nzmsec==-12345, hdr.nzmsec=0; end
    hdr.nvhdr		=fread(fid,1,'int32');
    hdr.internal4	=fread(fid,2,'int32');
    hdr.npts		=fread(fid,1,'int32');
    hdr.internal5	=fread(fid,2,'int32');
    hdr.unused3		=fread(fid,3,'int32');
    hdr.iftype		=fread(fid,1,'int32');
    hdr.idep		=fread(fid,1,'int32');
    hdr.iztype		=fread(fid,1,'int32');
    hdr.unused4		=fread(fid,1,'int32');
    hdr.iinst		=fread(fid,1,'int32');
    hdr.istreg		=fread(fid,1,'int32');
    hdr.ievreg		=fread(fid,1,'int32');
    hdr.ievtyp		=fread(fid,1,'int32');
    hdr.iqual		=fread(fid,1,'int32');
    hdr.isynth		=fread(fid,1,'int32');
    hdr.unused5		=fread(fid,10,'int32');
    hdr.leven		=fread(fid,1,'int32');
    hdr.lspol		=fread(fid,1,'int32');
    hdr.lovrok		=fread(fid,1,'int32');
    hdr.lcalda		=fread(fid,1,'int32');
    hdr.unused6		=fread(fid,1,'int32');
    hdr.kstnm		=char(fread(fid,8,'char')');
    hdr.kevnm		=char(fread(fid,16,'char')');
    hdr.khole		=char(fread(fid,8,'char')');
    hdr.ko			=char(fread(fid,8,'char')');
    hdr.ka			=char(fread(fid,8,'char')');
    hdr.kt			=char(fread(fid,[8,10],'char')');
    hdr.kf			=char(fread(fid,8,'char')');
    hdr.kuser		=char(fread(fid,[8,3],'char')');
    hdr.kcmpnm		=char(fread(fid,8,'char')');
    hdr.knetwk		=char(fread(fid,8,'char')');
    hdr.kdatrd		=char(fread(fid,8,'char')');
    hdr.kinst		=char(fread(fid,8,'char')');
    %hdr.kstnm,hdr.kevnm,hdr.khole,hdr.ko,hdr.ka,hdr.kt,hdr.kf,hdr.kuser,hdr.kcmpnm
return;