function [tdata,dfile] = readUTIG(dfile,startAt,readLen)
% TS_READUTIG Read UTIG/GeoAzur format OBS data into an ts_data object
% Usage:       tdata = readUTIG(dfile,startAt,length)
%      [tdata,dfile] = readUTIG(dfile,startAt,length)
%
% Inputs:
%   dfile:      A ts_file object.
%   startAt [0]:	if a scalar, start that many seconds into the file
%				if an ts_datetime, start at given time
%               if -1, return empty tdata, startTime=time of first sample,
%                   endtime=time of last sample.
%   length[0]:	number of seconds to read (if 0, read one day's data)
%
% Output:
%	tdata		an ts_data object.
%	dfile		is the same dfile object as input

%%%%%%%% START ALL ts_file.read* WITH THESE LINES
ts_init;
ts_debug(1,'Starting...'); tic;
if ~exist('startAt','var'), startAt=0; end
if ~exist('readLen','var'), readLen=0; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
global BLOCKSIZE HEADERSIZE
BLOCKSIZE=16384;
HEADERSIZE=40;

% SPECIFY AND OPEN FILE
fullfname =			dfile.filename;
ts_verbose(1,'Reading UTIG/GeoAzur OBS file %s',fullfname);
fid = fopen(fullfname,'rb','ieee-be');
if fid==-1, error('fopen() failed to open the file %s',fullfname); end

% GET HEADER AND DATA TIME BOUNDS
ts_debug(2,'READING HEADER FOR FIRST BLOCK');
%hdr = getblocks(fid,1,1);
nBlks=calcLastFullBlock(fid);
ts_debug(2,'READING LAST BLOCK''S HEADER');
[hdr,data] = getblocks(fid,nBlks,1);
lastLen=length(data(1,:));
samprate=hdr.samprate;
lastTime=hdr.datetime + lastLen/samprate;

dfile.privateFirstTime=hdr.datetime;
dfile.privateLastTime=lastTime;
ts_verbose(2,'firstTime=%s',char(firstTime));
ts_verbose(2,'lastTime =%s',char(lastTime));

if isnumeric(startAt),
    if startAt==-1,
        tdata=ts_data();
        tdata=set(tdata,'samprate',samprate);
        t=toc; ts_debug(1,'Exiting after %.1f seconds',t);
        fclose(fid);
        return
    end
end

dfile = setDFileInfo(dfile,hdr);% PUT PARAMETERS INTO DATAFILE OBJECT

%% GET AND STUFF DATA INTO ts_data OBJECT
[fp,startTime] = calcFilePos(dfile,firstTime,startAt,readLen,nBlks);

if readLen==0 && (lastTime-startTime) > 86400,
    ts_warning('You would have read %.1f days of data, limiting to 1 day',(lastTime-startTime)/86400);
    readLen=86400;
    [fp,startTime] = calcFilePos(dfile,firstTime,startAt,readLen,nBlks);
end

ts_debug(2,'READING DATA\n');
tdata = getData(samprate,dfile,startTime,fid,fp);
fclose(fid);

dfile.privateFirstTime=firstTime;
dfile.privateLastTime=lastTime;

ts_debug(1,'Exiting after %.1f seconds\n',toc);
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dfile = setDFileInfo(dfile,hdr)

numChans=decodeChanMask(hdr.CHAN_MASK);

dfile.channel(i).metadata.station=num2str(hdr.STATION_ID);
dfile.nChannels=numChans;	% Does this copy all values to new channels?
for i=1:numChans,
	dfile.channel(i).metadata.station=num2str(hdr.STATION_ID);
end
if numChans==4,         % Just made up for now
    dfile.channel(1).metadata.component='BHZ';
    dfile.channel(2).metadata.component='BHX';
    dfile.channel(3).metadata.component='BHY';
    dfile.channel(4).metadata.component='SHP';
end

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function tdata = getData(samprate,dfile,startTime,fid,fp)

[hdr,DATA]=getblocks(fid,fp.firstBlk,fp.numBlks);
DATA=DATA(:,fp.firstSamp:fp.firstSamp+fp.numSamps-1);  %%% GET ONLY DATA WE WANT FROM THESE BLOCKS (SKIP SAMPSKIP SAMPLES)

readchans = dfile.readchans;
tdata = ts_data(DATA(readchans,:),samprate,dfile);
tdata.channel(:).starttime = startTime;

end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ilastblock=calcLastFullBlock(fid)

global BLOCKSIZE
fseek(fid,0,'eof');
fpos=ftell(fid);
ilastblock = ceil(fpos/BLOCKSIZE);
%if rem(fpos,BLOCKSIZE)  > fpos,
if ilastblock*BLOCKSIZE  > fpos,
    ts_warning('Incomplete last block: will be ignored')
    ilastblock = ilastblock-1;
end
ts_debug(1,'%d full blocks in file',ilastblock);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [fp,startTime] = calcFilePos(dfile,firstTime,start,len,numBlks)
% firstTime is the start of data on the instrument, already drift
% corrected.  Will return address of first datapoint on or after the first
% time (to avoid getting, for example, 2008/12/31 23:59:59.9999, when
% 2009/01/01 was asked for: this would cause a problem in output filenames
% when writing data)

sampRate=get(dfile,'samprate');
%syncTime = get(dfile,'timezero');
%driftRate = get(dfile,'driftRate');
numChans = get(dfile,'numchans');

% CALCULATE BLOCKS TO READ
ts_debug(2,'firstTime = %s, samprate=%d\n',char(firstTime),sampRate);
if isa(start,'ts_datetime'),	% Read in at given time
    startTime=start;            % Absolute (GPS) time
    ts_debug(2,'startTime %s ',char(start));
else						% Read in at a given number of seconds from the beginning\
    startTime=firstTime+start;  % Absolute (GPS) time
    ts_debug(2,'start Offset %d seconds ',start);
end

[totSampsPerBlock,chSampsPerBlock] = calcSampsPerBlock(numChans); % TOTAL SAMPLES, NOT PER CHANNEL
offsetSecs=startTime-firstTime;
offsetSamps=ceil(sampRate*offsetSecs);  
lastSamp=numBlks*chSampsPerBlock;
numSamps = len*sampRate;
if len==0,
    numSamps=(numBlks*chSampsPerBlock);
elseif offsetSamps+numSamps-1 > lastSamp,
    numSamps=lastSamp-offsetSamps;
end
if offsetSamps>lastSamp, error('start is after end of file!'); end

ts_debug(1,'=> First read address = %d\n',round(offsetSamps+1));

%%% CALCULATE STARTING RECORD TO READ FROM
sampSkip = rem(offsetSamps,chSampsPerBlock);        % NUMBER OF SAMPLES TO SKIP IN THE BLOCK
firstBlk = floor(offsetSamps/chSampsPerBlock)+1;	% BLKS TO SKIP:
lastBlk  = floor((offsetSamps+numSamps-1)/chSampsPerBlock)+1;
if lastBlk>numBlks, error('lastBlk>numBlks'); end;

fp.firstBlk = firstBlk;                           % STARTING BLOCK NUMBER ON FILE
fp.firstSamp = sampSkip+1;
fp.numBlks  = lastBlk-firstBlk+1;
fp.numSamps  = numSamps;

startTime=firstTime;    
ts_debug(2,'Obtained startTime is %s',char(startTime));
end

%-----------------------------------------------------------------------------

function [hdr,data] = getblocks(fid,iblock,numblocks)
% Returns header from first block, demultiplexed data from all blocks
% iblock=block number (start at 1)

global BLOCKSIZE HEADERSIZE

fseek(fid,(iblock-1)*BLOCKSIZE,'bof'); % Go to requested block

% Read header
hdr = readhead(fid);

% Calculate number of samples
nCh = decodeChanMask(hdr.CHAN_MASK);
[dataspb,chanspb]=calcSampsPerBlock(nCh);
datasamps=dataspb*numblocks;
chansamps=chanspb*numblocks;

% Read in multiplexed data
TEMP=zeros(1,datasamps);
for i=1:numblocks,
    [tmp, count]=fread(fid,dataspb,'ubit24');
    tmp=tmp-2^23;
    if count~=dataspb
        error('Fread failed in readdata, end of file?');
    end
    istrt=1+(i-1)*dataspb;
    TEMP(1,istrt:istrt+dataspb-1) = tmp;
    fseek(fid,HEADERSIZE,'cof');
end

%demultiplex data
data=zeros(nCh,chansamps);
for i=1:nCh
    data(i,:)=reshape(TEMP(1,i:nCh:datasamps),1,chansamps);
end
end


%-----------------------------------------------------------------------------
function [datasamps,chansamps]=calcSampsPerBlock(nCh)
global BLOCKSIZE HEADERSIZE
databytes=BLOCKSIZE-HEADERSIZE;
datasamps=floor(databytes/3);  % 24-bit data
chansamps=datasamps/nCh;
if rem(datasamps,nCh), error('Blocks contain non-integral (%g) channels',chansamps); end
end

%-----------------------------------------------------------------------------
function header=readhead(fid)
% Assumes we are already at the right spot in the file
global HEADERSIZE

ts_debug(1,'Starting...  fid=%d\n',fid);
[HDR,count]=fread(fid,HEADERSIZE,'uint8'); % read 40 bytes
if count~=HEADERSIZE,
    error('Could not read header (asked for %d bytes, got %d)',HEADERSIZE,count);
end

%Decode Header:
header.DISK_WRITE =        HDR( 1)*256 + HDR( 2);
header.HDR_NUM  =        ((HDR( 3)*256 + HDR( 4))*256 + HDR( 5))*256 + HDR( 6);
header.HDRS_PER_WRITE  = ((HDR( 7)*256 + HDR( 8))*256 + HDR( 9))*256 + HDR(10);
header.CHAN_MASK =         HDR(11)*uint16(256) + HDR(12);
header.SAMP_INT_USEC =     HDR(13)*256 + HDR(14);
header.STATION_ID =        HDR(15)*256 + HDR(16);
year=               2000+HDR(17);
month=              HDR(18);
day=                HDR(19);
hour=               HDR(20);
minute=             HDR(21);
second=             HDR(22);
header.FTTICK =            HDR(23)*256 + HDR(24);
header.SOFT_VER=           HDR(25);
header.ADC_RATE =          HDR(26)*256 + HDR(27);
% 28-40 are reserved for future use.

header.datetime=ts_datetime(year,month,day,hour,minute,second);
header.samprate=1e6/header.SAMP_INT_USEC;
%Print info to screen
ts_debug(2,'  Disk Write Number:            %d',header.DISK_WRITE);
ts_debug(2,'  Header Number (this write): %ld',header.HDR_NUM);
ts_debug(2,'  Headers Per Disk Write:     %-4.0f',header.HDRS_PER_WRITE);
temp='                ';
for i=1:16, temp(i)=num2str(bitget(header.CHAN_MASK,i)); end;
ts_debug(2,'  Channel Mask:               %sb',temp);
ts_debug(2,'  Sampling Interval (usec):     %-6.0f\n',header.SAMP_INT_USEC);
ts_debug(2,'  Station ID:                   %d\n',header.STATION_ID);
ts_debug(2,'  Date and Time:                %s\n',char(header.datetime));
ts_debug(2,'  Fast Time Ticker:             %ld\n',header.FTTICK);
ts_debug(2,'  Software Version:             %d\n',header.SOFT_VER);
ts_debug(2,'  ADC Conversion Rate:          %ld',header.ADC_RATE);
ts_debug(1,'Exiting ...');
end

%-----------------------------------------------------------------------------
function [numChans,chans]= decodeChanMask(mask)
% Returns # of channels and list of channel #s
numChans = 0;
chans=zeros(1,8);
for i=1:8
    if bitget(mask,i),
        numChans=numChans+1;
        chans(numChans) = i;
    end
end

chlist=sprintf('%d ', chans);
ts_debug(1,'Channel Mask --> %d channels:  %s',numChans,chlist);

end
