function [obj,dfile] = readWEBB(dfile,start,readlen)
% READWEBB Read Webb-format OBS data (~1991-2002) into an ts_data object
%    obj = readWEBB(datafile,start,readlen)
% Inputs [default]:
%   dfile           A ts_file object containing (at least) the file
%                   name, the number of channels,
%                   and a 'custom' cell array containing:
%						{1} the number of bytes per record
%						{2} the number of samples per channel per record
%						{3} byteswap: if 1, use 'ieee-be', if 0, use 'ieee-le'
%						{4} headerAddresses: number of samples before and between each header block
%						{5} headerSize: the number of bytes in each header block
%						{6} the start address (in bytes) within the header of the time Word
%						{7} the clock rate (for the counter).  Usually 256 Hz,
%					    	but different for some experiments
%						{8} the data sampling rate (sps)
%						{9} the start time of the instrument clock (ts_atetime)
% start [0]:	if a scalar, start that many seconds into the file
%				if an ts_datetime, start at given time
% readlen[0]:	number of seconds to read (if 0, read to end of file)
% Output:
%   obj           A ts_data object
%   tfile         A ts_file object with the file start and end times
%                 stuffed in, if start=-1
%
% A WEBB file contains raw WEBB OBS and pressure gauge data.  This format was used on
% "Spahr Webb" OBSs from about 1988 to 2000. Each file contains multiple records and
% all channels for one instrument, as well as time stamps, but no instrument response
% information
%
%BUG: was reading beyond the end of the file on whole file read: I just
%     reduced the last record to read by 1

%
% W Crawford 2/1999, updated 7/2009

%%%%%%%% START ALL ts_file.read* WITH THESE LINES
ts_debug(1,'Starting'); tic;
if ~exist('start','var'), start=0; end
if ~exist('readlen','var'), readlen=0; end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch class(start),
	case 'char',
		start=ts_datetime(start);	% if char, assume it is input for ts_datetime
	case {'ts_datetime','double'},
	otherwise,
		error('start must be a double, string, or ts_datetime');
end

maxreadsize=1*1024*1024*1024;   % By default, don't read more than 1 GB at a time
if ~isa(dfile,'ts_file'),
		error('First argument must be an ts_file object')
end

if isa(start,'ts_datetime'),
    ts_verbose(1,'Reading Webb file %s, starting at %s',dfile.filename,char(start));
else
    ts_verbose(1,'Reading Webb file %s, starting at %.2f seconds from file start',dfile.filename,start);

end

custom=dfile.custom;
if ~iscell(custom) || length(custom) ~= 9,
    disp('  1: the number of bytes in one record');
    disp('  2: the number of samples per channel in one record');
    disp('  3: byteswap flag: 0=''ieee-le'' (Intel (most Webb instruments))');
    disp('                    1=''ieee-be'' (Motorola (STS-DOG))');
    disp('  4: headerAddresses: number of samples before and between each header block');
    disp('  5: headerSize: the number of bytes in each header block');
    disp('  6: the start address (in bytes) within the header of the time Word');
    disp('  7: the clock rate (for the counter).  Usually 256 Hz,');
    disp('  8: the sample rate');
    disp('  9: the instrument clock start time');
    error('custom must be an 9-element cell array');
end
wbb=getWebbCustom(dfile);
if dfile.nChannels==0, error('Number of channels undefined'); end

firstTime = readRecStartTime(dfile,1);
% NOTE: All clock drifts are handled by ts_file.read()
ts_debug(1,'firstTime=%s\n',char(firstTime));
dfile.privateFirstTime=firstTime;
[numRecs,numSamps]=getFileInfo(dfile,wbb);
dfile.privateLastTime=firstTime+(numSamps-1)/wbb.samprate;

if readlen==0,
    numBytes=numSamps*2*length(dfile.readchans);
	readlen=numSamps/wbb.samprate;
	if numBytes <= maxreadsize,
		fprintf('reading entire file (%s)\n',ts_datetime.seconds2str(readlen));
    else
		readlen=86400;
		fprintf('reading 1 day (data is %s long)\n',ts_datetime.seconds2str(readlen));
	end
end
switch class(start),
	case 'ts_datetime', startSecs = start-firstTime;
	case 'double',      startSecs = start;
        if start==-1,
            obj=dfile;
            return;
        end
    otherwise,          error('start must be a double, string, or ts_datetime');
end
firstAddress = round(max([startSecs*wbb.samprate,1]));

numSamps = readlen*wbb.samprate;
lastAddress = firstAddress + numSamps-1;
firstRec = floor((firstAddress-1)/wbb.datalen)+1;	% Records count from 1
lastRec = floor((lastAddress-1)/wbb.datalen)+1;
lastRec = floor((lastAddress-1)/wbb.datalen);       % Just a kludge to keep from overrunnin
tdata=readRecord(dfile,firstRec);
for recno = firstRec+1:lastRec,
	tdata = cat(tdata,readRecord(dfile,recno));
end

%addrOffset = firstAddress-(firstRec-1)*wbb.datalen;
%obj = cut_addr(tdata,addrOffset,addrOffset+numSamps-1);

reclen=wbb.datalen/wbb.samprate;
%startSecs, firstRec, reclen, (firstRec-1)*reclen
cutAt=startSecs-(firstRec-1)*reclen;
ts_debug(1,'startSecs=%g, firstRec=%d, reclen=%d, cutAt=%d', ...
    startSecs, firstRec, reclen,cutAt);

% obj=tdata.cut(cutAt,readlen); % Cutting is done by tdata.readAndCorrectTime
obj=tdata;

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function firstTime = readRecStartTime(dfile,recNo)
% Start counting records and addresses at "1"

wbb = getWebbCustom(dfile);
ts_debug(1,'ap_readWEBB:in readRecStartTime()');

fname =			dfile.filename;

fid = fopen(fname,'rb',wbb.machineformat);
if (fid <= 0), error('Could not read file %s',fname); end
if recNo>1, status = fseek(fid,(recNo-1)*wbb.reclen,'cof'); end

% SKIP FIRST SET OF DATA TO GET TO FIRST TIME BLOCK
dataSize=2; % 16-bit data
numtoskip = wbb.timeblocks(1);
dblocklen = numtoskip*dfile.nChannels;
fseek(fid,dblocklen*dataSize,'cof');
% READ IN FIRST TIME BLOCK
% Skip non-time bytes in the time block
blocksread = 0;
tblc = wbb.timebyteloc;
[a,count] = fread(fid,wbb.timeblocksize,'uint8');
clockcounter = a(tblc+1) + 256*(a(tblc+2) + 256*(a(tblc+3) +256*a(tblc+4)));
%timeaddr=numtoskip+.5;		% ASSUME Time counter read 1/2 sample after center of last sample
timeaddr=numtoskip+1;		% ASSUME Time counter at center of next sample (consistent with C version)

as=sprintf('%02x ',a);
ts_debug(1,'%8.1f: %s: %08x = %.3fs',timeaddr,as,clockcounter,clockcounter/wbb.clockrate);
% ship clock ticks back to center of sample #1
clockticks = clockcounter - (timeaddr-1)*(wbb.clockrate/wbb.samprate);

ts_debug(1,'startticks: %.3f',clockticks);

fclose(fid);

% Calculate uncorrected record start time
firstTime = wbb.timezero + clockticks/wbb.clockrate;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function tdata = readRecord(dfile,recno)
% Start counting records and addresses at "1"

wbb = getWebbCustom(dfile);
ts_debug(1,'Starting...');


readchans = dfile.readchans;

dataType = 'uint16';

fid = fopen(dfile.filename,'rb',wbb.machineformat);
if (fid == 0), error('Could not read file %s',fname); end
if recno>1, status = fseek(fid,(recno-1)*wbb.reclen,'bof'); end

itb=1;	caddr = 1; firstclock = 1; clockticks = 0;
chdata = zeros(dfile.nChannels,wbb.datalen);
oldSecs=-1;
while (1);
	if (caddr > 1),	% READ IN A TIME BLOCK
		% Skip non-time bytes in the time block
		blocksread = 0;
		tblc = wbb.timebyteloc;
		[a,count] = fread(fid,wbb.timeblocksize,'uint8');
		clockcounter = a(tblc+1) + 256*(a(tblc+2) + 256*(a(tblc+3) +256*a(tblc+4)));
		%timeaddr=caddr-.5;	% ASSUME Time counter is read 1/2 sample before center of next sample
		timeaddr=caddr;	% ASSUME Time counter is at the center of next sample (consistent with C version)
%		if DEBUG>1,
            as=sprintf('%02x ',a);
			clockSecs = clockcounter/wbb.clockrate;
			ts_debug(2,'%8.1f: %s: %08x = %.3fs',timeaddr,as,clockcounter,clockSecs);
			if oldSecs>0,
				delSecs=clockSecs-oldSecs;
				ts_debug(2,', del=%g secs => %g samps/sec\n',delSecs,(timeaddr-oldTimeaddr)/delSecs);
			else
				ts_debug(2,'\n');
			end
			oldSecs=clockSecs; oldTimeaddr=timeaddr;
%		end
		if (firstclock == 1),
			clockticks = clockcounter - (timeaddr-1)*(wbb.clockrate/wbb.samprate);
		end
		firstclock = 0;
	end
	numtoread = wbb.timeblocks(itb);
	if numtoread+caddr-1 > wbb.datalen; numtoread = wbb.datalen-caddr+1; end
	dblocklen = numtoread*dfile.nChannels;
	
	[data, count] = fread(fid,dblocklen,dataType);
%    whos,caddr,numtoread
	chdata(:,caddr:caddr+numtoread-1) = reshape(data,dfile.nChannels,numtoread);
	caddr = caddr+wbb.timeblocks(itb);
	if itb < length(wbb.timeblocks), itb = itb+1; end
	if caddr >= wbb.datalen, break, end
end
ts_debug(1,'startticks: %.3f',clockticks);
fclose(fid);

DEBUGMOISE=0;
if DEBUGMOISE,	% looking for # of 0x0ds in data, to search for MOISE bug...
	lbs = chdata - 256*floor(chdata/256);
	for i = 1:16,
		lsb(i) = length(find(lbs==(i-1)));
	end
	fprintf(' 0x%02x\n',0:15);
	fprintf('%5d\n',lsb);
end
tdata = ts_data(chdata,wbb.samprate,dfile);
%tdata = set(tdata,'recnum',recno);

for i = 1:length(readchans),
	tdata.channel(i).starttime=wbb.timezero+clockticks/wbb.clockrate;
end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function wbb = getWebbCustom(dfile)
wbb=cell2struct(dfile.custom,{'reclen',...
                                'datalen', ...
                                'byteswap', ...
                                'timeblocks', ...
                                'timeblocksize', ...
                                'timebyteloc', ...
                                'clockrate', ...
                                'samprate', ...
                                'timezero'}, ...
                                2);

if isempty(wbb.timeblocksize), wbb.timeblocksize = 16; end
if isempty(wbb.timebyteloc),
    switch(wbb.timeblocksize),
        case 4,  wbb.timebyteloc = 0;
        case 8,  wbb.timebyteloc = 2;
        case 16, wbb.timebyteloc = 4;
    end
end

ts_debug(1,'timeword %d(%d)',wbb.timeblocksize,wbb.timebyteloc);

wbb.datasize = 2;	% 16-bit data
wbb.machineformat = 'ieee-le'; % The standard Webb device uses a CPU-8088
if wbb.byteswap,
	wbb.machineformat = 'ieee-be';	% The STS-DOG, for example, uses a 68K processor
end
if ~isa(wbb.timezero,'ts_datetime'); error('timezero must be a ts_datetime object'); end

return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [numRecs,numSamps]=getFileInfo(dfile,wbb)
    % Get length of file
    fid=fopen(dfile.filename);
    fseek(fid,0,'eof');
    filebytes=ftell(fid);
    fclose(fid);
    numRecs=floor((filebytes-1)/wbb.reclen)+1;
    numSamps=numRecs*wbb.datalen;
return