function [tdata,dfile] = read(obj,startAt,readLen)
% Read time-series data into a ts_data object
    % Usage:
    %   tdata = obj.read(startAt,readLen)
    %			-or-
    %   [tdata,dfile] = obj.read(startAt,readLen);
    %
    % reads in a time series datafile
    %
    %
    % Input:
    % obj:    a ts_file object, created using 'ts_file(filename,type)'.
    %         The filename parameter can be a single character string or 
    %         a cell array of strings.
    %           To only read some of the channels, specify in the 'readchans'
    %           field of the object before reading, i.e.:
    %               obj.readchans=[1 2];    %
    % 
    % startAt [0]:	start of data to read:
    %               - if a scalar, start x seconds into the file.  If -1,
    %               then don't read anything, just get file info
    %               - if a ts_datetime object, start at the given time
    % length[0]:	number of seconds to read
    %               - if 0, read to end of file
    %
    % NOTE:   If you want to read from a continuous database, use readDB()
    %
    % NOTE 2: The start time read in will be drift corrected, if a
    %         clock drift is entered in the ts_file object.   The read 
    %         length will NOT be drift corrected, so that instruments
    % 		  with the same sampling rate but different drifts 
    %         return the same number of samples.  This should probably be
    %         changed, now that ts_data channels are independent (previously,
    %         they were all in one array).
    %
    % DEVELOPERS NOTE: This program calls private read functions for each data type.
    %         These files, named readXXXX, where XXXX is the data format, sit
    %         in the ts_file/private directory, where you can also find a template
    %         file.  They should not correct for drift or choose which channel to
    %         read: ts_file/read() will take care of that.  They don't even have
    %         to select which data to read, although it is often useful to speed up
    %         (or even permit) the reading of big files from which only a small
    %         chunk will be read.
    %

    % OLD STUFF, I DON'T LIKE
    % tdata = read(datafile,startAt,length,option)
    %		
    % option:		'next' reads the data right after the specified section
    %        		'previous' reads the data right before the specified
    %        		section
    %   also possible to use 'start' to specify which file in the cell array to
    %   read
    %
    %       if the file list contains several filenames (cell array), the program
    %       assumes that they are in chronological order.  You can also specify
    %       the filename using the wildcard '*', in which case the set of files is
    %       assumed to be in chrono order (THIS IS BAD, should also handle
    %       multiple files with same time span, different channel)

    %
    % W Crawford 2/1999
    %   last updated: 5/2009

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%% Standard beginning for ts_* functions %%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ts_debug(1,'Starting'); tic;             %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    if ~exist('startAt','var'), startAt = 0; end
    if ~exist('readLen','var'), readLen = 0; end

    %global TS_READ_LASTREAD    % AVOID REREADING FILES THAT ARE ALREADY READ (not yet implemented)

    dfile=obj;  % In case of an early return
    fname=obj.filename;

    %% VERIFY THAT THERE IS A READ ROUTINE FOR THE SPECIFIED DATA FORMAT
    callfnctn=sprintf('read%s',obj.format);
    if ~exist(callfnctn,'file'), 
        error('%s does not exist',callfnctn);
    end

    %% HANDLE TIME SYMBOLS IN FILENAME
    fnames=handleTimeSymbols(fname,startAt,readLen);
    
    %% FOR EACH DATA FILE
    for ifn=1:length(fnames), % Handles uniformly spaced files
        fname=fnames{ifn};    % May still refer to multiple files (cell array
                              %   or wildcards)

        %% CHECK FOR EXISTENCE OF INPUT FILE, EXPAND TO LIST OF FILES IF NECESSARY
        filenames=getFileList(fname);

        %% HANDLE startAt == character or startAt == -1
        if isnumeric(startAt),
            if startAt==-1,
                [dfile.privateFirstTime,dfile.privateLastTime]=...
                                        findStartEndTime(dfile,filenames);
                verbosemsg(1,'Data starts at %s, ends at %s\n',char(dfile.firstTime),char(dfile.lastTime));
                return;
             end
        elseif ischar(startAt),
            startAt=ts_datetime(startAt);
        elseif ~isa(startAt,'ts_datetime'),
            error('startAt must be numeric, a ts_datetime object, or time as a string');
        end
        
        %% READ THE DATA
        if length(filenames)==1,
            obj.filename=filenames{1};
            [tdata,dfile]=readAndCorrectTime(obj,startAt,readLen);
        else    % multiple file case: consecutive files, or diferent channels at same time.
            [tdata,dfile]=readMultipleFiles(obj,filenames,startAt,readLen);
        end
    end
    
    %% Pass on channel information, if it hasn't been modified by the read
        tdata=fillChannelInfo(tdata,obj);
    
    %% Order the output channels as specified by readchans
    if ~isequal(dfile.readchans,1:dfile.nChannels)
        tdata=orderChans(tdata,dfile);
    end
    
    ts_debug(2,'Input data = %s',char(tdata))
    
    %% Standard end for TiSKit functions %%%%%%%%%%%%%
    ts_debug(1,'Exiting after %.1f seconds',toc); %
    %%
end % end of function


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fnames=handleTimeSymbols(fname,startAt,readLen)    
    if ~iscell(fname),  % Check for time symbols in filename only if it is a char array
        if ts_datetime.hastimesymbols(fname),
            filedelta=3600; % Default files spacing is one hour
            fileoffset=0;   % Default file offset from interval is 0
            fnamesub = regexp(fname, ':', 'split');
            if length(fnamesub)>1
                filedelta=num2str(fnamesub{2});
                if length(fnamesub)>2,
                    fileoffset=num2str(fnamesub{3});
                end
            end
            fnames=getTimeNames(fnamesub, startAt, readLen,filedelta,fileoffset);
        else
            fnames={fname};
        end
    else
        fnames={fname};
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function filenames=getFileList(fname)
%% CHECKS FOR EXISTENCE OF INPUT FILE, EXPANDS TO LIST OF FILES IF
%% NECESSARY
    if ischar(fname),
        path=fileparts(fname);  %dir returns filename w/o path
        d=dir(fname);
        if isempty(d), error('No files matched "%s"',fname); end
        if length(d) > 1,
            verbosemsg(2,'%s expanded to :\n',FNS,fname);
            filenames=cell(1,length(d));
            for i=1:length(d),
                filenames{i}=fullfile(path,d(i).name);
                verbosemsg(2,'\t%s\n',filenames{i});
            end
        else
            filenames={fname};
        end
    else    % fname was already a Cell array of filenames
        filenames=fname;
        for i=1:length(filenames),
            d=dir(filenames{i});
            if isempty(d), error('No files matched %s',fname{i}); end
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fnames=getTimeNames(obj, startAt,readLen,fileDelta,fileOffset)
    if ~isa(startAt,'ts_datetime'),
        error('startAt must be a ts_datetime object if the filename has time symbols');
    end
    
    fname=obj.filename;
    firstFileTime = floor(startAt,fileDelta,fileOffset);
    lastFileTime  = floor(startAt+readLen,fileDelta,fileOffset);
    i=0;
    tm=firstFileTime;
    while tm<=lastFileTime,
        i=i+1;
        fnames{i}=char(tm,fname); %#ok<AGROW>
        tm=tm+fileDelta;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [tdata,dfile]=readMultipleFiles(obj,filenames,startAt,readLen)
    % Data should be continuous and the alphabetical order of files
    % should also be chronological
    firstRead=1; startGap=0;
    for i=1:length(filenames),
        % 1: Find files with data in the time span
        % 2: Identify different channels, if there are any..
        
        dfile = ts_file(obj);   % Copy the ts_file object;
        dfile.filename=filenames{i};
        [tdata,dfile]=readAndCorrectTime(dfile,startAt,readLen);
        
        ts_debug(2,'fstart=%s,  fend=%s,  startAt=%s, readLen=%d',...
            char(dfile.fileStart),char(dfile.fileEnd),char(startAt),readLen);
        if firstRead && isnumeric(startAt),    % Convert to time from start of first file
            startAt=fstart+startAt;
        end
        
        if ((fstart <= startAt) && (fend > startAt) || (fstart>startAt) && (fstart <startAt+readLen)),    % There is data in this file
            if fstart>startAt, % there is a data gap at the startAt
                startGap=(fstart-startAt);
                realstart=startAt;
                startAt=fstart;
                if readLen ~= 0,
                    readLen=readLen-startGap;
                end
            end
            if fend > startAt+readLen,    % The rest of data is in one file
                if firstRead,
                    [tdata,dfile]=readAndCorrectTime(dfile,startAt,readLen);
                    %cmd = sprintf('tdata=%s(dfile,startAt,readLen);',callfnctn);
                    %ts_debug(2,cmd);
                    %eval(cmd);
                    if startGap ~=0,    % Insert zeros at the beginning
                        tempdata=tdata;
                        tempdata.data=[];
                        tempdata=set(tempdata,'numsamps',0);
                        tempdata=set(tempdata,'starttime',realstart);
                        tdata=cat(tempdata,tdata);
                    end
                else
                    [tempdata,dfile]=readAndCorrectTime(dfile,startAt,readLen);
                    %cmd = sprintf('tempdata=%s(dfile,startAt,readLen);',callfnctn);
                    %ts_debug(2,cmd);
                    %eval(cmd);
                    ts_debug(2,'cat(tdata,tempdata)');
                    tdata=cat(tdata,tempdata);
                end
                break;
            else % will have to read a subsequent file for rest of data
                [tempdata,dfile]=readAndCorrectTime(dfile,readFunc,startAt,0);
                %cmd = sprintf('tempdata=%s(dfile,startAt,0);',callfnctn);
                %ts_debug(2,cmd);
                %eval(cmd);
                samprate=tempdata.samprate;
                numsamps=tempdata.numsamps;
                readLength = numsamps/samprate;
                oldstart=startAt; startAt=startAt+readLength;
                ts_debug(2,'Increasing startAt from %s to %s (+%.0f seconds)', ...
                    char(oldstart),char(startAt),numsamps/samprate);
                readLen=readLen-readLength;
                if firstRead,
                    tdata=tempdata;
                    firstRead=0;
                else
                    ts_debug(2,'cat(tdata,tempdata)');
                    tdata = cat(tdata,tempdata);
                end
            end
        elseif i==length(filenames),
            error('Did not find data in any of the files within the requested time interval:\n\t%s-%s', ...
                char(startAt),char(startAt+readLen));
        end
        startGap=0;
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [startTime,endTime]=findStartEndTime(obj,filenames)
    %% SPECIFY READ ROUTINE FOR THE SPECIFIED DATA FORMAT
    for i = 1:length(filenames),
        fname=filenames{i};
        dfile=ts_file(obj);    % otherwise, would just be a pointer
        dfile.filename=fname;
        [~,dfile]=readAndCorrectTime(dfile,-1);
        if i==1, startTime=dfile.firstTime; endTime=dfile.lastTime;
        else
            if dfile.firstTime < startTime; startTime=dfile.firstTime; end
            if dfile.lastTime > endTime;   endTime=dfile.lastTime; end
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ALL READS PASS THROUGH HERE TO CORRECT FOR CLOCK ERRORS AND CUT OUTPUTS
function [tdata,obj]=readAndCorrectTime(obj,startAt,readLen)
% Convert GPS time to instrument time, read in data, then correct results
% back to GPS time.  obj is a ts_file object
    if ~exist('readLen','var'), readLen=0; end
    %% SPECIFY READ ROUTINE FOR THE SPECIFIED DATA FORMAT
    callfnctn=sprintf('read%s',obj.format);
    if ~exist(callfnctn,'file'), 
        error('%s does not exist',callfnctn);
    end
    eval(sprintf('readFunc = @%s;',callfnctn));

   if ischar(startAt) || isa(startAt,'ts_datetime')
        ts_debug(1,'Reading file "%s" starting at GPS time %s',obj.filename,char(startAt));
        dt=ts_datetime(startAt)-obj.ch_metadata(1).timezero;	% ASSUMES ANY FILE READ HAS SAME CLOCK DRIFT/OFFSET
        ts_debug(1,'  starttime is %.1f days (%.2e seconds) after timezero',dt/86400., dt);
    end
    ts_debug(1,'  clockdrift(ch1)=%.2e, tzoffset(ch1)=%.1f',obj.ch_metadata(1).clockdrift, obj.ch_metadata(1).tzoffset, char(obj.ch_metadata(1).timezero));
    
    % Correct for time drift and offset
    clockCorrsStore=obj.clockCorrs;
    readChansStore=obj.readchans;
    if isequal(readChansStore,1:obj.nChannels),
        readChansStore=[];
    end
    instStartAt=obj.ch_metadata(1).instrumentTime(startAt);
    if ischar(startAt) || isa(startAt,'ts_datetime')
        ts_debug(1,'GPS start time %s converted to instrument start time %s',char(startAt),char(instStartAt));
    else
        ts_debug(1,'GPS time offset %.2f converted to %.2f',startAt,instStartAt);
    end
    % The read function must not try to apply clock
    % corrections or return only some channels
    obj.clockCorrs=[0 0];
    obj.readchans=[];
    [tdata,obj]=readFunc(obj,instStartAt,readLen);
    if ~isa(tdata,'ts_data'); 
           error('read Function %s did not return a ts_data object', callfnctn);
    end
    %% put clock corrs and read chans back in 
    obj.clockCorrs=clockCorrsStore;
    obj.readchans=readChansStore;
    %% Clock correct the times (file start and end, plus data start)
    obj.privateFirstTime=obj.ch_metadata(1).GPSTime(obj.firstTime);
    obj.privateLastTime =obj.ch_metadata(1).GPSTime(obj.lastTime);
    ts_debug(1,'Channel 1 first read time converted from %s (instrument time) to %s (GPS)', ...
            char(tdata.channel(1).starttime),char(obj.ch_metadata(1).GPSTime(tdata.channel(1).starttime)));
    
    for i=1:tdata.nChannels,
        tdata.channel(i).starttime=obj.ch_metadata(i).GPSTime(tdata.channel(i).starttime);
    end

    if isa(startAt,'numeric')
        startOffset=tdata.channel(1).starttime-obj.firstTime; % Seconds offset between start of data and start of file
        if startAt - startOffset > 0, % start of data is too early
            tdata=tdata.cut(startAt-startOffset,readLen);
        end
        return
    end
    tdata=tdata.cut(startAt,readLen);   % Cut data to requested length if necessary
end

function tdata=orderChans(tdata,dfile)
   oldtdata=tdata;
   nrCh=length(dfile.readchans);
   tdata.nChannels=nrCh;  % Allocates the correct # of channel objects
   for i=1:nrCh
        tdata.channel(i)=oldtdata.channel(dfile.readchans(i));
   end
end

function tdata=fillChannelInfo(tdata,dfile)
    for i=1:tdata.nChannels,
        if dfile.nChannels==tdata.nChannels
            iDFile=i;
        else
            iDFile=1;
        end
        tdata.channel(i).metadata=tdata.channel(i).metadata.update(dfile.ch_metadata(iDFile));
    end
end