classdef ts_shotinfo
    % Class for shot information
    %  Used in ts_data.writeSEGY to split the data according to the shots
    %
    % I haven't used this in a long time, don't know if the description is
    %  accurate, or if the class works.  Could be better organized (make 
    %  properties read-only so that all properties are guaranteed to have same
    %  length, overload length() and numel() operators).  Shot time could
    %  (should?) be ts_datetime object, numshots should be called nShots or
    %  replaced by the length() method, etcetera...
    % 
    % ts_shotinfo properties:
    %   number   - shot numbers   (vector)
    %   syear    - shot years
    %   sjday    - shot julian day
    %   shour    - shot hour
    %   sminute  - shot minute
    %   ssecond  - shot second
    %   lat      - Decimal degrees N
    %   lon      - Decimal degrees E
    %   depth    - shot depth below sealevel (m)
    %   wdepth   - water depth below sealevel at shot position
    %   numshots - number of shots in the list
    %
    % ts_shotinfo methods:
    %   ts_shotinfo - Creator, reads from a shotfile
    %   add         - Add a shot (have to use shots = add(shots, ...) )
    
    properties
        number=[];          % shot numbers
        syear;              % shot Time
        sjday;              % shot Time
        shour;              % shot Time
        sminute;              % shot Time
        ssecond;              % shot Time
        lat=[];              % Decimal degrees N
        lon=[];              % Decimal degrees E
        depth=[];            % shot depth below sealevel (m)
        wdepth=[];           % water depth below sealevel at shot position
    end % properties
    
    properties (Dependent=true)
        numshots;          % Number of shots in list
    end % Dependent properties
    
    methods
        %% CONSTRUCTOR
        function obj=ts_shotinfo(fname)
            % Create a ts_shotinfo object from a shotfile
            % Usage=obj=ts_shotinfo(fname)
            % Reads from a shotfile that has one header line, then
            % number, time (yy:jdy:HH:MM:SS.MSS delay lon lat elev depth wdepth
            if nargin==0; return; end
            ts_init();
            if ~exist(fname,'file')
                error('No such file: %s',fname);
            end
            fid=fopen(fname,'r');
            C=textscan(fid,'%d %d:%d:%d:%d:%f %f %f %f %f %f %f','HeaderLines',1);
            nums=C{1};
            obj.number=C{1};
            obj.syear=C{2}+1900;
            obj.sjday=C{3};
            obj.shour=C{4};
            obj.sminute=C{5};
            obj.ssecond=C{6};
            % C{7}=delay, not used
            obj.lon  = C{8};
            obj.lat = C{9};
            % C{10}=elevation, not used
            obj.depth = C{11};
            obj.wdepth= C{12};
        end % Constructor
        
        function obj=add(obj,time,lat,lon,number,depth,wdepth)
            % Usage=obj=add(obj,time,number,lat,lon,depth,wdepth)
            % You can stop at depth (wdepth will be set to 0)
            % THIS IS STUPID, JUST A PLACE HOLDER (should be able to type
            % myobj.add(...), not myobj=myobj.add(....) )
            if nargin<4,
                error('Must at least specify time, lat & lon');
            end
            i=obj.numshots+1;
            obj.time(i)=ts_datetime(time);
            obj.lat(i)=lat;
            obj.lon(i)=lon;
            if nargin>3,
                obj.number(i)=number;
                if nargin > 4, % shotdepth specified
                    obj.depth(i)=depth;
                    if nargin > 5, % water depth specified
                        obj.wdepth(i)=wdepth;
                    else
                        obj.wdepth(i)=0;
                    end
                else
                    if i==1,
                        error('You must specify shot depth for first shot')
                    else
                        obj.depth(i)=obj.depth(i-1);
                    end
                end
            else    % Choose # after preceding shot (or 1 if no preceding shot)
                if i==1;
                    obj.number(i)=1;
                else obj.number(i)=obj.number(i-1)+1;
                end
            end
        end % function
        
        %% RANGE CHECK ON SET
        function obj = set.syear(obj,value)
            if isnumeric(value), obj.syear=value;
            else error('syear must be a number');
            end
        end
        function obj = set.sjday(obj,value)
            if isnumeric(value), obj.sjday=value;
            else error('sjday must be a number');
            end
        end
        function obj = set.shour(obj,value)
            if isnumeric(value), obj.shour=value;
            else error('shour must be a number');
            end
        end
        function obj = set.sminute(obj,value)
            if isnumeric(value), obj.sminute=value;
            else error('sminute must be a number');
            end
        end
        function obj = set.ssecond(obj,value)
            if isnumeric(value), obj.ssecond=value;
            else error('ssecond must be a number');
            end
        end
        function obj = set.lat(obj,value)
            if isnumeric(value), obj.lat=value;
            else error('lat must be a number');
            end
        end
        function obj = set.lon(obj,value)
            if isnumeric(value), obj.lon=value;
            else error('lon must be a number');
            end
        end
        function obj = set.number(obj,value)
            if isnumeric(value), obj.number=value;
            else error('number must be a number');
            end
        end
        function obj = set.depth(obj,value)
            if isnumeric(value), obj.depth=value;
            else error('depth must be a number');
            end
        end
        function obj = set.wdepth(obj,value)
            if isnumeric(value), obj.wdepth=value;
            else error('wdepth must be a number');
            end
        end
        
        function value=get.numshots(obj)
            if isempty(obj.number)
                value=0;
            else
                value=length(obj.number);
            end
            %whos
            %value
        end
        
        %% TEXT DISPLAY
        function txt = char(obj)
            txt=sprintf ('Time                  Lat       Lon        #      Depth  WDepth\n');
            nMax=20;
            obj.numshots
            if obj.numshots> nMax,
                firstsample=1:round(nMax/2);
                lastsample =(obj.numshots-round(nMax/2)):obj.numshots;
            else
                firstsample=1:obj.numshots;
                lastsample=[];
            end
            for i=firstsample;
                    txt=sprintf('%s%04d/%03d_%02d:%02d:%02d.%03d %-9.5f %-10.5f %-6d %-6.1f %-5.0f\n',txt, ...
                        obj.syear(i),obj.sjday(i),obj.shour(i),obj.sminute(i),floor(obj.ssecond(i)),floor(1000*(obj.ssecond(i)-floor(obj.ssecond(i)))), ...
                        obj.lat(i),obj.lon(i),obj.number(i), obj.depth(i), obj.wdepth(i));
            end
            if ~isempty(lastsample),
                txt=sprintf('%s...\n',txt);
                for i=lastsample;
                    txt=sprintf('%s%04d/%03d_%02d:%02d:%02d.%03d %-9.5f %-10.5f %-6d %-6.1f %-5.0f\n',txt, ...
                        obj.syear(i),obj.sjday(i),obj.shour(i),obj.sminute(i),floor(obj.ssecond(i)),floor(1000*(obj.ssecond(i)-floor(obj.ssecond(i)))), ...
                        obj.lat(i),obj.lon(i),obj.number(i), obj.depth(i), obj.wdepth(i));
                end
            end
        end
        
        function disp(obj) % Displays object information when not followed by a semi-colon
            disp(char(obj));
        end
        
    end % methods
end % classdef
