function [axhand,linehand] = plot(obj,varargin)
    %PLOT Plot spectra in ts_spectra object
    % [ha,hl] = plot(obj, ...)
    %  Inputs:
    % 	obj is an ts_spectra object
    %	... are ts_figureproperties field, value pairs (type 'doc ts_figureproperties' for options)
    %  Outputs:
    %	ha is an array of handles to all the subplot axes.
    %	hl is an array of handles to all the plotted lines
    
    clf
    
    if ~isempty(varargin),
        [obj,figInfo]=setPlotProperties(obj,varargin{:});
    else
        figInfo=obj.figureprops;
    end
    
    freqs = obj.freqs;
    
    numchans = obj.nChannels;
    yscale = figInfo.yscale;
    linehand = [];
    overlay = figInfo.overlay;
    if overlay,        
        colorStrs='rgbcmyk'; cslen=length(colorStrs);
        lineStrs={'-','--','-.',':'}; lslen=length(lineStrs);
    end
    for i = 1:numchans,
        fontsize = figInfo.fontsize;
        %   obj             % DEBUG
        %   obj.sinfo(i)    % DEBUG
        %   figInfo    % DEBUG
        fmin = figInfo.subplot(i).xmin;
        fmax = figInfo.subplot(i).xmax;
        %	marker = figInfo.subplot(i).marker;
        if isnan(fmin), fmin = min(freqs); end
        if isnan(fmax), fmax = max(freqs); end
        inds = find(freqs>=fmin & freqs<=fmax);
        ymin = figInfo.subplot(i).ymin;
        ymax = figInfo.subplot(i).ymax;
        if ~overlay,
            [thePos,info] = figInfo.calcplotpos(numchans,i);
            axes('position',thePos);
            lineStr='b-';
        else
             if i==1,
                [thePos,info] = figInfo.calcplotpos(1,1);   % Single figure
                axes('position',thePos);
            end
            iLine = ceil(i/cslen); iLine=rem(iLine-1,lslen)+1;
            lineStr=lineStrs{iLine};
            colorStr=colorStrs(rem(i-1,cslen)+1);
        end
        data = obj.spectdata(i,inds);
        if ~isnan(ymin), lymin = ymin;
        else lymin = 10^floor(log10(min(data))); end
        if ~isnan(ymax), lymax = ymax;
        else lymax = 10^ceil(log10(max(data))); end
        if lymin <= 0 || isnan(lymin), lymin = 1e-100; end
        if lymax <= lymin || isnan(lymax), lymax = 10*lymin; end
        ystr = [obj.sinfo(i).name ' (' obj.sinfo(i).units ')'];
        if strcmp(yscale,'dB'),
            data = 10*log10(data);
            ystr = [obj.sinfo(i).name ' (dB ref 1 ' obj.sinfo(i).units ')'];
            lymin = 10*log10(lymin); lymax = 10*log10(lymax);
        end
        
        % PLOT DATA
        h = loglog(freqs(inds),data);
        if overlay, hold on, end
        %
        
        linehand = [linehand; h];
        if info.isBottom; xlabel('Frequency (Hz)','FontSize',fontsize);
        else set(gca,'XTickLabel',[]); end
        if ~overlay,
            ylabel(ystr,'FontSize',fontsize,'Interpreter','none');  
        end
        
        if strcmp(yscale,'dB') || strcmp(yscale,'linear')
            set(gca,'YScale','linear');
        end
        if ~overlay,
            axis([fmin fmax lymin lymax]);
        else
            if i==1, allymin=lymin; allymax=lymax;
            else allymin=min(lymin,allymin); allymax=max(lymax,allymax);
            end
            if i==numchans,
                axis([fmin fmax allymin allymax]);
            end
        end
        switch figInfo.subplot(i).grid,
            case 'xy', grid on;
            case 'x', set(gca,'XGrid','on'); set(gca,'YGrid','off');
            case 'y', set(gca,'YGrid','on'); set(gca,'XGrid','off');
            otherwise, grid off;
        end
        axhand(i) = gca;
        marker = figInfo.subplot(i).marker;
        if strcmp(marker,'.'), markersize = 8; else markersize = 6; end
        if ~overlay,
            set(h,'LineStyle',figInfo.subplot(i).linetype,'Marker', marker,'MarkerSize',markersize);
        else
            set(h,'LineStyle',lineStr,'Color',colorStr);
        end
        set(gca,'FontSize',fontsize);
        hs(i)=h;
        ylabels{i}=ystr;
    end
    if overlay, hold off, end
    
    % ADD LEGEND if overlay
    if overlay, legend(hs,ylabels); end

    % ADD TITLE
    figInfo.addtitle('Spectra')
    set(gcf,'Name','plot(ts_spectra)');
    
end

function [obj,figInfo]=setPlotProperties(obj,varargin)
    figInfo=obj.figureprops;      % Plot modifications are local
    for i=1:2:length(varargin),
        if isprop(figInfo,varargin{i})
            ts_debug(1,'setting figureprops property %s',varargin{i});
            eval(sprintf('figInfo.%s = varargin{i+1};',varargin{i}));
        elseif isprop(figInfo.subplot,varargin{i})
           ts_debug(1,'setting all subplot properties %s',varargin{i});
           for j=1:figInfo.nSubplots,
                eval(sprintf('figInfo.subplot(j).%s = varargin{i+1};',varargin{i}));
            end
        else
            if ischar(varargin{i})
            	error('property ''%s'' not found in figureprops or figureprops.subplot', varargin{i})
            else
                varargin{i}
                error('passed property #%d is not a string',i);
            end
        end
    end
end
