classdef ts_windowlist
    % List of windows (of time, of addresses...)
    %
    % These windows can be used to avoid bad data segments when calculating spectra
    % (or data cleaning transfer functions) using the 'baddata' field
    %
    % They can also be highlighted in time series plots using the 'highlight' field
    %
    % They can be hand-entered, or selected in tsplot
    %
    % Properties:
    %    starts    - READONLY: Vector of window starts
    %    ends      - READONLY: Vector of window ends 
    %    lengths   - READONLY: TODO: vector of window lengths (=ends-starts)
    %    window(i) - READONLY: the window [start end] at the specified index
    %    nWindows -  READONLY: TODO: the number of windows
    %
    % Methods:
    %     ts_windowlist - Creator
    %     add      - add a time window
    %     remove   - delete a time window
    %     window   - get a time window
    %     addbuffer  - Increases the length of ts_windowlist windows by a fixed amount
    %  Overloaded Matlab operators
    %     and      - '&': Returns only overlapping windows from 2 lists    
    %     or       - '|': Returns all windows from 2 lists
    %     not           : Returns windows NOT in the input list (must specify overall start and end values)  
    %     plus     - '+': Add this value to all windows
    %     minus    - '-': Subtract this value from all windows
    %     times    - '*': TODO: multiply all values by the given value
    %     divide   - '/': TODO: divide all values by the given value
    %
    % EXAMPLES:
    %  a=ts_windowlist('1:1000 3000:5000')
    %  a+500
    %  a/500
    %  ~a
    %  a.addbuffer(100)
    %  b=ts_windowlist('500:1500 4000:4100')
    %  a & b
    %  a | b

    % Used by ts_data.calcSpectra to allow the user to specify bad data
    %  sections not to use.  It's also supposed to be used by the time series
    %  plot functions, so that they can highlight excluded sections, but I
    %  never finished this part.
    %
    % This class may be badly written, has no overflow protection(?)", doesn't
    %   limit itself to integers
    %
    % Windows are stored internally in an Nx2 matrix of sequential [start end]s
    % Windows should never overlap: two overlapping windows should be merged
    % into a single window
    %

    properties
        matrix=[];
    end
    properties (Dependent=true)
        starts     % Array containing starting values (READONLY)
        ends       % Array containing ending values (READONLY)
        lengths    % Array containing lengths (READONLY)
        nWindows   % the number of windows
    end
    
    %% PUBLIC METHODS
    methods
        %% CREATOR
        function obj = ts_windowlist(inmatrix,thelength)
            % Creator for ts_windowlist: hold and manipulate a sequential list of offsets
            %
            % Usage:
            %	obj = ts_windowlist(matrix)    (enter Nx2 windows matrix or text)
            %	obj = ts_windowlist(offsets,length);
            %
            % Inputs:
            %   inmatrix: an Nx2 windows matrix
            %               OR
            %             a string written in Matlab address format: 'a:b c:d e:f'
            %               OR
            %             a vector of window start values, requiring a
            %               "lengths" value
            %	thelength: a scalar giving the length of each window
            %              ignored if 'matrix' is not a vector
            %
            % Examples:
            %   obj = ts_windowlist([0 100; 10000 10100; 100000 100100]);
            %   obj = ts_windowlist('0:100 10000:10100 100000:100100');
            %	obj = ts_windowlist([0 10000 100000],100);
            %
            
            % W Crawford 3/2007
            % updated 2/2018
            
            ts_init;	% Sets up global variables
            
            if ~exist('theinput','var'), return; end
            
            if ischar(inmatrix)
                obj= makefromstring(inmatrix);
            elseif isa(inmatrix,'numeric')
                obj=ts_windowlist();
                [M,N]=size(inmatrix);
                if N==2,
                    obj.matrix=inmatrix;
                    obj.clean_windows();  % Make sure everything is properly aligned
                elseif (M==1) || (N==1)
                    if ~exist('length','var'),
                        error('you specified offsets without a window length');
                    end
                    if length(thelength)>1, error('length must be a scalar'); end
                    obj.matrix=[inmatrix(:) inmatrix(:)+thelength];
                    obj.clean_windows()  % Make sure everything is properly aligned
                end
            else
                error('input must be a string or a vector,scalar pair');
            end
        end % function ts_windowlist
        
        %% GET DEPENDENT PROPERTIES
        function out=get.starts(obj)
            out=obj.matrix(:,1);
        end
        function out=get.ends(obj)
            out=obj.matrix(:,2);
        end
        function out=get.lengths(obj)
            out=obj.ends-obj.starts;
        end
        function out=get.nWindows(obj)
            out=length(obj.matrix(:,1));
        end
        %% DISPLAY AND CHAR METHODS
        function text=char(obj)
            text=[];
            for i=1:length(obj),
                text=[text sprintf(' %g:%g ',obj.privateStarts(i),obj.privateEnds(i))]; %#ok<AGROW>
            end
        end
        function disp(obj)
            disp(char(obj));
        end
        %% NORMAL METHODS
        function out=window(obj,index)
            if ~isint(index), error('index is not an integer'); end
            if index<1, error('index < 1 requested'); end
            if index>obj.nWindows, error('index %d > nWindows',index); end
            out=obj.matrix(index,:);
        end
        %% OVERLOADED OPERATORS
        function obj = and(sa,sb)
            % Combines 2 slists, selecting only shared offsets            
            if ischar(sb), sb=ts_windowlist(sb); end
            obj=ts_windowlist();            
            ia=1; ib=1; i=1;
            while (ia <= length(sa)) && (ib <= length(sb)),
                starta=sa.privateStarts(ia); enda=sa.privateEnds(ia);
                startb=sb.privateStarts(ib); endb=sb.privateEnds(ib);
                ts_debug(1,'------------------------')
                ts_debug(1,'%d: %g:%g\n',ia,starta,enda)
                ts_debug(1,'%d: %g:%g\n',ib,startb,endb)
                if startb < enda,	% Possible match
                    if endb >=starta,	% windows overlap
                        obj.privateStarts(i)=max(starta,startb);
                        if enda < endb,
                            obj.privateEnds(i) = enda;
                            ia=ia+1;
                        else
                            obj.privateEnds(i) = endb;
                            ib=ib+1;
                        end
                        i=i+1;
                    else	% sb(ib) ends before sa(ia): increment ib
                        ib=ib+1;
                    end
                else	% sb(ib) starts after sa(ia): increment ia
                    ia=ia+1;
                end
            end
        end
        function obj = mrdivide(obj,divisor)
            % Divide all offsets by a constant            
            for i=1:length(obj)
                obj.privateStarts(i) = obj.privateStarts(i)/divisor;
                obj.privateEnds(i) = obj.privateEnds(i)/divisor;
            end
        end        
        function obj = mtimes(obj,multiplier)
            % Multiply all offsets by a constant
            for i=1:length(obj)
                obj.privateStarts(i) = obj.privateStarts(i)*multiplier;
                obj.privateEnds(i) = obj.privateEnds(i)*multiplier;
            end
        end
        function obj = plus(obj,const)
            % Add a constant to all offsets
            for i=1:length(obj)
                obj.privateStarts(i) = obj.privateStarts(i)+const;
                obj.privateEnds(i) = obj.privateEnds(i)+const;
            end
        end
        function obj = or(sa,sb)
            % Combines 2 slists into one slist with all the offsets
            if ischar(sa), sa=ts_windowlist(sa); end
            if ischar(sb), sb=ts_windowlist(sb); end
            obj=ts_windowlist();
            ia=1; ib=1; i=0;
            while (ia <= length(sa)) && (ib <= length(sb)),
                starta=sa.privateStarts(ia); enda=sa.privateEnds(ia);
                startb=sb.privateStarts(ib); endb=sb.privateEnds(ib);
                ts_debug(1,'------------------------')
                ts_debug(1,'%d: %g:%g\n',ia,starta,enda)
                ts_debug(1,'%d: %g:%g\n',ib,startb,endb)
                if startb > enda,	% A ends before b starts, get A
                    thestart=starta;
                    theend=endb;
                    ia=ia+1;
                elseif starta > endb,
                    thestart=startb;
                    theend=endb;
                    ib=ib+1;
                else	% A and B overlap
                    thestart=min(starta,startb);
                    theend=max(enda,endb);
                    ia=ia+1; ib=ib+1;
                end
                
                % Now stuff window into new slist
                if i == 0,	% First time through
                    i=1;
                    obj.privateStarts(i)=thestart;
                    obj.privateEnds(i)=theend;
                else
                    if obj.privateEnds(i) > thestart,	% overlaps with existing window
                        obj.privateEnds(i)=theend;
                    else
                        i=i+1;
                        obj.privateStarts(i)=thestart;
                        obj.privateEnds(i)=theend;
                    end
                end
            end
            
            % once one input list is finished, finish the other
            if ia<=length(sa),
                for ii=ia:length(sa),
                    if obj.privateEnds(i) > sa.privateStarts(ii),
                        obj.privateEnds(i) = sa.privateEnds(ii);
                    else
                        i=i+1;
                        obj(i)=sa(ii);
                    end
                end
            elseif ib<=length(sb),
                for ii=ib:length(sb),
                    if obj.privateEnds(i) > sb.privateStarts(ii),
                        obj.privateEnds(i) = sb.privateEnds(ii);
                    else
                        i=i+1;
                        obj(i)=sb(ii);
                    end
                end
            end
        end
        function num = numel(obj)
            num=length(obj.privateStarts);
        end
        function num = length(obj)
            num=length(obj.privateStarts);
        end
        
        %% OTHER FUNCTIONS
        function s=addbuffer(s,buffer,option)
            % Increases the length of ts_windowlist windows by a fixed amount
            %
            %	option='end' only add to end of each window
            %	option='start' only subtract from start of each window
            
            %fprintf('Adding %d to %s of each window\n',buffer,option);
            addstart=1; addend=1;
            if exist('option','var'),
                switch option,
                    case 'end', addstart=0;
                    case 'start', addend=0;
                    otherwise, error('option must be ''start'' or ''end''');
                end
            end
            i=0;
            for iold=1:length(s),
                i=i+1;
                fprintf('iold=%d, i=%d\n',iold,i);
                s.privateStarts(i)=s.privateStarts(iold);
                s.privateEnds(i)  =s.privateEnds(iold);
                if addstart,
                    s.privateStarts(i)=s.privateStarts(i)-buffer;
                end
                if addend,
                    s.privateEnds(i)=s.privateEnds(i)+buffer;
                end
                if i>1,	% Check to see if we've overlapped with previous window
                    if s.privateStarts(i)<=s.privateEnds(i-1)
                        s.privateEnds(i-1)=s.privateEnds(i);
                        i=i-1;
                    end
                end
            end
            s.privateStarts=s.privateStarts(1:i);
            s.privateEnds=s.privateEnds(1:i);
        end % addbuffer
        function nobj=not(obj,maxoffset)
            % Inverts addresses in an slist
            %
            % Examples:
            %	nobj=~obj;
            %	nobj=not(obj,250000.5);
            %
            % All adresses previously listed are unlisted, all previously unlisted are
            % listed.  If you want to be able to have values beyond the end of the list
            % to be inverted, you must use the functional version and pass the maximum
            % offset to use
            
            nobj=ts_windowlist();
            
            inot=1;
            if obj.privateStarts(1) > 0,
                nobj.privateStarts(1)=0;
                nobj.privateEnds(1)=obj.privateStarts(1);
                inot=2;
            end
            for i=2:length(obj);
                nobj.privateStarts(inot)=obj.privateEnds(i-1);
                nobj.privateEnds(inot) = obj.privateStarts(i);
                inot=inot+1;
            end
            if exist('maxoffset','var'),
                if maxoffset>obj.privateEnds(end);
                    nobj.privateStarts(inot)=obj.privateEnds(end);
                    nobj.privateEnds(inot)=maxoffset;
                end
            end
        end        
   end % public methods
end % classdef


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function obj = makefromstring(thestring)
    % Makes an ts_windowlist from a string
    obj=ts_windowlist();
    i=1;
    
    R = thestring;
    while ~isempty(R),
        [T,R] = strtok(R);	% puts first whitespace-bounded string in T, rest in R
        if ~isempty(T),
            k = findstr(T,':');
            if isempty(k),		% If just a single value
                error('Must specify a range of offsets (even it there is only one bad point)');
            else
                A = sscanf(T,'%d:%d');
                if A(1) < lastValue, error('offsets are not sequential'); end
                obj.matrix(i,:) = [A(1) A(2)];
            end
            i=i+1;
        end
    end
end
