classdef ts_xferfun
    % PRIMARY TISKIT CLASS: Transfer functions
    %
    % 
    % ts_xferfun properties:
    %    freqs       - frequencies
    %    channel     - Data and metadata       (ts_xferfun_channel)
    %    figureprops - figure properties       (ts_figureproperties)
    %    nChannels   - number of channels      (READ-ONLY, DEPENDENT)
    %    gooddata    - returns channel(:).gooddata
    %                  or use to set all channel(:).gooddata) to same value
    %
    % ts_xferfun methods:
    %    ts_xferfun - Creator
    %    plus       - Combine two ts_xferfun objects
    %    plot       - Plot 
   
    %% PUBLIC PROPERTIES
    properties
        % First define the generic ts_xferfun object
        % Should be able to make vector of these
        freqs;                         % Frequencies
        channel=ts_xferfun_channel();  % Array of Data and metadata
        figureprops=ts_get('figureprops'); % Figure properties
                                           % Why in channel, too?????
    end
    %% DEPENDENT PROPERTIES
    properties (Dependent=true)
        nChannels;    % Number of channels of data
        gooddata;     % Use to set all channels' gooddata to the same value
                      % or access AND of all channels' gooddata
    end
    
    % PUBLIC METHODS
    methods
        % Constructor
        %function obj=ts_xferfun(varargin)
        function obj=ts_xferfun(a,b,c,d,e,f)
            % ts_xferfunc creator
            % Usage:
            %  tsf=ts_xferfunc()
            %  tsf=ts_xferfunc(xferfunc)
            %  tsf=ts_xferfunc(driveInfo,respInfo,xf,xferr,freqs,gooddata);
            %    where driveInfo and respInfo are cell arrays containing
            %    {channel#,channelname,channelunits} for the driving and
            %    response channels, respectively
            ts_init;	% Set up global variables
            if nargin==0; return;
            elseif nargin==1,

                if isa(a,'ts_xferfun'); obj=a; return; end
            elseif nargin==6,
                obj.channel(1).drivechan=a{1};
                obj.channel(1).respchan=b{1};
                obj.channel(1).data = c;
                obj.channel(1).uncertainty = d;
                obj.freqs = e;
                obj.channel(1).gooddata = f;
                obj.channel(1).name = [b{2} ' / '  a{2}];
                obj.channel(1).units = [b{3} ' / ' a{3}];
            else error('Ha!')
            end
        end
        
        function ch=char(obj)
            %CHAR Generate text description of ts_xferfun object
            
            ch = sprintf('%12s: %-13g\n','nChannels',obj.nChannels);
            ch = [ch sprintf('%12s: %-13g\n','numfreqs',length(obj.freqs))];
            for i=1:obj.nChannels,
                ch = [ch char(obj.channel(i))];
            end
        end
        
        function display(xfunc)
            %DISPLAY Command window display of ts_xferfun
            disp(char(xfunc));
        end
        
        %% ACCESS METHODS FOR DEPENDENT VARIABLES
        function out=get.nChannels(obj)
            out=length(obj.channel);
        end
        function out=get.gooddata(obj)
            out=obj.channel(:).gooddata;
        end
        function obj=set.gooddata(obj,val)
            for i=1:obj.nChannels,
                obj.channel(i).gooddata=val;
            end
        end
        
        function obj = plus(obj,obj2)
            % PLUS Combine two ts_xferfun objects
            % Usage: xfunc = xfa + xfb;
            %    Places xfb channels after the xfa channels.
            %    Allows  the transfer functions to be plotted together 
            %    and to remove correlated noise between channels
            %
             
            startxfs = length(obj.channel);
            addxfs = length(obj2.channel);
            if ~all(obj.freqs == obj2.freqs)
                error('Transfer function frequencies are not the same')
            end
            for j = 1:addxfs,
                obj.channel(j+startxfs) = obj2.channel(j);
            end
            ts_verbose(1,'Merged 2 ts_xferfun objects (%d- & %d- xfs) into one %d-xf object', ...
                startxfs,addxfs,length(obj.channel));
        end
    end
end