classdef ts_xferfun_channel
    % Class for individual Transfer Functions
    %  Created by ts_spectra.calcXF()
    % 
    % The transfer function is the LINEAR relation between two data channels.
    % It is calculated from the spectral levels, the coherency, and an assumption
    % of which channel contains most of the uncorrelated noise.
    % It can be used to remove a signal or noise recorded on one channel from another channel
    % The transfer function is defined as the Response channel divided by the driving channel
    % in the complex frequency domain.
    % The "gooddata" parameter is important for defining where the transfer function is valable
    % (there exists a linear relation between the two channels)
    %
    % ts_xferfun_channel properties:
    %  drivechan   - Driving Channel                           (integer)
    %  respchan    - Response Channel                          (integer)
    %  data        - Transfer function                         (numeric vector)
    %  uncertainty - Transfer function uncertainty             (numeric vector)
    %  gooddata    - 1 for good XF, 0 for not.  Used in        (numeric vector???)
    %                ts_spectra.clean and ts_data.blahblah...
    %  name        - Usually RespChanName / DriveChanName      (string)
    %  units       - RespChanUnits/DriveChanUnits              (string)
	%
    % ts_xferfun_channel methods:
    %    ts_xferfun_channel - Creator
   
    %% PUBLIC PROPERTIES
    properties
        drivechan    % Driving channel            (integer)
                     % SHOULD BE PROTECTED AFTER DECLARATION)
        respchan     % Response channel           (integer)
                     % SHOULD BE PROTECTED AFTER DECLARATION)
        data         % Data                       (numeric vector)
                     % SHOULD BE PROTECTED AFTER DECLARATION)
        uncertainty  % Data uncertainty           (numeric vector)
                     % SHOULD BE PROTECTED AFTER DECLARATION)
        gooddata     % 1 for good data, 0 for not (boolean vector)
        name         % Channel name               (character string)
                     % SHOULD BE PROTECTED AFTER DECLARATION)
        units        % Channel's units            (character string)
                     % SHOULD BE PROTECTED AFTER DECLARATION)
    end
    
    % PUBLIC METHODS
    methods
        % Constructor
        %function obj=ts_xferfun_channel(varargin)
        function obj=ts_xferfun(a)
            % ts_xferfunc creator
            % Usage:
            %  tsf=ts_xferfunc_channel()
            %  tsf=ts_xferfunc_channel(xferfunc_channel)
            ts_init;	% Set up global variables
            if nargin==0; return;
            elseif nargin==1,
                if isa(a,'ts_xferfun_channel'); obj=a; 
                    return; 
                else
                    error('Argument was not a ts_xferfun_channel object');
                end
            else error('Illegal number of arguments')
            end
        end
        
        function ch=char(obj)
            ch =     sprintf('      drivechan       respchan           name       units\n');
            ch = [ch sprintf(' %-14g %-14g %-14s %-14s', ...
                                obj.drivechan, ...
                                obj.respchan, ...
                                obj.name, ...
                                obj.units)];
        end
        
        function display(obj)
            disp(char(obj));
        end
        
    end
end