function lcpo2sac(filename,station,comps,starttime,endtime,synctime,driftrate,pos,destFormat,step,destDir,actions)
% LCPOTOSAC converts LCHEAPO data to SAC format
% Usage: lcpo2sac(fname,station,sync,drift,starttime,endtime,step,comps,destDir);
%
% Inputs: 
%	fname: LCHEAPO raw data filename
%	station: station name
%	sync: time at which LCPO was synced to GPS [string, or ts_datetime object]
%	drift: LCHEAPO driftrate (INSTelapsed-GPSElapsed)/GPSElapsed
%	pos: OBS position [latitude(decimal degrees), longitude(decimal degrees), depth(meters)]
%	starttime: first date to write (ts_datetime object, or '0' for start of
%			data)
%	endtime: date for which to stop writing (ts_datetime object, or '0' for end
%			of data)
%	comps:  the component name for each channel {cell array}
%           For example:
%               2-component short periods: {'SH P', 'SH Z'}
%               4-component short-periods: {'SH P', 'SH Z'}
%               broadbands:                {'BH P', 'BH Z'}
%   destFormat is the destination format ('SAC' or 'SEISAN')
%   step: size of individual files in seconds [DEFAULT=3600];
%	destDir: directory to write SAC files to.  Default = present directory
%	actions: allows you to process individual channels before saving.  You must make a cell array
%			with two columns: the first column will be the channel # in the
%			data, the second is a text string that will be processed on that
%			channel as if that channel were a ts_data object named 'mychannel'
%
%		For example: to highpass, 2-pole Butterworth filter the data from channels 1 and 3 at 4.5 Hz:
%			actions = {1, 'filter(mychannel,'BwHigh',2,4.5)'; ...
%					   3, 'filter(mychannel,'BwHigh',2,4.5)'};
%
% If you specify "0" for the starttime, will actually start at the first
% integral unit of "step" after the first data.  For example, if you set
% "step" to 3600 (1 hour) and the data starts at 2008/07/06 12:13:05, the
% first file written will be at 2008/07/06 13:00:00.
tic

%%% CHECK INPUT PARAMETERS
if ~exist('station'), error('You must specify a station'); end
if ~exist('filename'), error('You must specify a filename'); end
if ~exist('synctime'), error('You must specify synctime'); end
if ~exist('driftrate'), error('You must specify driftrate'); end
if ~exist('driftrate'), error('You must specify instrument postion'); end
if ~exist('starttime'), starttime=0; end
if ~exist('endtime'), endtime=0; end
if ~exist('step'), step=3600; end
if ~exist('comps'), comps={'P','HX','HY','Z'}; end
if ~exist('destDir'), destDir='.';
elseif isempty(destDir), destDir=','; end

dfile = ts_file(filename,'LCPO');
dfile = set(dfile,'synctime',synctime);
dfile = set(dfile,'driftrate',driftrate);
dfile = set(dfile, 'location',{pos(1) pos(2) -pos(3) pos(3)});

[starttime,endtime] = getStartEnd(dfile,starttime,endtime,step);

tt=starttime;
while tt < endtime-step,
    % READ THE DATA
	tdata = ap_readLCPO(dfile, tt,step);
	% MODIFY THE DATA IF REQUESTED
	for i = 1:length(actions),
		ich = actions{i,1};
		cmd = ['mychannel = ' actions{i,2} ';'];
		mychannel = extract(tdata,ich);
		eval(cmd);
		tdata.data(ich,:) = mychannel.data(1,:);
    end
    % CREATE/VERIFY THE DATE DIRECTORY STRUCTURE
	yeardir=fullfile(destDir,sprintf('%d',tt.year));
	monthdir=fullfile(yeardir,sprintf('%02d',tt.month));
	daydir=fullfile(monthdir,sprintf('%02d',tt.day));
	if ~exist(yeardir,'dir'), mkdir(yeardir); end
	if ~exist(monthdir,'dir'), mkdir(monthdir); end
	if ~exist(daydir,'dir'), mkdir(daydir); end
    
	fname=ts_write(td,destFormat);
    
	tt = tt+step;
end
te=toc;th=floor(te/3600);ts=te-3600*th; tm=floor(ts/60); ts=ts-tm*60;
fprintf('Station %s Took %.0f:%02d:%02d\n',station,th,tm,floor(ts));
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [s,e]=getStartEnd(dfile,s,e,step);
tdata = ap_readLCPO(dfile,0,1);	% LCPO file info is returned in custom parameter
A = get(tdata,'custom'); site_s=A{1}; site_e=A{2};

if isa(s,'ts_datetime'),
	if s < site_s,
		error('Your starttime (%s) is before the data starttime (%s)',s,site_s);
	elseif s > site_e,
		error('Your starttime (%s) is after the data endtime (%s)',s,site_e);
	end
else,
	if s==0,
		day_s=ts_datetime(site_s.year,site_s.jday,0,0,0);
		offset = site_s - day_s;
		nsteps=ceil(offset/step);
		s=day_s + nsteps*step;
	else,
		error('Unknown format for starttime');
	end
end

if isa(e,'ts_datetime'),
	if e<s,
		error('endtime (%s) is before starttime )%s)',e,s);
	end
else,
	if e==0,
		e=site_e;
	else,
		error('Illegal value for endtime');
	end
end
return;
		
