function convertLCPO(dfile,station,starttime,endtime,synctime,GPSEnd,instEnd,pos,destFormat,step,destDir,readchans,actions)
    % convertLCPO converts LCHEAPO data to a standard data format
    % Usage: convertLCPO(dfile,station,starttime,endtime,synctime,GPSEnd,instEnd,pos,destFormat,step,destDir,actions,outfname);
    %
    % Inputs:
    %	dfile: ts_file object with the filename and component information
    %	station: station name
    %	starttime: first date to write (ts_datetime object, or '0' for start of data)
    %	endtime: date for which to stop writing (ts_datetime object, or '0' for end of data)
    %	synctime: time at which LCPO was synced to GPS [string, or ts_datetime object]
    %	GPSend: GPS time at which end synchronization was obtained
    %   instEnd: time given by the instrument at the GPSend sync.
    %	pos: OBS position [latitude(decimal degrees), longitude(decimal degrees), depth(meters)]
    %   destFormat is the destination format ('SAC' or 'SEISAN')
    %   step: size of individual files in seconds [DEFAULT=3600];
    %	destDir: directory to write SAC files to.  Default = present directory
    %	outfname: output filename (empty means use default), otherwise use
    %             ts_datetime.char() format codes to specify how to code time
    %	actions: A text string that allows you to process the data before saving.
    %           The string will be processed on data stored in a ts_data object named 'd'
    %
    %		Examples:
    %			'd=d.filter(''BwHigh'',2,4.5)';                       % hipass filter data at 4.5 Hz:
    %			'd.channel(1)=d.channel(1).filter(''BwHigh'',2,4.5)'; % filter only channel 2
    %			'd=d.resample(''/25'')';                              % resample data at 25-times lower rate
    %
    % If you specify "0" for the starttime, will actually start at the first
    % integral unit of "step" after the first data.  For example, if you set
    % "step" to 3600 (1 hour) and the data starts at 2008/07/06 12:13:05, the
    % first file written will be at 2008/07/06 13:00:00.
    tStart=tic;
    if ts_get('verbose') > 0,
        ts_warning('verbose level is %d, you might want to set it to 0',ts_get('verbose'));
    end;
    if ts_get('debug') > 0,
        ts_warning('debug level is %d, you might want to set it to 0',ts_get('debug'));
    end;
    
    %%% CHECK INPUT PARAMETERS
    if ~exist('station','var'), error('You must specify a station'); end
    if ~exist('dfile','var'), error('You must specify a dfile'); end
    if ~exist('synctime','var'), error('You must specify synctime'); end
    if ~exist('GPSEnd','var'), error('You must specify GPSend'); end
    if ~exist('instEnd','var'), error('You must specify instEnd'); end
    if ~exist('pos','var'), error('You must specify instrument postion'); end
    if ~exist('starttime','var'), starttime=0; end
    if ~exist('endtime','var'), endtime=0; end
    if ~exist('destFormat','var'), destFormat='SEISAN';
    else
        switch upper(destFormat),
            case {'SEISAN','SAC','AH'};
                destFormat=upper(destFormat);
            otherwise,
                error('Unsupported destination format: %s',destFormat)
        end
    end
    if ~exist('step','var'), step=3600; end
    if ~exist('destDir','var'), destDir='.';
    elseif isempty(destDir), destDir='.'; end
    if ~exist('readchans','var'), readchans=[]; end
    if ~exist('actions','var'), actions=''; end
    
    % Create a diary file
    a=clock();
    dfname=sprintf('%04d%02d%02d-%02d%02d%02.0f_convertLCPO_%s.txt',a,station);
    fprintf('Creating diary file: %s\n',dfname)
    diary(dfname)
    
    % CALCULATE AND PRINT INSTRUMENT DRIFT RATE
    synctime=ts_datetime(synctime); GPSEnd=ts_datetime(GPSEnd); instEnd=ts_datetime(instEnd);
    if (GPSEnd-synctime) < 0,
        error('Your endtime (%s) is before your sync time (%s)!',char(GPSEnd),char(synctime));
    end
    driftrate=(instEnd-GPSEnd)/(GPSEnd-synctime);
    fprintf('%s driftrate = %.5e\n',station,driftrate);
    
    % PUT STATION, NETWORK AND CLOCK INFO INTO TS_FILE OBJECT
    for i=1:dfile.nChannels,
        dfile.ch_metadata(i).station=station;
        dfile.ch_metadata(i).coordinates=[pos(1) pos(2) -pos(3) pos(3)];
        dfile.ch_metadata(i).timezero=synctime;
        dfile.ch_metadata(i).clockdrift=driftrate; % Added 2011/12: prior runs did not integrate drift!!!
    end
    
    [starttime,endtime] = getStartEnd(dfile,starttime,endtime,step);
    
    
    tt=starttime;
    workdir=cd;
    while tt < endtime-step,
        % READ THE DATA
        d = read(dfile, tt,step);
        % MODIFY THE DATA IF REQUESTED
        if ~isempty(actions)
            eval([actions ';']);
        end
%         for i = 1:length(actions),
%             ich = actions{i,1};
%             cmd = ['mychannel = ' actions{i,2} ';'];
%             mychannel = d.channel(ich);
%             eval(cmd);
%             d.channel(ich)=mychannel;
%         end
        % CREATE/VERIFY THE DIRECTORY TREE
        yeardir=fullfile(destDir,sprintf('%d',tt.year));
        monthdir=fullfile(yeardir,sprintf('%02d',tt.month));
        if ~exist(yeardir,'dir'), mkdir(yeardir); end
        if ~exist(monthdir,'dir'), mkdir(monthdir); end
        
        cd(monthdir);
        
        eval(sprintf('writefn=@write%s;',destFormat));
        fname=writefn(d);
        
        ts_verbose(0,'Wrote %s',fname);
        cd(workdir);
        
        tt = tt+step;
    end
    te=toc(tStart);
    th=floor(te/3600);ts=te-3600*th; tm=floor(ts/60); ts=ts-tm*60;
    fprintf('Station %s Took %.0f:%02d:%02d\n',station,th,tm,floor(ts));
    
    %ts_set('verbose',tsv);
    diary off
    return;
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [s,e]=getStartEnd(dfile,s,e,step)
    
    [d,dfile] = read(dfile,0,1);	% LCPO file info is returned in custom parameter
    %A = dfile.custom, site_s=A{1}; site_e=A{2};
    site_s=dfile.firstTime; site_e=dfile.lastTime;
    
    if isa(s,'char'), s=ts_datetime(s); end
    if isa(e,'char'), e=ts_datetime(e); end
    
    if isa(s,'ts_datetime'),
        if s < site_s-1/d.channel(1).samprate,
            olds=s;
            saut=step*floor((site_s-s)/step);
            s=s+saut;
            while s < site_s-1/d.channel(1).samprate,
                s=s+step;
            end
            fprintf('Advanced starttime from %s to %s (data starttime=%s)\n', ...
                char(olds),char(s),char(site_s));
        end
        if s > site_e,
            error('Your starttime (%s) is after the data endtime (%s)',char(s),char(site_e));
        end
    else
        if s==0,
            day_s=ts_datetime(site_s.year,site_s.jday,0,0,0);
            offset = site_s - day_s;
            nsteps=ceil(offset/step);
            s=day_s + nsteps*step;
        else
            error('Unknown format for starttime');
        end
    end
    
    if isa(e,'ts_datetime'),
        if e<s,
            error('endtime (%s) is before starttime )%s)',e,s);
        elseif e>site_e
            warning('Requested endtime (%s) is beyond data end (%s), adjusting...', ...
                e,site_e);
            e=site_e;
        end
    else
        if e==0,
            e=site_e;
        else
            etest=s+e;
            if etest > site_e,
                warning('Requested endtime (%d seconds after %s) is beyond data end (%s), adjusting...', ...
                    etest,s,site_e);
                e=site_e;
            else
                e=etest;
            end
        end
    end
    return;
    
