function lcplotw(startTime,readChans,segLength)
    %LCPLOTW Plot L-Cheapo Ocean Bottom Seismometer data (uses TiSKit)
    % Usage: type lcplotw
    %        OR lcplotw(startTime,readChans,segLength,dataFiles);
    %           where:  ('[]' indicates default value)
    %             startTime (ts_datetime object) is the start time to suggest in
    %                   the dialog box [the latest start time for all files]
    %             readChans (vector) is a list of the channels to read by
    %                 default [all channels]
    %             segLength is the default read length (minutes) [15]
    %
    %             If any of the first three are unspecified or set to 0, the default values
    %             are used
    %
    %            If dataFiles isn't specified, the program will ask how many sites to read
    %            then open dialog boxes to choose each file
    %
    %  Afterwards, a dialog box opens allowing you to modify the start
    %  time, read length, clock drift and which channels to read
    
    % W Crawford 3/2005
    
    % Handle input arguments (default values always 0)
    if ~exist('startTime','var'), startTime=0; end
    if ~exist('readChans','var'), readChans=0; end
    if ~exist('segLength','var'), segLength=0; end
    
    %nSites = input('Enter number of sites to plot: ');
    
    % INPUT FILENAMES AND DISPLAY THEIR DATE RANGES
    %disp('Index      Filename   First Write           Last Write')
    [fileName, filePath]=uigetfile('*','Select ALL Files','MultiSelect','on');
    if isa(fileName,'cell'),
        nSites = length(fileName);
    else
        nSites=1; fileName = {fileName};
    end
    
    dfile=cell(nSites,1);
    for i=1:nSites,
        %[siteInfo(i).fileName, siteInfo(i).filePath]=uigetfile('*',sprintf('Select File %d',i));
        siteInfo(i).fileName = fileName{i};
        siteInfo(i).filePath = filePath;
        cd(siteInfo(i).filePath);
        dfile{i} = ts_file(siteInfo(i).fileName,'LCPO');
        dfile{i}.filename = [siteInfo(i).filePath filesep dfile{i}.filename];
        [tdata,df] = read(dfile{i},0,1);	% Just to get file info
        
        siteInfo(i).firstDir=df.firstTime;
        siteInfo(i).lastDir=df.lastTime;
        siteInfo(i).bits=df.custom{1};
        siteInfo(i).sampFreq=tdata.channel(1).samprate;
        dfile{i}.nChannels = tdata.nChannels;
        siteInfo(i).nChannels = tdata.nChannels;
        %	disp(sprintf('%4d   %12s   %s  %s',...
        %			i,get(dfile{i},'fileName'),char(siteFirstTime{i}),char(siteLastTime{i})));
    end
    
    [startTime,lengthSecs,outInfo] = chooseParameters(siteInfo,startTime,segLength,readChans);
    
    for i = 1:nSites,
        dfile{i}.readchans = outInfo(i).readChans;
        if dfile{i}.nChannels == 2.
            switch outInfo(i).instType
                case 1, dfile{i}=tsf_LCPO_L22(dfile{i});
                case 2, dfile{i}=tsf_LCPO_HydroOct2(dfile{i});
            end
        else  % 4-channel instruments
            switch outInfo(i).instType
                case 1, dfile{i}=tsf_LCPO_L28(dfile{i});
                case 2, dfile{i}=tsf_LCPO_T240(dfile{i});
                case 3, dfile{i}=tsf_LCPO_HydroOct4(dfile{i});
                case 4, dfile{i}=tsf_LCPO_4chGeneric(dfile{i});
            end
        end
        for j=1:dfile{i}.nChannels,
            dfile{i}.ch_metadata(j).information=sprintf('Instrument #%d',outInfo(i).instNum);
            dfile{i}.ch_metadata(j).station=sprintf('%d',outInfo(i).instNum);
        end
        
    end
    tsplot(dfile,startTime,lengthSecs);
end

function chNames=setCHNames(instType,siteNum)
    % SPECIFY THE  NAMES FOR EACH CHANNEL
    global GSITES
    if GSITES(siteNum).nChannels==2,
        if instType==1, chNames='ZH'; % SPOBS1 (L22)
        else            chNames='HH'; % HydrOct Test
        end
    else
        if instType==1,     chNames='H21Z'; % SPOBS2 (L28)
        elseif instType==2, chNames='21ZH'; % BBOBS1 (T240)
        elseif instType==3, chNames='HHHH'; % HydrOctopus
        else                chNames='????'; % Generic
        end
    end
end
%----------------------------------------------------------------------
% Open a dialog box to show the instrument times and to choose a plot time
% and length.  Modified from lcplot.m by Kerry Key
function [startTime,lengthSecs,info] = chooseParameters(site,startTime,segLength,readChans)
    %
    % Input
    %	site().fileName, 	   string
    %	site().firstDir,	ts_datetime object
    %	site().lastDir,	    ts_datetime object
    %	site().bits,
    %	site().sampFreq,
    %	site().nChannels,    scalar
    % Output
    %	info().readChans,    vector
    %	info().instNum,		 ts_datetime object
    %	info().clockDrift,	    ts_datetime object
    %   info().instType      2ch: 1=SP, 2=HydrOctTest
    %                        4ch: 1=SP, 2=BB, 3=HydrOct, 4=Generic
    %----------------------------------------------------------------------
    
    nSites = length(site);
    
    
    maxNumChans = max([site.nChannels]);
    % FILL UP GLOBALS FOR THE DIALOG BOX
    global SHOWCHANS INSTNUM YY MO DD HH MM SS PLTLEN LENUNITS INSTTYPES
    % set site as global for channel names in dialog box
    global GSITES
    GSITES=site;
    
    if readChans==0,
        SHOWCHANS = ones(nSites,maxNumChans);
    else
        SHOWCHANS = ones(nSites,1)*readChans(:)';
    end
    if segLength==0,
        PLTLEN=1;
        LENUNITS = 3; % 3=hours
    else
        PLTLEN=segLength;
        LENUNITS = 2; % 2=minutes
    end
    if ~isa(startTime,'ts_datetime'),	% Set startTime to the latest startTime of all sites
        startTime = site(1).firstDir;
        for i = 2:nSites, if site(i).firstDir - startTime > 0, startTime=site(i).firstDir; end, end
    end
    
    INSTNUM = 1:nSites;
    INSTTYPES = ones(1,nSites);
    for i=1:nSites,
        if site(i).nChannels==2
            INSTTYPES(i)=1;    % 1=SPOBS; 2=HydroOctTEST
        else
            INSTTYPES(i)=1; % 1=SPOBS; 2=BBOBS; 3=HydroOctopus; 4=Generic
        end
    end
    YY = startTime.year;
    MO = startTime.month;
    DD = startTime.day;
    HH = startTime.hour;
    MM = startTime.minute;
    SS = startTime.second;
    
    %%%% MAKE THE DIALOG BOX
    dy = 22;		% Height of info lines
    wirdWidth = 670+30*maxNumChans; % Width of dialog box
    %%% FLAG FOR COMPUTER TYPE (SPECIAL CASE IF A MAC)
    comp = computer;
    if strcmp(comp,'MAC2'); % it's a mac, adjust plot stuff
        FS = 11; % SMALLER FONT SIZE
        h0 = figure('Units','points', ...
            'Color',[0.8 0.8 0.8], ...
            'Position',[248.64 100.64 wirdWidth 167.28+(nSites-1)*21.32], ...
            'Tag','pltdialogfig');
    else  % PC, OR UNIX
        FS = 12;	% LARGER FONT SIZE
        h0 = figure('Units','points', ...
            'Color',[0.8 0.8 0.8], ...
            'Position',[248.64 100.64 wirdWidth 167.28+(nSites-1)*21.32], ...
            'Tag','pltdialogfig',...
            'ToolBar','none');  % TOOLBAR NONE OPTION SET
    end
    %%%%%%%%%%%% Lower Control Panel Stuff, always same postion:
    l1 = mkFrame([ 10   7   wirdWidth-20    58],       0.7,h0,'Framelower');
    
    % START TIME INPUT
    mkStatTxtB([ 50 40 100   20],'Start Time',16, 0.7,h0);
    mkETInt(   [ 10 12 50    20],'YY',        FS, 1  ,h0);
    mkStatTxt( [ 60 12 10    20],'/',         FS, 0.7,h0);
    mkETInt(   [ 70 12 30    20],'MO',        FS, 1  ,h0);
    mkStatTxt( [100 12 10    20],'/',         FS, 0.7,h0);
    mkETInt(   [110 12 30    20],'DD',        FS, 1  ,h0);
    mkETInt(   [150 12 30    20],'HH',  FS, 1  ,h0);
    mkStatTxt( [180 12 10    20],':',         FS, 0.7,h0);
    mkETInt(   [190 12 30    20],'MM',  FS, 1  ,h0);
    mkStatTxt( [220 12 10    20],':',         FS, 0.7,h0);
    mkETInt(   [230 12 30    20],'SS',  FS, 1  ,h0);
    % Plot Length INPUT
    mkStatTxtB([320 40  70    20],'Length', 16, 0.7,h0);
    mkETInt(   [300 12  50    20],'PLTLEN',FS, 1  ,h0);
    mkPopUp(   [360 12  90    26],'LENUNITS','Seconds|Minutes|Hours|Days',14,0.7,h0);
    % GO Button
    mkButton(  [wirdWidth-(70+30) 13.44  70  46.08],'PLOT','close(gcf);',18, 0.7,h0);
    
    %%%%%%%%%%%%% Upper stuff, has NSITES varying positions:
    %u1 = mkFrame(  [ 10 70    423    53+(NSITES-1)*21.32], 0.7, h0, 'FrameUpper');
    
    mkStatTxtB([wirdWidth/2-100 120+dy*(nSites-1) 200  40],'LCPLOTW',      30, 0.8,h0);
    y = 95+(nSites-1)*dy; fs=13;
    mkStatTxtB([  5 y    20  19],'#',         fs, 0.8,h0);
    mkStatTxtB([ 35 y    60  19],'File',      fs, 0.8,h0);
    mkStatTxtB([150 y    42  19],'Bits',      fs, 0.8,h0);
    mkStatTxtB([185 y    52  19],'Rate',      fs, 0.8,h0);
    mkStatTxtB([230 y    90  19],'First Dir', fs, 0.8,h0);
    mkStatTxtB([365 y    90  19],'Last Dir',  fs, 0.8,h0);
    mkStatTxtB([510 y+fs+3 90  19],'Channels',  fs, 0.8,h0);
    for j=1:maxNumChans,
        mkStatTxt([515+(j-1)*25 y 20  20],num2str(j),  fs, 0.8,h0);
    end
    % MAKE INSTTYPE HEADER TEXT
    mkStatTxtB([515+maxNumChans*25 y 80  20],'TYPE',  fs, 0.8,h0);
    
    
    %%% loop over files:
    for i=1:nSites,
        %%% DEFINE TEXT FOR FIRST AND LAST DIRECTORY TIMES:
        FIRST   = char(site(i).firstDir);
        %L       = FILES(i).DIR_COUNT;
        LAST    = char(site(i).lastDir);
        %%%
        y = 80+(nSites-i)*dy;
        mkETInt(   [  5 y  24 16],sprintf('INSTNUM(%d)',i),  11, 1  ,h0);
        mkStatTxtR([ 35 y  62 16],site(i).fileName,          11, 0.8,h0);
        mkStatTxtR([150 y  24 16],num2str(site(i).bits),     11, 0.8,h0);
        mkStatTxtR([185 y  40 16],num2str(site(i).sampFreq), 11, 0.8,h0);
        mkStatTxtR([230 y 130 16],FIRST,                     11, 0.8,h0);
        mkStatTxtR([365 y 130 16],LAST,                      11, 0.8,h0);
        % MAKE SHOWCHANS CHECKBOXES
        % The following was an attempt to put the channel names in
        % toggleboxes, but isn't complete until toggling the instrument
        % type renames the toggleboxes.
        for j=1:site(i).nChannels,
            mkChkBx([515+(j-1)*25 y 20 20], sprintf('SHOWCHANS(%d,%d)',i,j) ,h0);
            %mkToggleButton([445+(j-1)*25 y 20 20], sprintf('SHOWCHANS(%d,%d)',i,j),  chNames(j) ,h0);
        end
        % MAKE INSTTYPE PopUp
        if site(i).nChannels==2,
            mkPopUp(   [540+maxNumChans*25 y  90    26],sprintf('INSTTYPES(%d)',i),'SPOBS (ZH)|HydroOctTEST',14,0.7,h0);
        elseif site(i).nChannels==4,
            mkPopUp(   [515+maxNumChans*25 y-6  180    26],sprintf('INSTTYPES(%d)',i),'SPOBS (HXYZ)|BBOBS (XYZH)|HydroOct (HHHH)|Generic',11,0.7,h0);
        else
            error('Unsupported # of channels: %d',site(i).nChannels)
        end
    end     
            
    % WAIT FOR THE USER TO FINISH ENTERING VALUES AND RETURN the results
    waitfor(h0);
    
    % TRANSFER FROM GLOBALS TO RETURN VARIABLES
    %	info().readchans,    string
    %	info().instNum,		 ts_datetime object
    %	info().clockDrift,	    ts_datetime object
    %   info().instType      2ch: 1=SP, 2=HydrOctTest
    %                        4ch: 1=SP, 2=BB, 3=HydrOct, 4=Generic
    %----------------------------------------------------------------------
    startTime = ts_datetime(YY,MO,DD,HH,MM,SS);
    if LENUNITS==1,lengthSecs = PLTLEN;
    elseif LENUNITS==2, lengthSecs = PLTLEN*60;
    elseif LENUNITS==3, lengthSecs = PLTLEN*3600;
    elseif LENUNITS==4, lengthSecs = PLTLEN*86400;
    end
    
    %SHOWCHANS         % DEBUG
    % SET UP DEFAULT DIALOG VALUES
    for i = 1:nSites,
        info(i).instNum = INSTNUM(i);
        info(i).instType = INSTTYPES(i);
        info(i).readChans = [];
        for j=1:site(i).nChannels,
            if SHOWCHANS(i,j)==1,
                info(i).readChans=[info(i).readChans j];
            end
        end
        %	info(i).readChans    % DEBUG
        %SHOWCHANS = ones(nSites,max(inst.nChannels));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkStatTxt(pos,str,fSize,grayLvl,par)
    h = uicontrol('Parent',par, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl, ...
        'FontSize',fSize, ...
        'HorizontalAlignment','left', ...
        'ListboxTop',0, ...
        'Position',pos, ...
        'String',str, ...
        'Style','text', ...
        'Tag','StaticText2');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkStatTxtB(pos,str,fSize,grayLvl,par)
    h = uicontrol('Parent',par, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl, ...
        'FontSize',fSize, ...
        'FontWeight','bold', ...
        'HorizontalAlignment','left', ...
        'ListboxTop',0, ...
        'Position',pos, ...
        'String',str, ...
        'Style','text', ...
        'Tag','StaticText2');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkStatTxtR(pos,str,fSize,grayLvl,par)
    h = uicontrol('Parent',par, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl, ...
        'FontSize',fSize, ...
        'HorizontalAlignment','left', ...
        'ForegroundColor','red', ...
        'ListboxTop',0, ...
        'Position',pos, ...
        'String',str, ...
        'Style','text', ...
        'Tag','StaticText2');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%function h = mkEditTxt(str,callBk,pos,fSize,grayLvl,prnt,tag)
%h = uicontrol('Parent',prnt, ...
%	'Units','points', ...
%	'BackgroundColor',[1 1 1]*grayLvl, ...
%	'ListboxTop',0, ...
%	'Position',pos, ...
%    'FontSize',fSize, ...
%	'Callback', callBk,...
%    'Style','edit', ...
%	'Tag',tag);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkETInt(pos,vblStr,fSize,grayLvl,prnt)
    
    endVbl = strfind(vblStr,'(');
    if ~isempty(endVbl),
        vblBase = vblStr(1: (endVbl(1)-1));
    else vblBase = vblStr;
    end
    eval(sprintf('global %s',vblBase));
    %eval(vblStr);
    eval(['Str = sprintf(''%d'',' vblStr ');']);
    callBk = sprintf('global %s, %s = str2num(get(gcbo,''String''));',vblBase,vblStr);
    h = uicontrol('Parent',prnt, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl, ...
        'ListboxTop',0, ...
        'Position',pos, ...
        'FontSize',fSize, ...
        'Callback', callBk,...
        'String', Str,...
        'Style','edit', ...
        'Tag','ETint');
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function h = mkETExp(pos,vblStr,fSize,grayLvl,prnt)
%
%     endVbl = strfind(vblStr,'(');
%     if ~isempty(endVbl),
%         vblBase = vblStr(1: (endVbl(1)-1));
%     else vblBase = vblStr;
%     end
%     eval(sprintf('global %s',vblBase));
%
%     eval(['Str = sprintf(''%e'',' vblStr ');']);
%     callBk = sprintf('global %s, %s = str2num(get(gcbo,''Value''))',vblBase,vblStr);
%     h = uicontrol('Parent',prnt, ...
%         'Units','points', ...
%         'BackgroundColor',[1 1 1]*grayLvl, ...
%         'ListboxTop',0, ...
%         'Position',pos, ...
%         'FontSize',fSize, ...
%         'Callback', callBk,...
%         'String', Str,...
%         'Style','edit', ...
%         'Tag','ETint');
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkPopUp(pos,vblStr,menuStr,fSize,grayLvl,prnt)
    endVbl = strfind(vblStr,'(');
    if ~isempty(endVbl),
        vblBase = vblStr(1: (endVbl(1)-1));
    else vblBase = vblStr;
    end
    eval(sprintf('global %s',vblBase));
    %eval(sprintf('global %s',vblStr));
    eval(sprintf('val = %s;',vblStr));
    %sprintf('val = %s;',vblName);
    %vblName, val
    callBk = sprintf('global %s; %s = get(gcbo,''Value'');',vblBase,vblStr);
    %callBk = sprintf('global %s; %s = get(gcbo,''Value'');',vblStr,vblStr);
    h = uicontrol('Parent',prnt, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl, ...
        'FontSize',fSize, ...
        'FontWeight','bold', ...
        'ListboxTop',0, ...
        'Min',1, ...
        'Position',pos, ...
        'Callback',callBk,...
        'String',menuStr,...
        'Style','popupmenu', ...
        'Tag','myPopupMenu', ...
        'Value',val);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function h = mkBBChkBx(pos, vblStr, instNum,prnt)
%     
%     endVbl = strfind(vblStr,'(');
%     if ~isempty(endVbl),
%         vblBase = vblStr(1: (endVbl(1)-1));
%     else vblBase = vblStr;
%     end
%     eval(sprintf('global %s',vblBase));
%     eval(sprintf('val = %s;',vblStr));
%     text=setCHNames(eval(vblStr),instNum);
%     
%     callBk = sprintf('global %s, %s = get(gcbo,''Value'')==get(gcbo,''Max'');set(gcbo,''String'',setCHNames(%s,%d))',vblBase,vblStr,vblStr,instNum);
%     h = uicontrol('Parent',prnt, ...
%         'Units','points', ...
%         'Position',pos, ...
%         'Value',val, ...
%         'String',text,...
%         'Callback',callBk,...
%         'Style','checkbox', ...
%         'FontSize',11, ...
%         'Tag','myChkBox');
%     %'ListboxTop',0, ...
% end

function h = mkChkBx(pos, vblStr,prnt)
    
    endVbl = strfind(vblStr,'(');
    if ~isempty(endVbl),
        vblBase = vblStr(1: (endVbl(1)-1));
    else vblBase = vblStr;
    end
    eval(sprintf('global %s',vblBase));
    eval(sprintf('val = %s;',vblStr));
    callBk = sprintf('global %s, %s = get(gcbo,''Value'')==get(gcbo,''Max'');set(gcbo,''String'',''Hi'')',vblBase,vblStr);
    h = uicontrol('Parent',prnt, ...
        'Units','points', ...
        'Position',pos, ...
        'Value',val, ...
        'String','Hi',...
        'Callback',callBk,...
        'Style','checkbox', ...
        'FontSize',11, ...
        'Tag','myChkBox');
    %'ListboxTop',0, ...
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% function h = mkToggleButton(pos, vblStr, text, prnt)    % Like checkbox, but with text inside
%     
%     endVbl = strfind(vblStr,'(');
%     if ~isempty(endVbl),
%         vblBase = vblStr(1: (endVbl(1)-1));
%     else vblBase = vblStr;
%     end
%     eval(sprintf('global %s',vblBase));
%     eval(sprintf('val = %s;',vblStr));
%     callBk = sprintf('global %s, %s = get(gcbo,''Value'')==get(gcbo,''Max'');',vblBase,vblStr);
%     h = uicontrol('Parent',prnt, ...
%         'Units','points', ...
%         'Position',pos, ...
%         'Value',val, ...
%         'String',text,...
%         'Callback',callBk,...
%         'Style','togglebutton', ...
%         'Tag','myChkBox');
%     %	'ListboxTop',0, ...
%     %	'FontSize',pos(4), ...
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkButton(pos,str,callBk,fSize,grayLvl,prnt)
    h = uicontrol('Parent',prnt, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl, ...
        'FontSize',fSize, ...
        'FontWeight','bold', ...
        'ListboxTop',0, ...
        'Position',pos, ...
        'String',str, ...
        'Callback',callBk,...
        'Tag','myButton');
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function h = mkFrame(pos,grayLvl,par,tag)
    h = uicontrol('Parent',par, ...
        'Units','points', ...
        'BackgroundColor',[1 1 1]*grayLvl,  ...
        'ListboxTop',0, ...
        'Position',pos, ...
        'Style','frame', ...
        'Tag',tag);
end

