function utig2sac( sitename,filenames, startTime, endTime, sacLength, syncTime, clockdrift )
%UTIG2SAC Convert UTIG OBS data to another data file format
% Usage:
%   utig2sac(filenames,startTime,endTime,sacLength,syncTime,clockdrift)
% Inputs:
%   sitename  is a string that will be inserted into the SAC-file name
%   filenames is a string with wildcards matching all of the UTIG OBS filenames
%               (e.g., '*.10'), or a cell array of all of the filenames
%               (the routine will run much faster if they are listed in
%               chronological order)
%   startTime, endTime and syncTime may be strings of the form 'YYYY/MM/DD
%               HH:mm:SS'
%   sacLength is the length in seconds of each SAC file to output
%   clockdrift is (instElapsed-GPSElapsed)/GPSElapsed.  For example, an
%               instrument that is one second slow after a year will have
%               a clockdrift of -1/31536000 =  -3.17e-8

tic;
%% PUT DATA INTO APPROPRIATE VARIABLES
startTime=ts_datetime(startTime);   % Convert string to ap_datetime
endTime=  ts_datetime(endTime);
syncTime= ts_datetime(syncTime);

% If filenames is a cell array, force it to be row-oriented
if iscell(filenames), filenames=filenames{:}'; end

tfile=ts_file(filenames,'UTIG');
tfile=set(tfile,'syncTime',syncTime);
tfile=set(tfile,'clockdrift',clockdrift);

%% CHECK FOR PROBLEMS WITH REQUESTED TIMES
if endTime-startTime < sacLength,
    error('Requested end of data is less than one sacLength greater than requested start of data');
end

[td,fStart,fEnd]=read(tfile,-1);
if fStart > startTime,
    diffBlocks=ceil((fStart-startTime)/sacLength);
    reqStartTime=startTime;
    startTime = startTime+diffBlocks*sacLength;
    fprintf('You asked for starttime=%s\n',char(reqStartTime));
    fprintf('         Data starts at %s\n', char(fStart));
    fprintf('      Shifting start to %s\n', char(startTime));
end
if fEnd < endTime,
    reqEndTime=endTime;
    endTime=fEnd;
    fprintf('You asked for end time  %s\n', char(reqEndTime));
    fprintf('           Data ends at %s\n', char(fEnd)); 
    fprintf('   Shifting end time to %s\n', char(endTime));
end
if endTime-startTime < sacLength,
    error('End of data is less than one sacLength greater than start of data');
end

%% READ AND WRITE
readTime=startTime;

while readTime <= endTime-sacLength,
    tdata=read(tfile,readTime,sacLength);
    ts_write(tdata,format,['SAC/%y/%m/%d/%y%m%d_%H%M%S_' sitename]);
    readTime=readTime+sacLength;
end

fprintf('%.1f seconds',toc);
end     % End of function

